@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FOPException.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: FOPException.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.apps;
 
@@ -33,6 +33,8 @@ public class FOPException extends SAXException {
     private int line;
     private int column;
 
+    private String localizedMessage;
+
     /**
      * Constructs a new FOP exception with the specified detail message.
      * @param message the detail message.
@@ -78,7 +80,7 @@ public class FOPException extends SAXException {
     /**
      * Constructs a new exception with the specified detail message and cause.
      * @param message  the detail message
-     * @param cause the cause 
+     * @param cause the cause
      */
     public FOPException(String message, Exception cause) {
         super(message, cause);
@@ -101,7 +103,7 @@ public class FOPException extends SAXException {
      * @param systemId the system id of the FO document which is associated with the exception;
      *                 may be null.
      * @param line line number in the FO document which is associated with the exception.
-     * @param column clolumn number in the line which is associated with the exception.
+     * @param column column number in the line which is associated with the exception.
      */
     public void setLocation(String systemId, int line, int column) {
         this.systemId = systemId;
@@ -211,4 +213,23 @@ public class FOPException extends SAXException {
         }
     }
 
+    /**
+     * Sets the localized message for this exception.
+     * @param msg the localized message
+     */
+    public void setLocalizedMessage(String msg) {
+        this.localizedMessage = msg;
+    }
+
+    /** {@inheritDoc} */
+    public String getLocalizedMessage() {
+        if (this.localizedMessage != null) {
+            return this.localizedMessage;
+        } else {
+            return super.getLocalizedMessage();
+        }
+    }
+
+
+
 }
