@@ -15,18 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: FopFactory.java 637791 2008-03-17 08:59:51Z jeremias $ */
+/* $Id: FopFactory.java 1293736 2012-02-26 02:29:01Z gadams $ */
 
 package org.apache.fop.apps;
 
-import java.awt.color.ColorSpace;
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.net.MalformedURLException;
-import java.net.URL;
+import java.net.URI;
 import java.util.Collection;
 import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
 import java.util.Set;
 
 import javax.xml.transform.Source;
@@ -41,12 +42,15 @@ import org.apache.commons.logging.LogFactory;
 
 import org.apache.xmlgraphics.image.loader.ImageContext;
 import org.apache.xmlgraphics.image.loader.ImageManager;
+import org.apache.xmlgraphics.util.UnitConv;
 
 import org.apache.fop.fo.ElementMapping;
 import org.apache.fop.fo.ElementMappingRegistry;
 import org.apache.fop.fonts.FontCache;
+import org.apache.fop.fonts.FontManager;
 import org.apache.fop.hyphenation.HyphenationTreeResolver;
 import org.apache.fop.layoutmgr.LayoutManagerMaker;
+import org.apache.fop.render.ImageHandlerRegistry;
 import org.apache.fop.render.RendererFactory;
 import org.apache.fop.render.XMLHandlerRegistry;
 import org.apache.fop.util.ColorSpaceCache;
@@ -59,52 +63,65 @@ import org.apache.fop.util.ContentHandlerFactoryRegistry;
  * found and managed in the FOUserAgent.
  */
 public class FopFactory implements ImageContext {
-    
+
     /** logger instance */
     private static Log log = LogFactory.getLog(FopFactory.class);
-    
+
     /** Factory for Renderers and FOEventHandlers */
     private RendererFactory rendererFactory;
-    
+
     /** Registry for XML handlers */
     private XMLHandlerRegistry xmlHandlers;
-    
+
+    /** Registry for image handlers */
+    private ImageHandlerRegistry imageHandlers;
+
     /** The registry for ElementMapping instances */
     private ElementMappingRegistry elementMappingRegistry;
 
-    /** The registry for ContentHandlerFactory instance */ 
-    private ContentHandlerFactoryRegistry contentHandlerFactoryRegistry 
+    /** The registry for ContentHandlerFactory instance */
+    private ContentHandlerFactoryRegistry contentHandlerFactoryRegistry
                 = new ContentHandlerFactoryRegistry();
-    
+
     /** The resolver for user-supplied hyphenation patterns */
     private HyphenationTreeResolver hyphResolver = null;
 
     private ColorSpaceCache colorSpaceCache = null;
-    
+
     /** Image manager for loading and caching image objects */
     private ImageManager imageManager;
 
+    /** Font manager for font substitution, autodetection and caching **/
+    private FontManager fontManager;
+
     /** Configuration layer used to configure fop */
     private FopFactoryConfigurator config = null;
-        
+
     /**
      *  The base URL for all URL resolutions, especially for
      *  external-graphics.
      */
     private String base = null;
 
-    /** The base URL for all font URL resolutions. */
-    private String fontBase = null;
+    /**
+     *  Controls if accessibility is turned on or off
+     */
+    private boolean accessibility = false;
 
     /** The base URL for all hyphen URL resolutions. */
     private String hyphenBase = null;
 
+    /**
+     * Map of configured names of hyphenation pattern file names: ll_CC => name
+     */
+    private Map/*<String,String>*/ hyphPatNames = null;
+
     /**
      * FOP has the ability, for some FO's, to continue processing even if the
-     * input XSL violates that FO's content model.  This is the default  
+     * input XSL violates that FO's content model.  This is the default
      * behavior for FOP.  However, this flag, if set, provides the user the
      * ability for FOP to halt on all content model violations if desired.
-     */ 
+     */
     private boolean strictFOValidation = FopFactoryConfigurator.DEFAULT_STRICT_FO_VALIDATION;
 
     /**
@@ -114,12 +131,6 @@ public class FopFactory implements ImageContext {
     private boolean strictUserConfigValidation
         = FopFactoryConfigurator.DEFAULT_STRICT_USERCONFIG_VALIDATION;
 
-    /** Font cache to speed up auto-font configuration (null if disabled) */
-    private FontCache fontCache = null;
-
-    /** Allows enabling kerning on the base 14 fonts, default is false */
-    private boolean enableBase14Kerning = false;
-    
     /** Source resolution in dpi */
     private float sourceResolution = FopFactoryConfigurator.DEFAULT_SOURCE_RESOLUTION;
 
@@ -128,10 +139,14 @@ public class FopFactory implements ImageContext {
 
     /** Page height */
     private String pageHeight = FopFactoryConfigurator.DEFAULT_PAGE_HEIGHT;
-    
+
     /** Page width */
     private String pageWidth = FopFactoryConfigurator.DEFAULT_PAGE_WIDTH;
 
+    /** Complex scripts support enabled */
+    private boolean useComplexScriptFeatures
+        = FopFactoryConfigurator.DEFAULT_COMPLEX_SCRIPT_FEATURES;
+
     /** @see #setBreakIndentInheritanceOnReferenceAreaBoundary(boolean) */
     private boolean breakIndentInheritanceOnReferenceAreaBoundary
         = FopFactoryConfigurator.DEFAULT_BREAK_INDENT_INHERITANCE;
@@ -139,10 +154,10 @@ public class FopFactory implements ImageContext {
     /** Optional overriding LayoutManagerMaker */
     private LayoutManagerMaker lmMakerOverride = null;
 
-    private Set ignoredNamespaces;
+    private Set<String> ignoredNamespaces;
 
     private FOURIResolver foURIResolver;
-    
+
     /**
      * Main constructor.
      */
@@ -150,14 +165,23 @@ public class FopFactory implements ImageContext {
         this.config = new FopFactoryConfigurator(this);
         this.elementMappingRegistry = new ElementMappingRegistry(this);
         this.foURIResolver = new FOURIResolver(validateUserConfigStrictly());
+        this.fontManager = new FontManager() {
+
+            /** {@inheritDoc} */
+            @Override
+            public void setFontBaseURL(String fontBase) throws MalformedURLException {
+                super.setFontBaseURL(getFOURIResolver().checkBaseURL(fontBase));
+            }
+
+        };
         this.colorSpaceCache = new ColorSpaceCache(foURIResolver);
         this.imageManager = new ImageManager(this);
         this.rendererFactory = new RendererFactory();
         this.xmlHandlers = new XMLHandlerRegistry();
-        this.ignoredNamespaces = new java.util.HashSet();
-        setUseCache(FopFactoryConfigurator.DEFAULT_USE_CACHE);
+        this.imageHandlers = new ImageHandlerRegistry();
+        this.ignoredNamespaces = new java.util.HashSet<String>();
     }
-    
+
     /**
      * Returns a new FopFactory instance.
      * @return the requested FopFactory instance.
@@ -165,13 +189,13 @@ public class FopFactory implements ImageContext {
     public static FopFactory newInstance() {
         return new FopFactory();
     }
-    
+
     /**
      * Returns a new FOUserAgent instance. Use the FOUserAgent to configure special values that
      * are particular to a rendering run. Don't reuse instances over multiple rendering runs but
      * instead create a new one each time and reuse the FopFactory.
      * @return the newly created FOUserAgent instance initialized with default values
-     * @throws FOPException 
+     * @throws FOPException
      */
     public FOUserAgent newFOUserAgent() {
         FOUserAgent userAgent = new FOUserAgent(this);
@@ -179,12 +203,38 @@ public class FopFactory implements ImageContext {
     }
 
     /**
-     * Returns a new {@link Fop} instance. FOP will be configured with a default user agent 
+     * Sets accessibility support.
+     *
+     * @param value <code>true</code> to enable accessibility, <code>false</code> otherwise
+     */
+    void setAccessibility(boolean value) {
+        this.accessibility = value;
+    }
+
+    boolean isAccessibilityEnabled() {
+        return accessibility;
+    }
+
+    /**
+     * Sets complex script support.
+     * @param value <code>true</code> to enable complex script features,
+     * <code>false</code> otherwise
+     */
+    void setComplexScriptFeaturesEnabled(boolean value) {
+        this.useComplexScriptFeatures = value;
+    }
+
+    boolean isComplexScriptFeaturesEnabled() {
+        return useComplexScriptFeatures;
+    }
+
+    /**
+     * Returns a new {@link Fop} instance. FOP will be configured with a default user agent
      * instance.
      * <p>
      * MIME types are used to select the output format (ex. "application/pdf" for PDF). You can
      * use the constants defined in {@link MimeConstants}.
-     * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").     
+     * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").
      * @return the new Fop instance
      * @throws FOPException when the constructor fails
      */
@@ -193,29 +243,29 @@ public class FopFactory implements ImageContext {
     }
 
     /**
-     * Returns a new {@link Fop} instance. Use this factory method if you want to configure this 
+     * Returns a new {@link Fop} instance. Use this factory method if you want to configure this
      * very rendering run, i.e. if you want to set some metadata like the title and author of the
-     * document you want to render. In that case, create a new {@link FOUserAgent} 
+     * document you want to render. In that case, create a new {@link FOUserAgent}
      * instance using {@link #newFOUserAgent()}.
      * <p>
      * MIME types are used to select the output format (ex. "application/pdf" for PDF). You can
      * use the constants defined in {@link MimeConstants}.
      * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").
-     * @param userAgent the user agent that will be used to control the rendering run     
+     * @param userAgent the user agent that will be used to control the rendering run
      * @return the new Fop instance
      * @throws FOPException  when the constructor fails
      */
     public Fop newFop(String outputFormat, FOUserAgent userAgent) throws FOPException {
         return newFop(outputFormat, userAgent, null);
     }
-    
+
     /**
-     * Returns a new {@link Fop} instance. FOP will be configured with a default user agent 
+     * Returns a new {@link Fop} instance. FOP will be configured with a default user agent
      * instance. Use this factory method if your output type requires an output stream.
      * <p>
      * MIME types are used to select the output format (ex. "application/pdf" for PDF). You can
      * use the constants defined in {@link MimeConstants}.
-     * @param outputFormat the MIME type of the output format to use (ex. "application/pdf"). 
+     * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").
      * @param stream the output stream
      * @return the new Fop instance
      * @throws FOPException when the constructor fails
@@ -234,33 +284,35 @@ public class FopFactory implements ImageContext {
      * MIME types are used to select the output format (ex. "application/pdf" for PDF). You can
      * use the constants defined in {@link MimeConstants}.
      * @param outputFormat the MIME type of the output format to use (ex. "application/pdf").
-     * @param userAgent the user agent that will be used to control the rendering run     
+     * @param userAgent the user agent that will be used to control the rendering run
      * @param stream the output stream
      * @return the new Fop instance
      * @throws FOPException when the constructor fails
      */
-    public Fop newFop(String outputFormat, FOUserAgent userAgent, OutputStream stream) 
+    public Fop newFop(String outputFormat, FOUserAgent userAgent, OutputStream stream)
                 throws FOPException {
         if (userAgent == null) {
             throw new NullPointerException("The userAgent parameter must not be null!");
         }
         return new Fop(outputFormat, userAgent, stream);
     }
-    
+
     /**
      * Returns a new {@link Fop} instance. Use this factory method if you want to supply your
-     * own {@link org.apache.fop.render.Renderer Renderer} or 
-     * {@link org.apache.fop.fo.FOEventHandler FOEventHandler} 
+     * own {@link org.apache.fop.render.Renderer Renderer} or
+     * {@link org.apache.fop.fo.FOEventHandler FOEventHandler}
      * instance instead of the default ones created internally by FOP.
-     * @param userAgent the user agent that will be used to control the rendering run     
+     * @param userAgent the user agent that will be used to control the rendering run
      * @return the new Fop instance
      * @throws FOPException when the constructor fails
      */
     public Fop newFop(FOUserAgent userAgent) throws FOPException {
-        if (userAgent.getRendererOverride() == null 
-                && userAgent.getFOEventHandlerOverride() == null) {
-            throw new IllegalStateException("Either the overriding renderer or the overriding"
-                    + " FOEventHandler must be set when this factory method is used!");
+        if (userAgent.getRendererOverride() == null
+                && userAgent.getFOEventHandlerOverride() == null
+                && userAgent.getDocumentHandlerOverride() == null) {
+            throw new IllegalStateException("An overriding renderer,"
+                    + " FOEventHandler or IFDocumentHandler must be set on the user agent"
+                    + " when this factory method is used!");
         }
         return newFop(null, userAgent);
     }
@@ -274,7 +326,12 @@ public class FopFactory implements ImageContext {
     public XMLHandlerRegistry getXMLHandlerRegistry() {
         return this.xmlHandlers;
     }
-    
+
+    /** @return the image handler registry */
+    public ImageHandlerRegistry getImageHandlerRegistry() {
+        return this.imageHandlers;
+    }
+
     /** @return the element mapping registry */
     public ElementMappingRegistry getElementMappingRegistry() {
         return this.elementMappingRegistry;
@@ -318,41 +375,13 @@ public class FopFactory implements ImageContext {
         return this.lmMakerOverride;
     }
 
-    /**
-     * Checks if the given base URL is acceptable. It also normalizes the URL.
-     * @param base the base URL to check
-     * @return the normalized URL
-     * @throws MalformedURLException if there's a problem with a file URL
-     */
-    private String checkBaseURL(String base) throws MalformedURLException {
-        if (!base.endsWith("/")) {
-            // The behavior described by RFC 3986 regarding resolution of relative
-            // references may be misleading for normal users:
-            // file://path/to/resources + myResource.res -> file://path/to/myResource.res
-            // file://path/to/resources/ + myResource.res -> file://path/to/resources/myResource.res
-            // We assume that even when the ending slash is missing, users have the second
-            // example in mind
-            base += "/";
-        }
-        File dir = new File(base);
-        try {
-            base = (dir.isDirectory() ? dir.toURL() : new URL(base)).toExternalForm(); 
-        } catch (MalformedURLException mfue) {
-            if (strictUserConfigValidation) {
-                throw mfue;
-            }
-            log.error(mfue.getMessage());
-        }
-        return base;
-    }
-    
     /**
      * Sets the base URL.
-     * @param base base URL
+     * @param base the base URL
      * @throws MalformedURLException if there's a problem with a file URL
      */
     public void setBaseURL(String base) throws MalformedURLException {
-        this.base = checkBaseURL(base);
+        this.base = foURIResolver.checkBaseURL(base);
     }
 
     /**
@@ -362,19 +391,25 @@ public class FopFactory implements ImageContext {
     public String getBaseURL() {
         return this.base;
     }
-    
+
     /**
      * Sets the font base URL.
      * @param fontBase font base URL
      * @throws MalformedURLException if there's a problem with a file URL
+     * @deprecated use getFontManager().setFontBaseURL(fontBase) instead
      */
+    @Deprecated
     public void setFontBaseURL(String fontBase) throws MalformedURLException {
-        this.fontBase = checkBaseURL(fontBase);
+        getFontManager().setFontBaseURL(fontBase);
     }
 
-    /** @return the font base URL */
+    /**
+     * @return the font base URL
+     * @deprecated use getFontManager().setFontBaseURL(fontBase) instead
+     */
+    @Deprecated
     public String getFontBaseURL() {
-        return this.fontBase;
+        return getFontManager().getFontBaseURL();
     }
 
     /** @return the hyphen base URL */
@@ -396,12 +431,29 @@ public class FopFactory implements ImageContext {
                 }
             });
         }
-        this.hyphenBase = checkBaseURL(hyphenBase);
+        this.hyphenBase = foURIResolver.checkBaseURL(hyphenBase);
     }
-    
+
+    /**
+     * @return the hyphPatNames
+     */
+    public Map getHyphPatNames() {
+        return hyphPatNames;
+    }
+
+    /**
+     * @param hyphPatNames the hyphPatNames to set
+     */
+    public void setHyphPatNames(Map hyphPatNames) {
+        if (hyphPatNames == null) {
+            hyphPatNames = new HashMap();
+        }
+        this.hyphPatNames = hyphPatNames;
+    }
+
     /**
      * Sets the URI Resolver. It is used for resolving factory-level URIs like hyphenation
-     * patterns and as backup for URI resolution performed during a rendering run. 
+     * patterns and as backup for URI resolution performed during a rendering run.
      * @param uriResolver the new URI resolver
      */
     public void setURIResolver(URIResolver uriResolver) {
@@ -416,11 +468,19 @@ public class FopFactory implements ImageContext {
         return foURIResolver;
     }
 
+    /**
+     * Returns the FO URI Resolver.
+     * @return the FO URI Resolver
+     */
+    public FOURIResolver getFOURIResolver() {
+        return foURIResolver;
+    }
+
     /** @return the HyphenationTreeResolver for resolving user-supplied hyphenation patterns. */
     public HyphenationTreeResolver getHyphenationTreeResolver() {
         return this.hyphResolver;
     }
-    
+
     /**
      * Sets the HyphenationTreeResolver to be used for resolving user-supplied hyphenation files.
      * @param hyphResolver the HyphenationTreeResolver instance
@@ -447,7 +507,7 @@ public class FopFactory implements ImageContext {
     }
 
     /**
-     * @return true if the indent inheritance should be broken when crossing reference area 
+     * @return true if the indent inheritance should be broken when crossing reference area
      *         boundaries (for more info, see the javadoc for the relative member variable)
      */
     public boolean isBreakIndentInheritanceOnReferenceAreaBoundary() {
@@ -470,20 +530,26 @@ public class FopFactory implements ImageContext {
     public void setBreakIndentInheritanceOnReferenceAreaBoundary(boolean value) {
         this.breakIndentInheritanceOnReferenceAreaBoundary = value;
     }
-    
-    /** @return true if kerning on base 14 fonts is enabled */
+
+    /**
+     * @return true if kerning on base 14 fonts is enabled
+     * @deprecated use getFontManager().isBase14KerningEnabled() instead
+     */
+    @Deprecated
     public boolean isBase14KerningEnabled() {
-        return this.enableBase14Kerning;
+        return getFontManager().isBase14KerningEnabled();
     }
-    
+
     /**
      * Controls whether kerning is activated on base 14 fonts.
      * @param value true if kerning should be activated
+     * @deprecated use getFontManager().setBase14KerningEnabled(boolean) instead
      */
+    @Deprecated
     public void setBase14KerningEnabled(boolean value) {
-        this.enableBase14Kerning = value;
+        getFontManager().setBase14KerningEnabled(value);
     }
-    
+
     /** @return the resolution for resolution-dependant input */
     public float getSourceResolution() {
         return this.sourceResolution;
@@ -496,7 +562,7 @@ public class FopFactory implements ImageContext {
      * @see #getSourceResolution()
      */
     public float getSourcePixelUnitToMillimeter() {
-        return 25.4f / getSourceResolution(); 
+        return UnitConv.IN2MM / getSourceResolution();
     }
 
     /**
@@ -507,7 +573,7 @@ public class FopFactory implements ImageContext {
     public void setSourceResolution(float dpi) {
         this.sourceResolution = dpi;
         if (log.isDebugEnabled()) {
-            log.debug("source-resolution set to: " + sourceResolution 
+            log.debug("source-resolution set to: " + sourceResolution
                     + "dpi (px2mm=" + getSourcePixelUnitToMillimeter() + ")");
         }
     }
@@ -524,7 +590,7 @@ public class FopFactory implements ImageContext {
      * @see #getTargetResolution()
      */
     public float getTargetPixelUnitToMillimeter() {
-        return 25.4f / this.targetResolution; 
+        return UnitConv.IN2MM / this.targetResolution;
     }
 
     /**
@@ -544,21 +610,21 @@ public class FopFactory implements ImageContext {
     public void setSourceResolution(int dpi) {
         setSourceResolution((float)dpi);
     }
-    
+
     /**
      * Gets the default page-height to use as fallback,
      * in case page-height="auto"
-     * 
+     *
      * @return the page-height, as a String
      */
     public String getPageHeight() {
         return this.pageHeight;
     }
-    
+
     /**
      * Sets the page-height to use as fallback, in case
      * page-height="auto"
-     * 
+     *
      * @param pageHeight    page-height as a String
      */
     public void setPageHeight(String pageHeight) {
@@ -567,21 +633,21 @@ public class FopFactory implements ImageContext {
             log.debug("Default page-height set to: " + pageHeight);
         }
     }
-    
+
     /**
      * Gets the default page-width to use as fallback,
      * in case page-width="auto"
-     * 
+     *
      * @return the page-width, as a String
      */
     public String getPageWidth() {
         return this.pageWidth;
     }
-    
+
     /**
      * Sets the page-width to use as fallback, in case
      * page-width="auto"
-     * 
+     *
      * @param pageWidth    page-width as a String
      */
     public void setPageWidth(String pageWidth) {
@@ -590,27 +656,27 @@ public class FopFactory implements ImageContext {
             log.debug("Default page-width set to: " + pageWidth);
         }
     }
-    
+
     /**
      * Adds a namespace to the set of ignored namespaces.
-     * If FOP encounters a namespace which it cannot handle, it issues a warning except if this 
+     * If FOP encounters a namespace which it cannot handle, it issues a warning except if this
      * namespace is in the ignored set.
      * @param namespaceURI the namespace URI
      */
     public void ignoreNamespace(String namespaceURI) {
         this.ignoredNamespaces.add(namespaceURI);
     }
-    
+
     /**
      * Adds a collection of namespaces to the set of ignored namespaces.
-     * If FOP encounters a namespace which it cannot handle, it issues a warning except if this 
+     * If FOP encounters a namespace which it cannot handle, it issues a warning except if this
      * namespace is in the ignored set.
      * @param namespaceURIs the namespace URIs
      */
-    public void ignoreNamespaces(Collection namespaceURIs) {
+    public void ignoreNamespaces(Collection<String> namespaceURIs) {
         this.ignoredNamespaces.addAll(namespaceURIs);
     }
-    
+
     /**
      * Indicates whether a namespace URI is on the ignored list.
      * @param namespaceURI the namespace URI
@@ -619,14 +685,14 @@ public class FopFactory implements ImageContext {
     public boolean isNamespaceIgnored(String namespaceURI) {
         return this.ignoredNamespaces.contains(namespaceURI);
     }
-    
+
     /** @return the set of namespaces that are ignored by FOP */
-    public Set getIgnoredNamespace() {
+    public Set<String> getIgnoredNamespace() {
         return Collections.unmodifiableSet(this.ignoredNamespaces);
     }
 
     //------------------------------------------- Configuration stuff
-    
+
     /**
      * Set the user configuration.
      * @param userConfigFile the configuration file
@@ -646,16 +712,25 @@ public class FopFactory implements ImageContext {
     public void setUserConfig(String uri) throws SAXException, IOException {
         config.setUserConfig(uri);
     }
-    
+
     /**
      * Set the user configuration.
      * @param userConfig configuration
-     * @throws FOPException if a configuration problem occurs 
+     * @throws FOPException if a configuration problem occurs
      */
     public void setUserConfig(Configuration userConfig) throws FOPException {
         config.setUserConfig(userConfig);
     }
 
+    /**
+     * Set the base URI for the user configuration
+     * Useful for programmatic configurations
+     * @param baseURI the base URI
+     */
+    public void setUserConfigBaseURI(URI baseURI) {
+        config.setBaseURI(baseURI);
+    }
+
     /**
      * Get the user configuration.
      * @return the user configuration
@@ -681,39 +756,46 @@ public class FopFactory implements ImageContext {
         return this.strictUserConfigValidation;
     }
 
-    //------------------------------------------- Cache related stuff
+    //------------------------------------------- Font related stuff
 
     /**
      * Whether or not to cache results of font triplet detection/auto-config
      * @param useCache use cache or not
+     * @deprecated use getFontManager().setUseCache(boolean) instead
      */
+    @Deprecated
     public void setUseCache(boolean useCache) {
-        if (useCache) {
-            this.fontCache = FontCache.load();
-            if (this.fontCache == null) {
-                this.fontCache = new FontCache();
-            }
-        } else {
-            this.fontCache = null;
-        }
+        getFontManager().setUseCache(useCache);
     }
 
     /**
      * Cache results of font triplet detection/auto-config?
      * @return whether this factory is uses the cache
+     * @deprecated use getFontManager().useCache() instead
      */
+    @Deprecated
     public boolean useCache() {
-        return (this.fontCache != null);
+        return getFontManager().useCache();
     }
 
     /**
      * Returns the font cache instance used by this factory.
      * @return the font cache
+     * @deprecated use getFontManager().getFontCache() instead
      */
+    @Deprecated
     public FontCache getFontCache() {
-        return this.fontCache;
+        return getFontManager().getFontCache();
     }
-    
+
+    /**
+     * Returns the font manager.
+     * @return the font manager
+     */
+    public FontManager getFontManager() {
+        return this.fontManager;
+    }
+
     /**
      * Attempts to resolve the given URI.
      * Will use the configured resolver and if not successful fall back
@@ -721,7 +803,7 @@ public class FopFactory implements ImageContext {
      * @param href URI to access
      * @param baseUri the base URI to resolve against
      * @return A {@link javax.xml.transform.Source} object, or null if the URI
-     * cannot be resolved. 
+     * cannot be resolved.
      * @see org.apache.fop.apps.FOURIResolver
      */
     public Source resolveURI(String href, String baseUri) {
@@ -733,22 +815,15 @@ public class FopFactory implements ImageContext {
         }
         return source;
     }
-        
+
     /**
-     * Create (if needed) and return an ICC ColorSpace instance.
-     * 
-     * The ICC profile source is taken from the src attribute of the color-profile FO element.
-     * If the ICC ColorSpace is not yet in the cache a new one is created and stored in the cache.
-     * 
-     * The FOP URI resolver is used to try and locate the ICC file. 
-     * If that fails null is returned.
-     * 
-     * @param baseUri a base URI to resolve relative URIs
-     * @param iccProfileSrc ICC Profile source to return a ColorSpace for
-     * @return ICC ColorSpace object or null if ColorSpace could not be created 
+     * Returns the color space cache for this instance.
+     * <p>
+     * Note: this method should not be considered as part of FOP's external API.
+     * @return the color space cache
      */
-    public ColorSpace getColorSpace(String baseUri, String iccProfileSrc) {
-        return colorSpaceCache.get(baseUri, iccProfileSrc);
+    public ColorSpaceCache getColorSpaceCache() {
+        return this.colorSpaceCache;
     }
 
 }
