@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,14 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: AreaTreeHandler.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: AreaTreeHandler.java 1293736 2012-02-26 02:29:01Z gadams $ */
 
 package org.apache.fop.area;
 
 // Java
 import java.io.OutputStream;
-import java.util.Iterator;
 import java.util.List;
+import java.util.Locale;
 
 import org.xml.sax.SAXException;
 
@@ -49,7 +49,7 @@ import org.apache.fop.layoutmgr.TopLevelLayoutManager;
 
 /**
  * Area tree handler for formatting objects.
- * 
+ *
  * Concepts: The area tree is to be as small as possible. With minimal classes
  * and data to fully represent an area tree for formatting objects. The area
  * tree needs to be simple to render and follow the spec closely. This area tree
@@ -61,7 +61,7 @@ import org.apache.fop.layoutmgr.TopLevelLayoutManager;
  * type of renderer.
  */
 public class AreaTreeHandler extends FOEventHandler {
-    
+
     private static Log log = LogFactory.getLog(AreaTreeHandler.class);
 
     // Recorder of debug statistics
@@ -73,12 +73,15 @@ public class AreaTreeHandler extends FOEventHandler {
     /** The AreaTreeModel in use */
     protected AreaTreeModel model;
 
+    // Flag for controlling complex script features (default: true).
+    private boolean useComplexScriptFeatures = true;
+
     // Keeps track of all meaningful id references
     private IDTracker idTracker;
 
     // The fo:root node of the document
     private Root rootFObj;
-    
+
     // The formatting results to be handed back to the caller.
     private FormattingResults results = new FormattingResults();
 
@@ -88,7 +91,7 @@ public class AreaTreeHandler extends FOEventHandler {
 
     /**
      * Constructor.
-     * 
+     *
      * @param userAgent FOUserAgent object for process
      * @param outputFormat the MIME type of the output format to use (ex.
      * "application/pdf").
@@ -101,13 +104,15 @@ public class AreaTreeHandler extends FOEventHandler {
 
         setupModel(userAgent, outputFormat, stream);
 
-        lmMaker = userAgent.getFactory().getLayoutManagerMakerOverride();
+        this.lmMaker = userAgent.getFactory().getLayoutManagerMakerOverride();
         if (lmMaker == null) {
             lmMaker = new LayoutManagerMapping();
         }
 
-        idTracker = new IDTracker();
-        
+        this.idTracker = new IDTracker();
+
+        this.useComplexScriptFeatures = userAgent.isComplexScriptFeaturesEnabled();
+
         if (log.isDebugEnabled()) {
             statistics = new Statistics();
         }
@@ -115,7 +120,7 @@ public class AreaTreeHandler extends FOEventHandler {
 
     /**
      * Sets up the AreaTreeModel instance for use by the AreaTreeHandler.
-     * 
+     *
      * @param userAgent FOUserAgent object for process
      * @param outputFormat the MIME type of the output format to use (ex.
      * "application/pdf").
@@ -124,53 +129,67 @@ public class AreaTreeHandler extends FOEventHandler {
      */
     protected void setupModel(FOUserAgent userAgent, String outputFormat,
             OutputStream stream) throws FOPException {
-        model = new RenderPagesModel(userAgent, outputFormat, fontInfo, stream);
+        if (userAgent.isConserveMemoryPolicyEnabled()) {
+            this.model = new CachedRenderPagesModel(userAgent, outputFormat, fontInfo, stream);
+        } else {
+            this.model = new RenderPagesModel(userAgent, outputFormat, fontInfo, stream);
+        }
     }
 
     /**
      * Get the area tree model for this area tree.
-     * 
+     *
      * @return AreaTreeModel the model being used for this area tree
      */
     public AreaTreeModel getAreaTreeModel() {
-        return model;
+        return this.model;
     }
 
     /**
      * Get the LayoutManager maker for this area tree.
-     * 
+     *
      * @return LayoutManagerMaker the LayoutManager maker being used for this
      *         area tree
      */
     public LayoutManagerMaker getLayoutManagerMaker() {
-        return lmMaker;
+        return this.lmMaker;
     }
 
     /**
      * Get the IDTracker for this area tree.
-     * 
+     *
      * @return IDTracker used to track reference ids for items in this area tree
      */
     public IDTracker getIDTracker() {
-        return idTracker;
+        return this.idTracker;
     }
 
     /**
      * Get information about the rendered output, like number of pages created.
-     * 
+     *
      * @return the results structure
      */
     public FormattingResults getResults() {
         return this.results;
     }
 
+    /**
+     * Check whether complex script features are enabled.
+     *
+     * @return true if using complex script features
+     */
+    public boolean isComplexScriptFeaturesEnabled() {
+        return useComplexScriptFeatures;
+    }
+
     /**
      * Prepare AreaTreeHandler for document processing This is called from
      * FOTreeBuilder.startDocument()
-     * 
+     *
      * @throws SAXException
      *             if there is an error
      */
+    @Override
     public void startDocument() throws SAXException {
         // Initialize statistics
         if (statistics != null) {
@@ -178,6 +197,14 @@ public class AreaTreeHandler extends FOEventHandler {
         }
     }
 
+    @Override
+    public void startRoot(Root root) {
+        Locale locale = root.getLocale();
+        if (locale != null) {
+            model.setDocumentLocale(locale);
+        }
+    }
+
     /**
      * finish the previous pageSequence
      */
@@ -190,26 +217,31 @@ public class AreaTreeHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
+    @Override
     public void startPageSequence(PageSequence pageSequence) {
         startAbstractPageSequence(pageSequence);
     }
-    
+
     private void startAbstractPageSequence(AbstractPageSequence pageSequence) {
         rootFObj = pageSequence.getRoot();
+
+        //Before the first page-sequence...
+        if (this.prevPageSeqLM == null) {
+            // extension attachments from fo:root
+            wrapAndAddExtensionAttachments(rootFObj.getExtensionAttachments());
+            // extension attachments from fo:declarations
+            if (rootFObj.getDeclarations() != null) {
+                wrapAndAddExtensionAttachments(
+                        rootFObj.getDeclarations().getExtensionAttachments());
+            }
+        }
+
         finishPrevPageSequence(pageSequence.getInitialPageNumber());
         pageSequence.initPageNumber();
-        // extension attachments from fo:root
-        wrapAndAddExtensionAttachments(rootFObj.getExtensionAttachments());
-        // extension attachments from fo:declarations
-        if (rootFObj.getDeclarations() != null) {
-            wrapAndAddExtensionAttachments(rootFObj.getDeclarations().getExtensionAttachments());
-        }
     }
 
-    private void wrapAndAddExtensionAttachments(List list) {
-        Iterator i = list.iterator();
-        while (i.hasNext()) {
-            ExtensionAttachment attachment = (ExtensionAttachment) i.next();
+    private void wrapAndAddExtensionAttachments(List<ExtensionAttachment> list) {
+        for (ExtensionAttachment attachment : list) {
             addOffDocumentItem(new OffDocumentExtensionAttachment(attachment));
         }
     }
@@ -217,9 +249,10 @@ public class AreaTreeHandler extends FOEventHandler {
     /**
      * End the PageSequence. The PageSequence formats Pages and adds them to the
      * AreaTree. The area tree then handles what happens with the pages.
-     * 
+     *
      * @param pageSequence the page sequence ending
      */
+    @Override
     public void endPageSequence(PageSequence pageSequence) {
 
         if (statistics != null) {
@@ -239,29 +272,31 @@ public class AreaTreeHandler extends FOEventHandler {
     }
 
     /** {@inheritDoc} */
+    @Override
     public void startExternalDocument(ExternalDocument document) {
         startAbstractPageSequence(document);
     }
 
     /** {@inheritDoc} */
+    @Override
     public void endExternalDocument(ExternalDocument document) {
         if (statistics != null) {
             statistics.end();
         }
-        
+
         ExternalDocumentLayoutManager edLM;
         edLM = getLayoutManagerMaker().makeExternalDocumentLayoutManager(this, document);
         edLM.activateLayout();
         // preserve the current PageSequenceLayoutManger for the
         // force-page-count check at the beginning of the next PageSequence
         prevPageSeqLM = edLM;
-        
+
     }
 
     /**
      * Called by the PageSequenceLayoutManager when it is finished with a
      * page-sequence.
-     * 
+     *
      * @param pageSequence the page-sequence just finished
      * @param pageCount The number of pages generated for the page-sequence
      */
@@ -275,18 +310,19 @@ public class AreaTreeHandler extends FOEventHandler {
 
     /**
      * End the document.
-     * 
+     *
      * @throws SAXException if there is some error
      */
+    @Override
     public void endDocument() throws SAXException {
 
         finishPrevPageSequence(null);
         // process fox:destination elements
         if (rootFObj != null) {
-            List destinationList = rootFObj.getDestinationList();
+            List<Destination> destinationList = rootFObj.getDestinationList();
             if (destinationList != null) {
                 while (destinationList.size() > 0) {
-                    Destination destination = (Destination) destinationList.remove(0);
+                    Destination destination = destinationList.remove(0);
                     DestinationData destinationData = new DestinationData(destination);
                     addOffDocumentItem(destinationData);
                 }
@@ -301,6 +337,7 @@ public class AreaTreeHandler extends FOEventHandler {
                     model.handleOffDocumentItem(data);
                 }
             }
+            idTracker.signalIDProcessed(rootFObj.getId());
         }
         model.endDocument();
 
@@ -313,21 +350,22 @@ public class AreaTreeHandler extends FOEventHandler {
      * Add a OffDocumentItem to the area tree model. This checks if the
      * OffDocumentItem is resolvable and attempts to resolve or add the
      * resolvable ids for later resolution.
-     * 
+     *
      * @param odi the OffDocumentItem to add.
      */
     private void addOffDocumentItem(OffDocumentItem odi) {
         if (odi instanceof Resolvable) {
             Resolvable res = (Resolvable) odi;
             String[] ids = res.getIDRefs();
-            for (int count = 0; count < ids.length; count++) {
-                List pageVPList = idTracker.getPageViewportsContainingID(ids[count]);
-                if (pageVPList != null) {
-                    res.resolveIDRef(ids[count], pageVPList);
+            for (String id : ids) {
+                List<PageViewport> pageVPList = idTracker.getPageViewportsContainingID(id);
+                if (pageVPList != null && !pageVPList.isEmpty()) {
+                    res.resolveIDRef(id, pageVPList);
                 } else {
-                    log.warn(odi.getName() + ": Unresolved id reference \""
-                            + ids[count] + "\" found.");
-                    idTracker.addUnresolvedIDRef(ids[count], res);
+                    AreaEventProducer eventProducer = AreaEventProducer.Provider.get(
+                            getUserAgent().getEventBroadcaster());
+                    eventProducer.unresolvedIDReference(this, odi.getName(), id);
+                    idTracker.addUnresolvedIDRef(id, res);
                 }
             }
             // check to see if ODI is now fully resolved, if so process it
@@ -341,7 +379,7 @@ public class AreaTreeHandler extends FOEventHandler {
 
     /**
      * Generates and returns a unique key for a page viewport.
-     * 
+     *
      * @return the generated key.
      */
     public String generatePageViewportKey() {
@@ -353,11 +391,12 @@ public class AreaTreeHandler extends FOEventHandler {
      * Tie a PageViewport with an ID found on a child area of the PV. Note that
      * an area with a given ID may be on more than one PV, hence an ID may have
      * more than one PV associated with it.
-     * 
+     *
      * @param id the property ID of the area
      * @param pv a page viewport that contains the area with this ID
-     * @deprecated use getIdTracker().associateIDWithPageViewport(id, pv) instead
+     * @deprecated use getIDTracker().associateIDWithPageViewport(id, pv) instead
      */
+    @Deprecated
     public void associateIDWithPageViewport(String id, PageViewport pv) {
         idTracker.associateIDWithPageViewport(id, pv);
     }
@@ -366,10 +405,11 @@ public class AreaTreeHandler extends FOEventHandler {
      * This method tie an ID to the areaTreeHandler until this one is ready to
      * be processed. This is used in page-number-citation-last processing so we
      * know when an id can be resolved.
-     * 
+     *
      * @param id the id of the object being processed
-     * @deprecated use getIdTracker().signalPendingID(id) instead
+     * @deprecated use getIDTracker().signalPendingID(id) instead
      */
+    @Deprecated
     public void signalPendingID(String id) {
         idTracker.signalPendingID(id);
     }
@@ -378,57 +418,62 @@ public class AreaTreeHandler extends FOEventHandler {
      * Signals that all areas for the formatting object with the given ID have
      * been generated. This is used to determine when page-number-citation-last
      * ref-ids can be resolved.
-     * 
+     *
      * @param id the id of the formatting object which was just finished
-     * @deprecated use getIdTracker().signalIDProcessed(id) instead
+     * @deprecated use getIDTracker().signalIDProcessed(id) instead
      */
+    @Deprecated
     public void signalIDProcessed(String id) {
         idTracker.signalIDProcessed(id);
     }
 
     /**
      * Check if an ID has already been resolved
-     * 
+     *
      * @param id the id to check
      * @return true if the ID has been resolved
-     * @deprecated use getIdTracker().alreadyResolvedID(id) instead
+     * @deprecated use getIDTracker().alreadyResolvedID(id) instead
      */
+    @Deprecated
     public boolean alreadyResolvedID(String id) {
         return idTracker.alreadyResolvedID(id);
     }
 
     /**
      * Tries to resolve all unresolved ID references on the given page.
-     * 
+     *
      * @param pv page viewport whose ID refs to resolve
-     * @deprecated use getIdTracker().tryIDResolution(pv) instead
+     * @deprecated use getIDTracker().tryIDResolution(pv) instead
      */
+    @Deprecated
     public void tryIDResolution(PageViewport pv) {
         idTracker.tryIDResolution(pv);
     }
 
     /**
-     * Get the list of page viewports that have an area with a given id.
-     * 
+     * Get the set of page viewports that have an area with a given id.
+     *
      * @param id the id to lookup
      * @return the list of PageViewports
-     * @deprecated use getIdTracker().getPageViewportsContainingID(id) instead
+     * @deprecated use getIDTracker().getPageViewportsContainingID(id) instead
      */
-    public List getPageViewportsContainingID(String id) {
+    @Deprecated
+    public List<PageViewport> getPageViewportsContainingID(String id) {
         return idTracker.getPageViewportsContainingID(id);
     }
 
     /**
      * Add an Resolvable object with an unresolved idref
-     * 
+     *
      * @param idref the idref whose target id has not yet been located
      * @param res the Resolvable object needing the idref to be resolved
-     * @deprecated use getIdTracker().addUnresolvedIDRef(idref, res) instead
+     * @deprecated use getIDTracker().addUnresolvedIDRef(idref, res) instead
      */
+    @Deprecated
     public void addUnresolvedIDRef(String idref, Resolvable res) {
         idTracker.addUnresolvedIDRef(idref, res);
     }
-    
+
     private class Statistics {
         // for statistics gathering
         private Runtime runtime;
@@ -441,7 +486,6 @@ public class AreaTreeHandler extends FOEventHandler {
 
         /**
          * Default constructor
-         * @param areaTreeHandler area tree handler
          */
         protected Statistics() {
             this.runtime = Runtime.getRuntime();
