@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: DestinationData.java 609567 2008-01-07 10:52:09Z jeremias $ */
+/* $Id: DestinationData.java 1062901 2011-01-24 18:06:25Z adelmelle $ */
 
 package org.apache.fop.area;
 
@@ -56,7 +56,7 @@ public class DestinationData extends AbstractOffDocumentItem implements Resolvab
         this.idRef = idRef;
         this.idRefs = new String[] {idRef};
     }
-    
+
     /**
      * Get the idref for this destination
      *
@@ -66,9 +66,7 @@ public class DestinationData extends AbstractOffDocumentItem implements Resolvab
         return idRef;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public String[] getIDRefs() {
         return idRefs;
     }
@@ -97,12 +95,12 @@ public class DestinationData extends AbstractOffDocumentItem implements Resolvab
      * Resolves the idref of this object by getting the PageViewport
      * object that corresponds to the IDRef
      *
-     * {@inheritDoc} List)
-     * @todo check to make sure it works if multiple bookmark-items
+     * {@inheritDoc}
+     * TODO check to make sure it works if multiple bookmark-items
      * have the same idref
      */
-    public void resolveIDRef(String id, List pages) {
-        pageRef = (PageViewport) pages.get(0);
+    public void resolveIDRef(String id, List<PageViewport> pages) {
+        pageRef = pages.get(0);
         // TODO get rect area of id on page
     }
 
