@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PageViewport.java 798511 2009-07-28 12:53:58Z jeremias $ */
+/* $Id: PageViewport.java 1311120 2012-04-08 23:48:11Z gadams $ */
 
 package org.apache.fop.area;
 
@@ -25,7 +25,6 @@ import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
-import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -33,8 +32,16 @@ import java.util.Set;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
-import org.apache.fop.fo.Constants;
+import org.apache.fop.apps.FOPException;
+import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.pagination.SimplePageMaster;
+import org.apache.fop.traits.WritingModeTraitsGetter;
+
+import static org.apache.fop.fo.Constants.EN_FIC;
+import static org.apache.fop.fo.Constants.EN_FSWP;
+import static org.apache.fop.fo.Constants.EN_LEWP;
+import static org.apache.fop.fo.Constants.EN_LSWP;
+import static org.apache.fop.fo.Constants.FO_REGION_BODY;
 
 /**
  * Page viewport that specifies the viewport area and holds the page contents.
@@ -44,7 +51,7 @@ import org.apache.fop.fo.pagination.SimplePageMaster;
  * This is the level that creates the page.
  * The page (reference area) is then rendered inside the page object
  */
-public class PageViewport extends AreaTreeObject implements Resolvable, Cloneable {
+public class PageViewport extends AreaTreeObject implements Resolvable {
 
     private Page page;
     private Rectangle viewArea;
@@ -63,31 +70,29 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
 
     private transient PageSequence pageSequence;
 
-    // list of id references and the rectangle on the page
-    //private Map idReferences = null;
-
     // set of IDs that appear first (or exclusively) on this page:
-    private Set idFirsts = new java.util.HashSet();
+    private Set<String> idFirsts = new java.util.HashSet<String>();
 
     // this keeps a list of currently unresolved areas or extensions
     // once an idref is resolved it is removed
     // when this is empty the page can be rendered
-    private Map unresolvedIDRefs = new java.util.HashMap();
+    private Map<String, List<Resolvable>> unresolvedIDRefs
+            = new java.util.HashMap<String, List<Resolvable>>();
 
-    private Map pendingResolved = null;
+    private Map<String, List<PageViewport>> pendingResolved = null;
 
     // hashmap of markers for this page
     // start and end are added by the fo that contains the markers
-    private Map markerFirstStart = null;
-    private Map markerLastStart = null;
-    private Map markerFirstAny = null;
-    private Map markerLastEnd = null;
-    private Map markerLastAny = null;
+    private Map<String, Marker> markerFirstStart = null;
+    private Map<String, Marker> markerLastStart = null;
+    private Map<String, Marker> markerFirstAny = null;
+    private Map<String, Marker> markerLastEnd = null;
+    private Map<String, Marker> markerLastAny = null;
 
     /**
      * logging instance
      */
-    protected static Log log = LogFactory.getLog(PageViewport.class);
+    protected static final Log log = LogFactory.getLog(PageViewport.class);
 
     /**
      * Create a page viewport.
@@ -95,8 +100,10 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @param pageNumber the page number
      * @param pageStr String representation of the page number
      * @param blank true if this is a blank page
+     * @param spanAll true if the first span area spans all columns
      */
-    public PageViewport(SimplePageMaster spm, int pageNumber, String pageStr, boolean blank) {
+    public PageViewport(SimplePageMaster spm, int pageNumber, String pageStr,
+            boolean blank, boolean spanAll) {
         this.simplePageMasterName = spm.getMasterName();
         setExtensionAttachments(spm.getExtensionAttachments());
         setForeignAttributes(spm.getForeignAttributes());
@@ -107,14 +114,26 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         this.pageNumberString = pageStr;
         this.viewArea = new Rectangle(0, 0, pageWidth, pageHeight);
         this.page = new Page(spm);
-        createSpan(false);
+        createSpan(spanAll);
+    }
+
+    /**
+     * Create a page viewport.
+     * @param spm SimplePageMaster indicating the page and region dimensions
+     * @param pageNumber the page number
+     * @param pageStr String representation of the page number
+     * @param blank true if this is a blank page
+     */
+    public PageViewport(SimplePageMaster spm, int pageNumber, String pageStr, boolean blank) {
+        this(spm, pageNumber, pageStr, blank, false);
     }
 
     /**
      * Copy constructor.
      * @param original the original PageViewport to copy from
+     * @throws FOPException when cloning of the page is not supported
      */
-    public PageViewport(PageViewport original) {
+    public PageViewport(PageViewport original) throws FOPException {
         if (original.extensionAttachments != null) {
             setExtensionAttachments(original.extensionAttachments);
         }
@@ -124,7 +143,11 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         this.pageIndex = original.pageIndex;
         this.pageNumber = original.pageNumber;
         this.pageNumberString = original.pageNumberString;
-        this.page = (Page)original.page.clone();
+        try {
+            this.page = (Page) original.page.clone();
+        } catch (CloneNotSupportedException e) {
+            throw new FOPException(e);
+        }
         this.viewArea = new Rectangle(original.viewArea);
         this.simplePageMasterName = original.simplePageMasterName;
         this.blank = original.blank;
@@ -241,8 +264,8 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
 
     /**
      * Add an "ID-first" to this page.
-     * This is typically called by the AreaTreeHandler when associating
-     * an ID with a PageViewport.
+     * This is typically called by the {@link AreaTreeHandler} when associating
+     * an ID with a {@link PageViewport}.
      *
      * @param id the id to be registered as first appearing on this page
      */
@@ -264,9 +287,9 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
 
     /**
      * Add an idref to this page.
-     * All idrefs found for child areas of this PageViewport are added
-     * to unresolvedIDRefs, for subsequent resolution by AreaTreeHandler
-     * calls to this object's resolveIDRef().
+     * All idrefs found for child areas of this {@link PageViewport} are added
+     * to unresolvedIDRefs, for subsequent resolution by {@link AreaTreeHandler}
+     * calls to this object's {@code resolveIDRef()}.
      *
      * @param idref the idref
      * @param res the child element of this page that needs this
@@ -274,14 +297,14 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      */
     public void addUnresolvedIDRef(String idref, Resolvable res) {
         if (unresolvedIDRefs == null) {
-            unresolvedIDRefs = new HashMap();
+            unresolvedIDRefs = new HashMap<String, List<Resolvable>>();
         }
-        List list = (List)unresolvedIDRefs.get(idref);
-        if (list == null) {
-            list = new ArrayList();
-            unresolvedIDRefs.put(idref, list);
+        List<Resolvable> pageViewports = unresolvedIDRefs.get(idref);
+        if (pageViewports == null) {
+            pageViewports = new ArrayList<Resolvable>();
+            unresolvedIDRefs.put(idref, pageViewports);
         }
-        list.add(res);
+        pageViewports.add(res);
     }
 
     /**
@@ -299,24 +322,22 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      */
     public String[] getIDRefs() {
         return (unresolvedIDRefs == null) ? null
-            : (String[]) unresolvedIDRefs.keySet().toArray(new String[] {});
+            : unresolvedIDRefs.keySet().toArray(
+                new String[unresolvedIDRefs.keySet().size()]);
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public void resolveIDRef(String id, List pages) {
+    /** {@inheritDoc} */
+    public void resolveIDRef(String id, List<PageViewport> pages) {
         if (page == null) {
             if (pendingResolved == null) {
-                pendingResolved = new HashMap();
+                pendingResolved = new HashMap<String, List<PageViewport>>();
             }
             pendingResolved.put(id, pages);
         } else {
             if (unresolvedIDRefs != null) {
-                List todo = (List)unresolvedIDRefs.get(id);
+                List<Resolvable> todo = unresolvedIDRefs.get(id);
                 if (todo != null) {
-                    for (int count = 0; count < todo.size(); count++) {
-                        Resolvable res = (Resolvable)todo.get(count);
+                    for (Resolvable res : todo) {
                         res.resolveIDRef(id, pages);
                     }
                 }
@@ -350,7 +371,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @param isfirst if the area being added has is-first trait
      * @param islast if the area being added has is-last trait
      */
-    public void addMarkers(Map marks, boolean starting,
+    public void addMarkers(Map<String, Marker> marks, boolean starting,
             boolean isfirst, boolean islast) {
 
         if (marks == null) {
@@ -367,14 +388,13 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         if (starting) {
             if (isfirst) {
                 if (markerFirstStart == null) {
-                    markerFirstStart = new HashMap();
+                    markerFirstStart = new HashMap<String, Marker>();
                 }
                 if (markerFirstAny == null) {
-                    markerFirstAny = new HashMap();
+                    markerFirstAny = new HashMap<String, Marker>();
                 }
                 // first on page: only put in new values, leave current
-                for (Iterator iter = marks.keySet().iterator(); iter.hasNext();) {
-                    Object key = iter.next();
+                for (String key : marks.keySet()) {
                     if (!markerFirstStart.containsKey(key)) {
                         markerFirstStart.put(key, marks.get(key));
                         if (log.isTraceEnabled()) {
@@ -391,7 +411,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                     }
                 }
                 if (markerLastStart == null) {
-                    markerLastStart = new HashMap();
+                    markerLastStart = new HashMap<String, Marker>();
                 }
                 // last on page: replace all
                 markerLastStart.putAll(marks);
@@ -401,11 +421,10 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 }
             } else {
                 if (markerFirstAny == null) {
-                    markerFirstAny = new HashMap();
+                    markerFirstAny = new HashMap<String, Marker>();
                 }
                 // first on page: only put in new values, leave current
-                for (Iterator iter = marks.keySet().iterator(); iter.hasNext();) {
-                    Object key = iter.next();
+                for (String key : marks.keySet()) {
                     if (!markerFirstAny.containsKey(key)) {
                         markerFirstAny.put(key, marks.get(key));
                         if (log.isTraceEnabled()) {
@@ -419,7 +438,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
             // at the end of the area, register is-last and any areas
             if (islast) {
                 if (markerLastEnd == null) {
-                    markerLastEnd = new HashMap();
+                    markerLastEnd = new HashMap<String, Marker>();
                 }
                 // last on page: replace all
                 markerLastEnd.putAll(marks);
@@ -429,7 +448,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 }
             }
             if (markerLastAny == null) {
-                markerLastAny = new HashMap();
+                markerLastAny = new HashMap<String, Marker>();
             }
             // last on page: replace all
             markerLastAny.putAll(marks);
@@ -449,11 +468,11 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @param pos the position to retrieve
      * @return Object the marker found or null
      */
-    public Object getMarker(String name, int pos) {
-        Object mark = null;
+    public Marker getMarker(String name, int pos) {
+        Marker mark = null;
         String posName = null;
         switch (pos) {
-            case Constants.EN_FSWP:
+            case EN_FSWP:
                 if (markerFirstStart != null) {
                     mark = markerFirstStart.get(name);
                     posName = "FSWP";
@@ -463,13 +482,13 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                     posName = "FirstAny after " + posName;
                 }
             break;
-            case Constants.EN_FIC:
+            case EN_FIC:
                 if (markerFirstAny != null) {
                     mark = markerFirstAny.get(name);
                     posName = "FIC";
                 }
             break;
-            case Constants.EN_LSWP:
+            case EN_LSWP:
                 if (markerLastStart != null) {
                     mark = markerLastStart.get(name);
                     posName = "LSWP";
@@ -479,7 +498,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                     posName = "LastAny after " + posName;
                 }
             break;
-            case Constants.EN_LEWP:
+            case EN_LEWP:
                 if (markerLastEnd != null) {
                     mark = markerLastEnd.get(name);
                     posName = "LEWP";
@@ -490,7 +509,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
                 }
             break;
             default:
-                throw new RuntimeException();
+                assert false;
         }
         if (log.isTraceEnabled()) {
             log.trace("page " + pageNumberString + ": " + "Retrieving marker " + name
@@ -537,22 +556,19 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         page = (Page) in.readObject();
         unresolvedIDRefs = page.getUnresolvedReferences();
         if (unresolvedIDRefs != null && pendingResolved != null) {
-            for (Iterator iter = pendingResolved.keySet().iterator();
-                         iter.hasNext();) {
-                String id = (String) iter.next();
-                resolveIDRef(id, (List)pendingResolved.get(id));
+            for (String id : pendingResolved.keySet()) {
+                resolveIDRef(id, pendingResolved.get(id));
             }
             pendingResolved = null;
         }
     }
 
-    /**
-     * Clone this page.
-     * Used by the page master to create a copy of an original page.
-     * @return a copy of this page and associated viewports
-     */
-    public Object clone() {
-        return new PageViewport(this);
+    /** {@inheritDoc} */
+    public Object clone() throws CloneNotSupportedException {
+        PageViewport pvp = (PageViewport) super.clone();
+        pvp.page = (Page) page.clone();
+        pvp.viewArea = (Rectangle) viewArea.clone();
+        return pvp;
     }
 
     /**
@@ -564,9 +580,8 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         page = null;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
+    @Override
     public String toString() {
         StringBuffer sb = new StringBuffer(64);
         sb.append("PageViewport: page=");
@@ -589,8 +604,7 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
      * @return BodyRegion object
      */
     public BodyRegion getBodyRegion() {
-        return (BodyRegion) getPage().getRegionViewport(
-                Constants.FO_REGION_BODY).getRegionReference();
+        return (BodyRegion) getPage().getRegionViewport(FO_REGION_BODY).getRegionReference();
     }
 
     /**
@@ -647,4 +661,14 @@ public class PageViewport extends AreaTreeObject implements Resolvable, Cloneabl
         return getPage().getRegionViewport(id).getRegionReference();
     }
 
+    /**
+     * Sets the writing mode traits for the page associated with this viewport.
+     * @param wmtg a WM traits getter
+     */
+    public void setWritingModeTraits(WritingModeTraitsGetter wmtg) {
+        if ( page != null ) {
+            page.setWritingModeTraits(wmtg);
+        }
+    }
+
 }
