@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CommandLineOptions.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: CommandLineOptions.java 1341585 2012-05-22 18:26:15Z gadams $ */
 
 package org.apache.fop.cli;
 
@@ -41,6 +41,7 @@ import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.FopFactory;
 import org.apache.fop.apps.MimeConstants;
+import org.apache.fop.fonts.FontManager;
 import org.apache.fop.pdf.PDFAMode;
 import org.apache.fop.pdf.PDFEncryptionManager;
 import org.apache.fop.pdf.PDFEncryptionParams;
@@ -50,7 +51,7 @@ import org.apache.fop.render.awt.AWTRenderer;
 import org.apache.fop.render.intermediate.IFContext;
 import org.apache.fop.render.intermediate.IFDocumentHandler;
 import org.apache.fop.render.intermediate.IFSerializer;
-import org.apache.fop.render.pdf.PDFRenderer;
+import org.apache.fop.render.pdf.PDFConfigurationConstants;
 import org.apache.fop.render.print.PagesMode;
 import org.apache.fop.render.print.PrintRenderer;
 import org.apache.fop.render.xml.XMLRenderer;
@@ -109,7 +110,7 @@ public class CommandLineOptions {
     private boolean useStdIn = false;
     /* true if System.out (stdout) should be used for the output file */
     private boolean useStdOut = false;
-    /* true if a catalog resolver should be used for entity and uri resolution */ 
+    /* true if a catalog resolver should be used for entity and uri resolution */
     private boolean useCatalogResolver = false;
     /* rendering options (for the user agent) */
     private Map renderingOptions = new java.util.HashMap();
@@ -117,6 +118,8 @@ public class CommandLineOptions {
     private int targetResolution = 0;
     /* control memory-conservation policy */
     private boolean conserveMemoryPolicy = false;
+    /* true if a complex script features are enabled */
+    private boolean useComplexScriptFeatures = true;
 
     private FopFactory factory = FopFactory.newInstance();
     private FOUserAgent foUserAgent;
@@ -129,6 +132,8 @@ public class CommandLineOptions {
 
     private String mimicRenderer = null;
 
+    private boolean flushCache = false;
+
     /**
      * Construct a command line option object.
      */
@@ -166,7 +171,9 @@ public class CommandLineOptions {
                 }
                 checkSettings();
                 setUserConfig();
-
+                if (flushCache) {
+                    flushCache();
+                }
                 //Factory config is set up, now we can create the user agent
                 foUserAgent = factory.newFOUserAgent();
                 foUserAgent.getRendererOptions().putAll(renderingOptions);
@@ -176,6 +183,9 @@ public class CommandLineOptions {
                 addXSLTParameter("fop-output-format", getOutputFormat());
                 addXSLTParameter("fop-version", Version.getVersion());
                 foUserAgent.setConserveMemoryPolicy(conserveMemoryPolicy);
+                if (!useComplexScriptFeatures) {
+                    foUserAgent.setComplexScriptFeaturesEnabled(false);
+                }
             } else {
                 return false;
             }
@@ -198,17 +208,14 @@ public class CommandLineOptions {
                 System.err.println("Couldn't set system look & feel!");
             }
 
-            AWTRenderer renderer = new AWTRenderer(true);
-            renderer.setRenderable(inputHandler); //set before user agent!
-            renderer.setUserAgent(foUserAgent);
+            AWTRenderer renderer = new AWTRenderer(foUserAgent, inputHandler, true, true);
             foUserAgent.setRendererOverride(renderer);
         } else if (MimeConstants.MIME_FOP_AREA_TREE.equals(outputmode)
                && mimicRenderer != null) {
             // render from FO to Intermediate Format
             Renderer targetRenderer = foUserAgent.getRendererFactory().createRenderer(
                    foUserAgent, mimicRenderer);
-            XMLRenderer xmlRenderer = new XMLRenderer();
-            xmlRenderer.setUserAgent(foUserAgent);
+            XMLRenderer xmlRenderer = new XMLRenderer(foUserAgent);
 
             //Tell the XMLRenderer to mimic the target renderer
             xmlRenderer.mimicRenderer(targetRenderer);
@@ -284,6 +291,10 @@ public class CommandLineOptions {
                 factory.setStrictValidation(false);
             } else if (args[i].equals("-conserve")) {
                 conserveMemoryPolicy = true;
+            } else if (args[i].equals("-flush")) {
+                flushCache = true;
+            } else if (args[i].equals("-cache")) {
+                parseCacheOption(args, i);
             } else if (args[i].equals("-dpi")) {
                 i = i + parseResolution(args, i);
             } else if (args[i].equals("-q") || args[i].equals("--quiet")) {
@@ -346,8 +357,11 @@ public class CommandLineOptions {
             } else if (args[i].equals("-a")) {
                 this.renderingOptions.put(Accessibility.ACCESSIBILITY, Boolean.TRUE);
             } else if (args[i].equals("-v")) {
-                /* Currently just print the version */
+                /* verbose mode although users may expect version; currently just print the version */
                 printVersion();
+                if (args.length == 1) {
+                    return false;
+                }
             } else if (args[i].equals("-param")) {
                   if (i + 2 < args.length) {
                       String name = args[++i];
@@ -372,6 +386,16 @@ public class CommandLineOptions {
                 getPDFEncryptionParams().setAllowEditContent(false);
             } else if (args[i].equals("-noannotations")) {
                 getPDFEncryptionParams().setAllowEditAnnotations(false);
+            } else if (args[i].equals("-nocs")) {
+                useComplexScriptFeatures = false;
+            } else if (args[i].equals("-nofillinforms")) {
+                getPDFEncryptionParams().setAllowFillInForms(false);
+            } else if (args[i].equals("-noaccesscontent")) {
+                getPDFEncryptionParams().setAllowAccessContent(false);
+            } else if (args[i].equals("-noassembledoc")) {
+                getPDFEncryptionParams().setAllowAssembleDocument(false);
+            } else if (args[i].equals("-noprinthq")) {
+                getPDFEncryptionParams().setAllowPrintHq(false);
             } else if (args[i].equals("-version")) {
                 printVersion();
                 return false;
@@ -385,6 +409,17 @@ public class CommandLineOptions {
         return true;
     }    // end parseOptions
 
+    private int parseCacheOption(String[] args, int i) throws FOPException {
+        if ((i + 1 == args.length)
+                || (isOption(args[i + 1]))) {
+            throw new FOPException("if you use '-cache', you must specify "
+              + "the name of the font cache file");
+        } else {
+            factory.getFontManager().setCacheFile(new File(args[i + 1]));
+            return 1;
+        }
+    }
+
     private int parseConfigurationOption(String[] args, int i) throws FOPException {
         if ((i + 1 == args.length)
                 || (isOption(args[i + 1]))) {
@@ -556,7 +591,7 @@ public class CommandLineOptions {
 
     private int parsePrintOutputOption(String[] args, int i) throws FOPException {
         setOutputMode(MimeConstants.MIME_FOP_PRINT);
-        if ((i + 1 <= args.length)
+        if ((i + 1 < args.length)
                 && (args[i + 1].charAt(0) != '-')) {
             String arg = args[i + 1];
             String[] parts = arg.split(",");
@@ -785,14 +820,14 @@ public class CommandLineOptions {
 
     private PDFEncryptionParams getPDFEncryptionParams() throws FOPException {
         PDFEncryptionParams params = (PDFEncryptionParams)renderingOptions.get(
-                        PDFRenderer.ENCRYPTION_PARAMS);
+                        PDFConfigurationConstants.ENCRYPTION_PARAMS);
         if (params == null) {
             if (!PDFEncryptionManager.checkAvailableAlgorithms()) {
                 throw new FOPException("PDF encryption requested but it is not available."
                         + " Please make sure MD5 and RC4 algorithms are available.");
             }
             params = new PDFEncryptionParams();
-            renderingOptions.put(PDFRenderer.ENCRYPTION_PARAMS, params);
+            renderingOptions.put(PDFConfigurationConstants.ENCRYPTION_PARAMS, params);
         }
         return params;
     }
@@ -961,10 +996,12 @@ public class CommandLineOptions {
                         "FO output mode is only available if you use -xml and -xsl");
             } else if (outputmode.equals(MimeConstants.MIME_FOP_AREA_TREE)) {
                 throw new FOPException(
-                    "Area Tree Output is not available if Intermediate Format is used as input!");
+                    "Area Tree Output is not available if Intermediate Format"
+                    + " is used as input!");
             } else if (outputmode.equals(MimeConstants.MIME_FOP_IF)) {
                 throw new FOPException(
-                    "Intermediate Output is not available if Intermediate Format is used as input!");
+                    "Intermediate Output is not available if Intermediate Format"
+                    + " is used as input!");
             }
             if (iffile != null && !iffile.exists()) {
                 throw new FileNotFoundException("Error: intermediate format file "
@@ -1148,21 +1185,38 @@ public class CommandLineOptions {
             + "  -q                quiet mode  \n"
             + "  -c cfg.xml        use additional configuration file cfg.xml\n"
             + "  -l lang           the language to use for user information \n"
+            + "  -nocs             disable complex script features\n"
             + "  -r                relaxed/less strict validation (where available)\n"
             + "  -dpi xxx          target resolution in dots per inch (dpi) where xxx is a number\n"
             + "  -s                for area tree XML, down to block areas only\n"
-            + "  -v                run in verbose mode (currently simply print FOP version and continue)\n\n"
+            + "  -v                run in verbose mode (currently simply print FOP version"
+            + " and continue)\n\n"
             + "  -o [password]     PDF file will be encrypted with option owner password\n"
             + "  -u [password]     PDF file will be encrypted with option user password\n"
             + "  -noprint          PDF file will be encrypted without printing permission\n"
             + "  -nocopy           PDF file will be encrypted without copy content permission\n"
             + "  -noedit           PDF file will be encrypted without edit content permission\n"
             + "  -noannotations    PDF file will be encrypted without edit annotation permission\n"
+            + "  -nofillinforms    PDF file will be encrypted without"
+            + " fill in interactive form fields permission\n"
+            + "  -noaccesscontent  PDF file will be encrypted without"
+            + " extract text and graphics permission\n"
+            + "  -noassembledoc    PDF file will be encrypted without"
+            + " assemble the document permission\n"
+            + "  -noprinthq        PDF file will be encrypted without"
+            + " print high quality permission\n"
             + "  -a                enables accessibility features (Tagged PDF etc., default off)\n"
             + "  -pdfprofile prof  PDF file will be generated with the specified profile\n"
             + "                    (Examples for prof: PDF/A-1b or PDF/X-3:2003)\n\n"
-            + "  -conserve         Enable memory-conservation policy (trades memory-consumption for disk I/O)\n"
-            + "                    (Note: currently only influences whether the area tree is serialized.)\n\n"
+            + "  -conserve         enable memory-conservation policy (trades memory-consumption"
+            + " for disk I/O)\n"
+            + "                    (Note: currently only influences whether the area tree is"
+            + " serialized.)\n\n"
+
+            + "  -cache            specifies a file/directory path location"
+            + " for the font cache file\n"
+            + "  -flush            flushes the current font cache file\n\n"
+
             + " [INPUT]  \n"
             + "  infile            xsl:fo input file (the same as the next) \n"
             + "                    (use '-' for infile to pipe input from stdin)\n"
@@ -1192,14 +1246,16 @@ public class CommandLineOptions {
             + "  -at [mime] out    representation of area tree as XML (outfile req'd) \n"
             + "                    specify optional mime output to allow the AT to be converted\n"
             + "                    to final format later\n"
-            + "  -if [mime] out    representation of document in intermediate format XML (outfile req'd)\n"
+            + "  -if [mime] out    representation of document in intermediate format XML"
+            + " (outfile req'd)\n"
             + "                    specify optional mime output to allow the IF to be converted\n"
             + "                    to final format later\n"
             + "  -print            input file will be rendered and sent to the printer \n"
             + "                    see options with \"-print help\" \n"
             + "  -out mime outfile input will be rendered using the given MIME type\n"
             + "                    (outfile req'd) Example: \"-out application/pdf D:\\out.pdf\"\n"
-            + "                    (Tip: \"-out list\" prints the list of supported MIME types)\n"
+            + "                    (Tip: \"-out list\" prints the list of supported MIME types"
+            + " and exits)\n"
             //+ "  -mif outfile      input will be rendered as MIF (FrameMaker) (outfile req'd)\n"
             //+ "                    Experimental feature - requires additional fop-sandbox.jar.\n"
             + "  -svg outfile      input will be rendered as an SVG slides file (outfile req'd) \n"
@@ -1333,5 +1389,14 @@ public class CommandLineOptions {
         }
     }
 
+    private void flushCache() throws FOPException {
+        FontManager fontManager = factory.getFontManager();
+        File cacheFile = fontManager.getCacheFile();
+        if (!fontManager.deleteCache()) {
+            System.err.println("Failed to flush the font cache file '"
+                    + cacheFile + "'.");
+            System.exit(1);
+        }
+    }
 }
 
