@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: URISpecification.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: URISpecification.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.datatypes;
 
@@ -25,7 +25,10 @@ import java.io.UnsupportedEncodingException;
 /**
  * This class contains method to deal with the <uri-specification> datatype from XSL-FO.
  */
-public class URISpecification {
+public final class URISpecification {
+
+    private URISpecification() {
+    }
 
     /**
      * Get the URL string from a wrapped URL.
@@ -94,7 +97,7 @@ public class URISpecification {
         return false;
     }
 
-    private final static char[] HEX_DIGITS = {
+    private static final char[] HEX_DIGITS = {
         '0', '1', '2', '3', '4', '5', '6', '7',
         '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
     };
