@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ElementMapping.java 722092 2008-12-01 15:30:59Z jeremias $ */
+/* $Id: ElementMapping.java 1058295 2011-01-12 19:58:50Z jeremias $ */
 
 package org.apache.fop.fo;
 
@@ -30,7 +30,7 @@ import org.apache.xmlgraphics.util.QName;
 
 /**
  * Abstract base class for Element Mappings (including FO Element Mappings)
- * which provide the framework of valid elements and attibutes for a given
+ * which provide the framework of valid elements and attributes for a given
  * namespace.
  */
 public abstract class ElementMapping {
@@ -38,7 +38,7 @@ public abstract class ElementMapping {
     public static final String DEFAULT = "<default>";
 
     /** The HashMap table of formatting objects defined by the ElementMapping */
-    protected HashMap foObjs = null;
+    protected HashMap<String, Maker> foObjs = null;
     //Please don't change that to java.util.Map as that can break extensions.
 
     /** The namespace for the ElementMapping */
@@ -49,7 +49,7 @@ public abstract class ElementMapping {
      *
      * @return Table of Maker objects for this ElementMapping
      */
-    public HashMap getTable() {
+    public HashMap<String, Maker> getTable() {
         if (foObjs == null) {
             initialize();
         }
