@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: NumericProperty.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: NumericProperty.java 1303891 2012-03-22 17:04:12Z vhennebert $ */
 
 package org.apache.fop.fo.expr;
 
@@ -23,10 +23,11 @@ import java.awt.Color;
 
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.datatypes.Length;
-import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.datatypes.Numeric;
+import org.apache.fop.datatypes.PercentBaseContext;
 import org.apache.fop.fo.properties.FixedLength;
 import org.apache.fop.fo.properties.Property;
+import org.apache.fop.util.CompareUtil;
 
 /**
  * A numeric property which hold the final absolute result of an expression
@@ -127,4 +128,25 @@ public class NumericProperty extends Property implements Numeric, Length {
             return value + "^" + dim;
         }
     }
+
+    @Override
+    public int hashCode() {
+        final int prime = 31;
+        int result = 1;
+        result = prime * result + dim;
+        result = prime * result + CompareUtil.getHashCode(value);
+        return result;
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj) {
+            return true;
+        }
+        if (!(obj instanceof NumericProperty)) {
+            return false;
+        }
+        NumericProperty other = (NumericProperty) obj;
+        return dim == other.dim && CompareUtil.equal(value, other.value);
+    }
 }
