@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ExternalGraphic.java 628652 2008-02-18 09:00:08Z jeremias $ */
+/* $Id: ExternalGraphic.java 932497 2010-04-09 16:34:29Z vhennebert $ */
 
 package org.apache.fop.fo.flow;
 
@@ -28,6 +28,7 @@ import org.apache.xmlgraphics.image.loader.ImageException;
 import org.apache.xmlgraphics.image.loader.ImageInfo;
 import org.apache.xmlgraphics.image.loader.ImageManager;
 
+import org.apache.fop.ResourceEventProducer;
 import org.apache.fop.apps.FOPException;
 import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.datatypes.Length;
@@ -44,7 +45,7 @@ import org.apache.fop.fo.properties.FixedLength;
  * inline area that can be added to the area tree.
  */
 public class ExternalGraphic extends AbstractGraphics {
-    
+
     // The value of properties relevant for fo:external-graphic.
     // All but one of the e-g properties are kept in AbstractGraphics
     private String src;
@@ -55,9 +56,10 @@ public class ExternalGraphic extends AbstractGraphics {
     private int intrinsicWidth;
     private int intrinsicHeight;
     private Length intrinsicAlignmentAdjust;
-    
+
     /**
-     * Create a new External graphic node.
+     * Create a new ExternalGraphic node that is a child
+     * of the given {@link FONode}.
      *
      * @param parent the parent of this node
      */
@@ -69,7 +71,7 @@ public class ExternalGraphic extends AbstractGraphics {
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
         src = pList.get(PR_SRC).getString();
-        
+
         //Additional processing: obtain the image's intrinsic size and baseline information
         url = URISpecification.getURL(src);
         FOUserAgent userAgent = getUserAgent();
@@ -78,11 +80,17 @@ public class ExternalGraphic extends AbstractGraphics {
         try {
             info = manager.getImageInfo(url, userAgent.getImageSessionContext());
         } catch (ImageException e) {
-            log.error("Image not available: " + e.getMessage());
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageError(this, url, e, getLocator());
         } catch (FileNotFoundException fnfe) {
-            log.error(fnfe.getMessage());
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageNotFound(this, url, fnfe, getLocator());
         } catch (IOException ioe) {
-            log.error("I/O error while loading image: " + ioe.getMessage());
+            ResourceEventProducer eventProducer = ResourceEventProducer.Provider.get(
+                    getUserAgent().getEventBroadcaster());
+            eventProducer.imageIOError(this, url, ioe, getLocator());
         }
         if (info != null) {
             this.intrinsicWidth = info.getSize().getWidthMpt();
@@ -93,7 +101,6 @@ public class ExternalGraphic extends AbstractGraphics {
                     = FixedLength.getInstance(-baseline);
             }
         }
-        //TODO Report to caller so he can decide to throw an exception
     }
 
     /** {@inheritDoc} */
@@ -106,9 +113,11 @@ public class ExternalGraphic extends AbstractGraphics {
      * {@inheritDoc}
      * <br>XSL Content Model: empty
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
             invalidChildError(loc, nsURI, localName);
+        }
     }
 
     /** @return the "src" property */
@@ -126,7 +135,10 @@ public class ExternalGraphic extends AbstractGraphics {
         return "external-graphic";
     }
 
-    /** {@inheritDoc} */
+    /**
+     * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_EXTERNAL_GRAPHIC}
+     */
     public int getNameId() {
         return FO_EXTERNAL_GRAPHIC;
     }
