@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Inline.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: Inline.java 1310853 2012-04-07 20:23:56Z gadams $ */
 
 package org.apache.fop.fo.flow;
 
@@ -26,19 +26,17 @@ import org.apache.fop.datatypes.Length;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
-import org.apache.fop.fo.properties.StructurePointerPropertySet;
 
 /**
  * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_inline">
  * <code>fo:inline</code></a> formatting object.
  */
-public class Inline extends InlineLevel implements StructurePointerPropertySet {
+public class Inline extends InlineLevel {
     // The value of properties relevant for fo:inline.
     // See also superclass InlineLevel
     private Length alignmentAdjust;
     private int alignmentBaseline;
     private Length baselineShift;
-    private String ptr;  // used for accessibility
     private int dominantBaseline;
     // Unused but valid items, commented out for performance:
     //     private CommonRelativePosition commonRelativePosition;
@@ -68,7 +66,6 @@ public class Inline extends InlineLevel implements StructurePointerPropertySet {
         alignmentBaseline = pList.get(PR_ALIGNMENT_BASELINE).getEnum();
         baselineShift = pList.get(PR_BASELINE_SHIFT).getLength();
         dominantBaseline = pList.get(PR_DOMINANT_BASELINE).getEnum();
-        ptr = pList.get(PR_X_PTR).getString(); // used for accessibility
     }
 
     /** {@inheritDoc} */
@@ -119,8 +116,9 @@ public class Inline extends InlineLevel implements StructurePointerPropertySet {
                 }
             } else if (!isBlockOrInlineItem(nsURI, localName)) {
                 invalidChildError(loc, nsURI, localName);
-            } else if (!canHaveBlockLevelChildren && isBlockItem(nsURI, localName)) {
-                invalidChildError(loc, getParent().getName(), nsURI, getName(), "rule.inlineContent");
+            } else if (!canHaveBlockLevelChildren && isBlockItem(nsURI, localName)
+                       && !isNeutralItem(nsURI, localName)) {
+                invalidChildError(loc, getName(), nsURI, localName, "rule.inlineContent");
             } else {
                 blockOrInlineItemFound = true;
             }
@@ -147,11 +145,6 @@ public class Inline extends InlineLevel implements StructurePointerPropertySet {
         return dominantBaseline;
     }
 
-    /** {@inheritDoc} */
-    public String getPtr() {
-        return ptr;
-    }
-
     /** {@inheritDoc} */
     public String getLocalName() {
         return "inline";
