@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ListItemLabel.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: ListItemLabel.java 1242848 2012-02-10 16:51:08Z phancock $ */
 
 package org.apache.fop.fo.flow;
 
@@ -40,13 +40,13 @@ public class ListItemLabel extends AbstractListItemPart {
     /** {@inheritDoc} */
     protected void startOfNode() throws FOPException {
         super.startOfNode();
-        getFOEventHandler().startListLabel();
+        getFOEventHandler().startListLabel(this);
     }
 
     /** {@inheritDoc} */
     protected void endOfNode() throws FOPException {
         super.endOfNode();
-        getFOEventHandler().endListLabel();
+        getFOEventHandler().endListLabel(this);
     }
 
     /** {@inheritDoc} */
