@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MultiProperties.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: MultiProperties.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.fo.flow;
 
@@ -37,11 +37,11 @@ public class MultiProperties extends FObj {
     //     private CommonAccessibility commonAccessibility;
     // End of property values
 
-    static boolean notImplementedWarningGiven = false;
+    private static boolean notImplementedWarningGiven = false;
 
     // used for input FO validation
-    boolean hasMultiPropertySet = false;
-    boolean hasWrapper = false;
+    private boolean hasMultiPropertySet = false;
+    private boolean hasWrapper = false;
 
     /**
      * Base constructor
