@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MultiToggle.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: MultiToggle.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.fo.flow;
 
@@ -39,7 +39,7 @@ public class MultiToggle extends FObj {
     // public ToBeImplementedProperty prSwitchTo;
     // End of property values
 
-    static boolean notImplementedWarningGiven = false;
+    private static boolean notImplementedWarningGiven = false;
 
     /**
      * Base constructor
