@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: Wrapper.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: Wrapper.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.fo.flow;
 
@@ -26,7 +26,10 @@ import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FOText;
 import org.apache.fop.fo.FObjMixed;
+import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.ValidationException;
+import org.apache.fop.fo.properties.CommonAccessibility;
+import org.apache.fop.fo.properties.CommonAccessibilityHolder;
 
 /**
  * Class modelling the <a href=http://www.w3.org/TR/xsl/#fo_wrapper">
@@ -34,13 +37,13 @@ import org.apache.fop.fo.ValidationException;
  * The <code>fo:wrapper</code> object serves as a property holder for
  * its child node objects.
  */
-public class Wrapper extends FObjMixed {
-    // The value of properties relevant for fo:wrapper.
-    // End of property values
+public class Wrapper extends FObjMixed implements CommonAccessibilityHolder {
 
     // used for FO validation
     private boolean blockOrInlineItemFound = false;
 
+    private CommonAccessibility commonAccessibility;
+
     /**
      * Create a Wrapper instance that is a child of the
      * given {@link FONode}
@@ -51,6 +54,24 @@ public class Wrapper extends FObjMixed {
         super(parent);
     }
 
+    @Override
+    public void bind(PropertyList pList) throws FOPException {
+        super.bind(pList);
+        commonAccessibility = CommonAccessibility.getInstance(pList);
+    }
+
+    @Override
+    protected void startOfNode() throws FOPException {
+        super.startOfNode();
+        getFOEventHandler().startWrapper(this);
+    }
+
+    @Override
+    protected void endOfNode() throws FOPException {
+        super.endOfNode();
+        getFOEventHandler().endWrapper(this);
+    }
+
     /**
      * {@inheritDoc}
      * <br>XSL Content Model: marker* (#PCDATA|%inline;|%block;)*
@@ -76,7 +97,8 @@ public class Wrapper extends FObjMixed {
                 try {
                     FONode.validateChildNode(this.parent, loc, nsURI, localName);
                 } catch (ValidationException vex) {
-                    invalidChildError(loc, getName(), FO_URI, localName, "rule.wrapperInvalidChildForParent");
+                    invalidChildError(loc, getName(), FO_URI, localName,
+                                      "rule.wrapperInvalidChildForParent");
                 }
                 blockOrInlineItemFound = true;
             } else {
@@ -122,5 +144,15 @@ public class Wrapper extends FObjMixed {
     public int getNameId() {
         return FO_WRAPPER;
     }
+
+    public CommonAccessibility getCommonAccessibility() {
+        return commonAccessibility;
+    }
+
+    @Override
+    public boolean isDelimitedTextRangeBoundary ( int boundary ) {
+        return false;
+    }
+
 }
 
