@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CollapsingBorderResolver.java 614993 2008-01-24 21:10:22Z vhennebert $ */
+/* $Id: CollapsingBorderResolver.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.flow.table;
 
@@ -24,6 +24,7 @@ import java.util.Iterator;
 import java.util.List;
 
 import org.apache.fop.fo.properties.CommonBorderPaddingBackground;
+import org.apache.fop.layoutmgr.table.CollapsingBorderModel;
 
 /**
  * A class that implements the border-collapsing model.
@@ -32,6 +33,8 @@ class CollapsingBorderResolver implements BorderResolver {
 
     private Table table;
 
+    private CollapsingBorderModel collapsingBorderModel;
+
     /**
      * The previously registered row, either in the header or the body(-ies), but not in
      * the footer (handled separately).
@@ -70,34 +73,37 @@ class CollapsingBorderResolver implements BorderResolver {
      */
     private abstract class Resolver {
 
-        protected TableBody tablePart;
+        protected TablePart tablePart;
 
         protected boolean firstInPart;
 
+        private BorderSpecification borderStartTableAndBody;
+        private BorderSpecification borderEndTableAndBody;
+
         /**
          * Integrates border-before specified on the table and its column.
-         * 
+         *
          * @param row the first row of the table (in the header, or in the body if the
          * table has no header)
+         * @param withNormal
          * @param withLeadingTrailing
-         * @param withNonLeadingTrailing
          * @param withRest
          */
-        void resolveBordersFirstRowInTable(List/*<GridUnit>*/ row, boolean withLeadingTrailing,
-                boolean withNonLeadingTrailing, boolean withRest) {
+        void resolveBordersFirstRowInTable(List/*<GridUnit>*/ row, boolean withNormal,
+                boolean withLeadingTrailing, boolean withRest) {
             assert firstInTable;
             for (int i = 0; i < row.size(); i++) {
                 TableColumn column = table.getColumn(i);
                 ((GridUnit) row.get(i)).integrateBorderSegment(
-                        CommonBorderPaddingBackground.BEFORE, column, withLeadingTrailing,
-                        withNonLeadingTrailing, withRest);
+                        CommonBorderPaddingBackground.BEFORE, column, withNormal,
+                        withLeadingTrailing, withRest);
             }
             firstInTable = false;
         }
 
         /**
          * Resolves border-after for the first row, border-before for the second one.
-         * 
+         *
          * @param rowBefore
          * @param rowAfter
          */
@@ -113,29 +119,29 @@ class CollapsingBorderResolver implements BorderResolver {
         }
 
         /** Integrates the border-after of the part. */
-        void resolveBordersLastRowInPart(List/*<GridUnit>*/ row, boolean withLeadingTrailing,
-                boolean withNonLeadingTrailing, boolean withRest) {
+        void resolveBordersLastRowInPart(List/*<GridUnit>*/ row, boolean withNormal,
+                boolean withLeadingTrailing, boolean withRest) {
             for (int i = 0; i < row.size(); i++) {
                 ((GridUnit) row.get(i)).integrateBorderSegment(CommonBorderPaddingBackground.AFTER,
-                        tablePart, withLeadingTrailing, withNonLeadingTrailing, withRest);
+                        tablePart, withNormal, withLeadingTrailing, withRest);
             }
         }
 
         /**
          * Integrates border-after specified on the table and its columns.
-         * 
+         *
          * @param row the last row of the footer, or of the last body if the table has no
          * footer
+         * @param withNormal
          * @param withLeadingTrailing
-         * @param withNonLeadingTrailing
          * @param withRest
          */
-        void resolveBordersLastRowInTable(List/*<GridUnit>*/ row, boolean withLeadingTrailing,
-                boolean withNonLeadingTrailing, boolean withRest) {
+        void resolveBordersLastRowInTable(List/*<GridUnit>*/ row, boolean withNormal,
+                boolean withLeadingTrailing, boolean withRest) {
             for (int i = 0; i < row.size(); i++) {
                 TableColumn column = table.getColumn(i);
                 ((GridUnit) row.get(i)).integrateBorderSegment(CommonBorderPaddingBackground.AFTER,
-                        column, withLeadingTrailing, withNonLeadingTrailing, withRest);
+                        column, withNormal, withLeadingTrailing, withRest);
             }
         }
 
@@ -143,7 +149,7 @@ class CollapsingBorderResolver implements BorderResolver {
          * Integrates either border-before specified on the table and its columns if the
          * table has no header, or border-after specified on the cells of the header's
          * last row. For the case the grid unit are at the top of a page.
-         * 
+         *
          * @param row
          */
         void integrateLeadingBorders(List/*<GridUnit>*/ row) {
@@ -151,7 +157,7 @@ class CollapsingBorderResolver implements BorderResolver {
                 GridUnit gu = (GridUnit) row.get(i);
                 ConditionalBorder border = (ConditionalBorder) leadingBorders.get(i);
                 gu.integrateCompetingBorder(CommonBorderPaddingBackground.BEFORE, border,
-                        true, false, true);
+                        false, true, true);
             }
         }
 
@@ -159,7 +165,7 @@ class CollapsingBorderResolver implements BorderResolver {
          * Integrates either border-after specified on the table and its columns if the
          * table has no footer, or border-before specified on the cells of the footer's
          * first row. For the case the grid unit are at the bottom of a page.
-         * 
+         *
          * @param row
          */
         void integrateTrailingBorders(List/*<GridUnit>*/ row) {
@@ -167,13 +173,17 @@ class CollapsingBorderResolver implements BorderResolver {
                 GridUnit gu = (GridUnit) row.get(i);
                 ConditionalBorder border = (ConditionalBorder) trailingBorders.get(i);
                 gu.integrateCompetingBorder(CommonBorderPaddingBackground.AFTER, border,
-                        true, false, true);
+                        false, true, true);
             }
         }
 
-        void startPart(TableBody part) {
+        void startPart(TablePart part) {
             tablePart = part;
             firstInPart = true;
+            borderStartTableAndBody = collapsingBorderModel.determineWinner(table.borderStart,
+                    tablePart.borderStart);
+            borderEndTableAndBody = collapsingBorderModel.determineWinner(table.borderEnd,
+                    tablePart.borderEnd);
         }
 
         /**
@@ -183,11 +193,13 @@ class CollapsingBorderResolver implements BorderResolver {
          * <li>Integrates the border-before of the containing part, if first row;</li>
          * <li>Resolves border-start/end between grid units.</li>
          * </ul>
-         * 
+         *
          * @param row the row being finished
          * @param container the containing element
          */
         void endRow(List/*<GridUnit>*/ row, TableCellContainer container) {
+            BorderSpecification borderStart = borderStartTableAndBody;
+            BorderSpecification borderEnd = borderEndTableAndBody;
             // Resolve before- and after-borders for the table-row
             if (container instanceof TableRow) {
                 TableRow tableRow = (TableRow) container;
@@ -200,6 +212,10 @@ class CollapsingBorderResolver implements BorderResolver {
                     gu.integrateBorderSegment(CommonBorderPaddingBackground.AFTER, tableRow,
                             last, last, true);
                 }
+                borderStart = collapsingBorderModel.determineWinner(borderStart,
+                        tableRow.borderStart);
+                borderEnd = collapsingBorderModel.determineWinner(borderEnd,
+                        tableRow.borderEnd);
             }
             if (firstInPart) {
                 // Integrate the border-before of the part
@@ -215,7 +231,7 @@ class CollapsingBorderResolver implements BorderResolver {
             Iterator colIter = table.getColumns().iterator();
             TableColumn col = (TableColumn) colIter.next();
             gu.integrateBorderSegment(CommonBorderPaddingBackground.START, col);
-            gu.integrateBorderSegment(CommonBorderPaddingBackground.START, container);
+            gu.integrateBorderSegment(CommonBorderPaddingBackground.START, borderStart);
             while (guIter.hasNext()) {
                 GridUnit nextGU = (GridUnit) guIter.next();
                 TableColumn nextCol = (TableColumn) colIter.next();
@@ -228,7 +244,7 @@ class CollapsingBorderResolver implements BorderResolver {
                 col = nextCol;
             }
             gu.integrateBorderSegment(CommonBorderPaddingBackground.END, col);
-            gu.integrateBorderSegment(CommonBorderPaddingBackground.END, container);
+            gu.integrateBorderSegment(CommonBorderPaddingBackground.END, borderEnd);
         }
 
         void endPart() {
@@ -250,16 +266,15 @@ class CollapsingBorderResolver implements BorderResolver {
                  * The two only sensible values for border-before on the header's first row are:
                  * - at the beginning of the table (normal case)
                  * - if the header is repeated after each page break
-                 * To represent those values we (ab)use the nonLeadingTrailing and the rest
-                 * fields of ConditionalBorder. But strictly speaking this is not their
-                 * purposes.
+                 * To represent those values we (ab)use the normal and the rest fields of
+                 * ConditionalBorder. But strictly speaking this is not their purposes.
                  */
                 for (Iterator guIter = row.iterator(); guIter.hasNext();) {
                     ConditionalBorder borderBefore = ((GridUnit) guIter.next()).borderBefore;
-                    borderBefore.leadingTrailing = borderBefore.nonLeadingTrailing;
-                    borderBefore.rest = borderBefore.nonLeadingTrailing;
+                    borderBefore.leadingTrailing = borderBefore.normal;
+                    borderBefore.rest = borderBefore.normal;
                 }
-                resolveBordersFirstRowInTable(row, false, true, true);
+                resolveBordersFirstRowInTable(row, true, false, true);
             }
             previousRow = row;
         }
@@ -272,14 +287,14 @@ class CollapsingBorderResolver implements BorderResolver {
              * The border-after of a header is always the same. Leading and rest don't
              * apply to cells in the header since they are never broken. To ease
              * resolution we override the (normally unused) leadingTrailing and rest
-             * fields of ConditionalBorder with the only sensible nonLeadingTrailing
-             * field. That way grid units from the body will always resolve against the
-             * same, normal header border.
+             * fields of ConditionalBorder with the only sensible normal field. That way
+             * grid units from the body will always resolve against the same, normal
+             * header border.
              */
             for (Iterator guIter = previousRow.iterator(); guIter.hasNext();) {
                 ConditionalBorder borderAfter = ((GridUnit) guIter.next()).borderAfter;
-                borderAfter.leadingTrailing = borderAfter.nonLeadingTrailing;
-                borderAfter.rest = borderAfter.nonLeadingTrailing;
+                borderAfter.leadingTrailing = borderAfter.normal;
+                borderAfter.rest = borderAfter.normal;
                 leadingBorders.add(borderAfter);
             }
             /* TODO Temporary hack for resolved borders in header */
@@ -311,8 +326,8 @@ class CollapsingBorderResolver implements BorderResolver {
             // See same method in ResolverInHeader for an explanation of the hack
             for (Iterator guIter = footerFirstRow.iterator(); guIter.hasNext();) {
                 ConditionalBorder borderBefore = ((GridUnit) guIter.next()).borderBefore;
-                borderBefore.leadingTrailing = borderBefore.nonLeadingTrailing;
-                borderBefore.rest = borderBefore.nonLeadingTrailing;
+                borderBefore.leadingTrailing = borderBefore.normal;
+                borderBefore.rest = borderBefore.normal;
                 trailingBorders.add(borderBefore);
             }
         }
@@ -324,10 +339,10 @@ class CollapsingBorderResolver implements BorderResolver {
             // See endRow method in ResolverInHeader for an explanation of the hack
             for (Iterator guIter = footerLastRow.iterator(); guIter.hasNext();) {
                 ConditionalBorder borderAfter = ((GridUnit) guIter.next()).borderAfter;
-                borderAfter.leadingTrailing = borderAfter.nonLeadingTrailing;
-                borderAfter.rest = borderAfter.nonLeadingTrailing;
+                borderAfter.leadingTrailing = borderAfter.normal;
+                borderAfter.rest = borderAfter.normal;
             }
-            resolveBordersLastRowInTable(footerLastRow, false, true, true);
+            resolveBordersLastRowInTable(footerLastRow, true, false, true);
         }
     }
 
@@ -351,7 +366,7 @@ class CollapsingBorderResolver implements BorderResolver {
                 firstInBody = false;
                 for (Iterator iter = row.iterator(); iter.hasNext();) {
                     GridUnit gu = (GridUnit) iter.next();
-                    gu.borderBefore.leadingTrailing = gu.borderBefore.nonLeadingTrailing;
+                    gu.borderBefore.leadingTrailing = gu.borderBefore.normal;
                 }
             }
         }
@@ -361,18 +376,37 @@ class CollapsingBorderResolver implements BorderResolver {
                 resolverInFooter.endTable();
             } else {
                 // Trailing and rest borders already resolved with integrateTrailingBorders
-                resolveBordersLastRowInTable(previousRow, false, true, false);
+                resolveBordersLastRowInTable(previousRow, true, false, false);
             }
             for (Iterator iter = previousRow.iterator(); iter.hasNext();) {
                 GridUnit gu = (GridUnit) iter.next();
-                gu.borderAfter.leadingTrailing = gu.borderAfter.nonLeadingTrailing;
+                gu.borderAfter.leadingTrailing = gu.borderAfter.normal;
             }
         }
     }
 
     CollapsingBorderResolver(Table table) {
         this.table = table;
+        collapsingBorderModel = CollapsingBorderModel.getBorderModelFor(table.getBorderCollapse());
         firstInTable = true;
+        // Resolve before and after borders between the table and each table-column
+        int index = 0;
+        do {
+            TableColumn col = table.getColumn(index);
+            // See endRow method in ResolverInHeader for an explanation of the hack
+            col.borderBefore.integrateSegment(table.borderBefore, true, false, true);
+            col.borderBefore.leadingTrailing = col.borderBefore.rest;
+            col.borderAfter.integrateSegment(table.borderAfter, true, false, true);
+            col.borderAfter.leadingTrailing = col.borderAfter.rest;
+            /*
+             * TODO The border resolution must be done only once for each table column,
+             * even if it's repeated; otherwise, re-resolving against the table's borders
+             * will lead to null border specifications.
+             *
+             * Eventually table columns should probably be cloned instead.
+             */
+            index += col.getNumberColumnsRepeated();
+        } while (index < table.getNumberOfColumns());
     }
 
     /** {@inheritDoc} */
@@ -381,21 +415,19 @@ class CollapsingBorderResolver implements BorderResolver {
     }
 
     /** {@inheritDoc} */
-    public void startPart(TableBody part) {
-        if (part.isTableHeader()) {
+    public void startPart(TablePart part) {
+        if (part instanceof TableHeader) {
             delegate = new ResolverInHeader();
         } else {
             if (leadingBorders == null || table.omitHeaderAtBreak()) {
                 // No header, leading borders determined by the table
                 leadingBorders = new ArrayList(table.getNumberOfColumns());
                 for (Iterator colIter = table.getColumns().iterator(); colIter.hasNext();) {
-                    // See endRow method in ResolverInHeader for an explanation of the hack
                     ConditionalBorder border = ((TableColumn) colIter.next()).borderBefore;
-                    border.leadingTrailing = border.rest;
                     leadingBorders.add(border);
                 }
             }
-            if (part.isTableFooter()) {
+            if (part instanceof TableFooter) {
                 resolverInFooter = new ResolverInFooter();
                 delegate = resolverInFooter;
             } else {
@@ -403,9 +435,7 @@ class CollapsingBorderResolver implements BorderResolver {
                     // No footer, trailing borders determined by the table
                     trailingBorders = new ArrayList(table.getNumberOfColumns());
                     for (Iterator colIter = table.getColumns().iterator(); colIter.hasNext();) {
-                        // See endRow method in ResolverInHeader for an explanation of the hack
                         ConditionalBorder border = ((TableColumn) colIter.next()).borderAfter;
-                        border.leadingTrailing = border.rest;
                         trailingBorders.add(border);
                     }
                 }
@@ -428,13 +458,13 @@ class CollapsingBorderResolver implements BorderResolver {
         if (headerLastRow != null) {
             for (Iterator iter = headerLastRow.iterator(); iter.hasNext();) {
                 GridUnit gu = (GridUnit) iter.next();
-                gu.borderAfter.leadingTrailing = gu.borderAfter.nonLeadingTrailing;
+                gu.borderAfter.leadingTrailing = gu.borderAfter.normal;
             }
         }
         if (footerLastRow != null) {
             for (Iterator iter = footerLastRow.iterator(); iter.hasNext();) {
                 GridUnit gu = (GridUnit) iter.next();
-                gu.borderAfter.leadingTrailing = gu.borderAfter.nonLeadingTrailing;
+                gu.borderAfter.leadingTrailing = gu.borderAfter.normal;
             }
         }
         /* End of temporary hack */
