@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ColumnNumberManager.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: ColumnNumberManager.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.fo.flow.table;
 
@@ -77,8 +77,7 @@ public class ColumnNumberManager {
         for (int i = 0; i < pendingSpans.size(); i++) {
             pSpan = (PendingSpan) pendingSpans.get(i);
             if (pSpan != null) {
-                pSpan.rowsLeft--;
-                if (pSpan.rowsLeft == 0) {
+                if ( pSpan.decrRowsLeft() == 0 ) {
                     pendingSpans.set(i, null);
                 } else {
                     usedColumnIndices.set(i);
