@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPageSequence.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: AbstractPageSequence.java 1342680 2012-05-25 15:15:28Z vhennebert $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -24,6 +24,8 @@ import org.apache.fop.datatypes.Numeric;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
 import org.apache.fop.fo.PropertyList;
+import org.apache.fop.fo.properties.CommonAccessibility;
+import org.apache.fop.fo.properties.CommonAccessibilityHolder;
 
 /**
  * Abstract base class for the <a href="http://www.w3.org/TR/xsl/#fo_page-sequence">
@@ -31,20 +33,26 @@ import org.apache.fop.fo.PropertyList;
  * <a href="http://xmlgraphics.apache.org/fop/0.95/extensions.html#external-document">
  * <code>fox:external-document</code></a> extension object.
  */
-public abstract class AbstractPageSequence extends FObj {
+public abstract class AbstractPageSequence extends FObj implements CommonAccessibilityHolder {
 
-    // The value of properties relevant for fo:page-sequence.
+    /** initial page number */
     protected Numeric initialPageNumber;
+    /** forced page count */
     protected int forcePageCount;
     private String format;
     private int letterValue;
     private char groupingSeparator;
     private int groupingSize;
-    private Numeric referenceOrientation; //XSL 1.1
-    // End of property values
+    private Numeric referenceOrientation;
+    private String language;
+    private String country;
+    private String numberConversionFeatures;
+
+    private CommonAccessibility commonAccessibility;
 
     private PageNumberGenerator pageNumberGenerator;
 
+    /** starting page number */
     protected int startingPageNumber = 0;
 
     /**
@@ -67,12 +75,17 @@ public abstract class AbstractPageSequence extends FObj {
         groupingSeparator = pList.get(PR_GROUPING_SEPARATOR).getCharacter();
         groupingSize = pList.get(PR_GROUPING_SIZE).getNumber().intValue();
         referenceOrientation = pList.get(PR_REFERENCE_ORIENTATION).getNumeric();
+        language = pList.get(PR_LANGUAGE).getString();
+        country = pList.get(PR_COUNTRY).getString();
+        numberConversionFeatures = pList.get(PR_X_NUMBER_CONVERSION_FEATURES).getString();
+        commonAccessibility = CommonAccessibility.getInstance(pList);
     }
 
     /** {@inheritDoc} */
     protected void startOfNode() throws FOPException {
         this.pageNumberGenerator = new PageNumberGenerator(
-                format, groupingSeparator, groupingSize, letterValue);
+                format, groupingSeparator, groupingSize, letterValue,
+                numberConversionFeatures, language, country);
 
     }
 
@@ -120,6 +133,10 @@ public abstract class AbstractPageSequence extends FObj {
         return pageNumberGenerator.makeFormattedPageNumber(pageNumber);
     }
 
+    public CommonAccessibility getCommonAccessibility() {
+        return commonAccessibility;
+    }
+
     /**
      * Public accessor for the ancestor Root.
      * @return the ancestor Root
