@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RegionBody.java 557337 2007-07-18 17:37:14Z adelmelle $ */
+/* $Id: RegionBody.java 1296496 2012-03-02 22:19:46Z gadams $ */
 
 package org.apache.fop.fo.pagination;
 
@@ -27,13 +27,15 @@ import org.apache.fop.datatypes.FODimension;
 import org.apache.fop.datatypes.Length;
 import org.apache.fop.datatypes.LengthBase;
 import org.apache.fop.datatypes.Numeric;
-import org.apache.fop.datatypes.SimplePercentBaseContext;
+import org.apache.fop.datatypes.PercentBaseContext;
+import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.PropertyList;
 import org.apache.fop.fo.properties.CommonMarginBlock;
 
 /**
- * The fo:region-body element.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_region-body">
+ * <code>fo:region-body</code></a> object.
  */
 public class RegionBody extends Region {
     // The value of properties relevant for fo:region-body.
@@ -43,42 +45,43 @@ public class RegionBody extends Region {
     // End of property values
 
     /**
-     * @see org.apache.fop.fo.FONode#FONode(FONode)
+     * Create a RegionBody instance that is a child of the
+     * given parent {@link FONode}.
+     * @param parent    the {@link FONode} that is to be the parent
      */
     public RegionBody(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void bind(PropertyList pList) throws FOPException {
         super.bind(pList);
         commonMarginBlock = pList.getMarginBlockProps();
         columnCount = pList.get(PR_COLUMN_COUNT).getNumeric();
         columnGap = pList.get(PR_COLUMN_GAP).getLength();
-        
+
         if ((getColumnCount() > 1) && (getOverflow() == EN_SCROLL)) {
             /* This is an error (See XSL Rec, fo:region-body description).
              * The Rec allows for acting as if "1" is chosen in
              * these cases, but we will need to be able to change Numeric
              * values in order to do this.
              */
-            attributeError("If overflow property is set to \"scroll\"," 
-                    + " a column-count other than \"1\" may not be specified.");
+            getFOValidationEventProducer().columnCountErrorOnRegionBodyOverflowScroll(this,
+                    getName(), getLocator());
         }
     }
 
     /**
-     * Return the Common Margin Properties-Block.
-     * @return the Common Margin Properties-Block.
+     * Return the {@link CommonMarginBlock} instance attached to
+     * this instance.
+     * @return the {@link CommonMarginBlock} instance
      */
     public CommonMarginBlock getCommonMarginBlock() {
         return commonMarginBlock;
     }
 
     /**
-     * Return the "column-count" property.
+     * Return the value of the <code>column-count<code> property.
      * @return the "column-count" property.
      */
     public int getColumnCount() {
@@ -86,53 +89,47 @@ public class RegionBody extends Region {
     }
 
     /**
-     * Return the "column-gap" property.
+     * Return the value of the <code>column-gap</code> property.
      * @return the "column-gap" property.
      */
     public int getColumnGap() {
         return columnGap.getValue();
     }
 
-    /**
-     * {@inheritDoc} 
-     */
-    public Rectangle getViewportRectangle (FODimension reldims, SimplePageMaster spm) {
+    /** {@inheritDoc} */
+    public Rectangle getViewportRectangle (FODimension reldims) {
         /* Special rules apply to resolving margins in the page context.
          * Contrary to normal margins in this case top and bottom margin
          * are resolved relative to the height. In the property subsystem
          * all margin properties are configured to using BLOCK_WIDTH.
          * That's why we 'cheat' here and setup a context for the height but
          * use the LengthBase.BLOCK_WIDTH.
-         * Also the values are resolved relative to the page size 
+         * Also the values are resolved relative to the page size
          * and reference orientation.
          */
-        SimplePercentBaseContext pageWidthContext;
-        SimplePercentBaseContext pageHeightContext;
-        if (spm.getReferenceOrientation() % 180 == 0) {
-            pageWidthContext = new SimplePercentBaseContext(null, 
-                                                            LengthBase.CONTAINING_BLOCK_WIDTH,
-                                                            spm.getPageWidth().getValue());
-            pageHeightContext = new SimplePercentBaseContext(null,
-                                                             LengthBase.CONTAINING_BLOCK_WIDTH,
-                                                             spm.getPageHeight().getValue());
-        } else {
-            // invert width and height since top left are rotated by 90 (cl or ccl)
-            pageWidthContext = new SimplePercentBaseContext(null, 
-                                                            LengthBase.CONTAINING_BLOCK_WIDTH,
-                                                            spm.getPageHeight().getValue());
-            pageHeightContext = new SimplePercentBaseContext(null,
-                                                             LengthBase.CONTAINING_BLOCK_WIDTH,
-                                                             spm.getPageWidth().getValue());
-        }
+        PercentBaseContext pageWidthContext
+            = getPageWidthContext(LengthBase.CONTAINING_BLOCK_WIDTH);
+        PercentBaseContext pageHeightContext
+            = getPageHeightContext(LengthBase.CONTAINING_BLOCK_WIDTH);
 
         int start;
         int end;
-        if (spm.getWritingMode() == EN_LR_TB) { // Left-to-right
-            start = commonMarginBlock.marginLeft.getValue(pageWidthContext);
-            end = commonMarginBlock.marginRight.getValue(pageWidthContext);
-        } else { // all other supported modes are right-to-left
+        // [TBD] WRITING MODE ALERT
+        switch ( getWritingMode().getEnumValue() ) {
+        case Constants.EN_RL_TB:
             start = commonMarginBlock.marginRight.getValue(pageWidthContext);
             end = commonMarginBlock.marginLeft.getValue(pageWidthContext);
+            break;
+        case Constants.EN_TB_LR:
+        case Constants.EN_TB_RL:
+            start = commonMarginBlock.marginTop.getValue(pageWidthContext);
+            end = commonMarginBlock.marginBottom.getValue(pageWidthContext);
+            break;
+        case Constants.EN_LR_TB:
+        default:
+            start = commonMarginBlock.marginLeft.getValue(pageWidthContext);
+            end = commonMarginBlock.marginRight.getValue(pageWidthContext);
+            break;
         }
         int before = commonMarginBlock.spaceBefore.getOptimum(pageHeightContext)
                         .getLength().getValue(pageHeightContext);
@@ -143,9 +140,7 @@ public class RegionBody extends Region {
                     reldims.bpd - before - after);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected String getDefaultRegionName() {
         return "xsl-region-body";
     }
@@ -154,9 +149,10 @@ public class RegionBody extends Region {
     public String getLocalName() {
         return "region-body";
     }
-    
+
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_REGION_BODY}
      */
     public int getNameId() {
         return FO_REGION_BODY;
