@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BookmarkTree.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: BookmarkTree.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.fo.pagination.bookmarks;
 
@@ -31,33 +31,32 @@ import org.apache.fop.fo.ValidationException;
 import org.apache.fop.fo.pagination.Root;
 
 /**
- * The fo:bookmark-tree formatting object, first introduced in the 
- * XSL 1.1 WD.  Prototype version only, subject to change as XSL 1.1 WD
- * evolves.
+ * Class modelling the <a href="http://www.w3.org/TR/xsl/#fo_bookmark-tree">
+ * <code>fo:bookmark-tree</code></a> object, first introduced in the
+ * XSL 1.1 WD.
  */
 public class BookmarkTree extends FObj {
     private ArrayList bookmarks = new ArrayList();
 
     /**
-     * @see org.apache.fop.fo.FONode#FONode(FONode)
+     * Create a new BookmarkTree object that is a child
+     * of the given {@link FONode}.
+     *
+     * @param parent the {@link FONode} parent
      */
     public BookmarkTree(FONode parent) {
         super(parent);
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     protected void addChildNode(FONode obj) {
         if (obj instanceof Bookmark) {
             bookmarks.add(obj);
         }
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    protected void endOfNode() throws FOPException {
+    /** {@inheritDoc} */
+        protected void endOfNode() throws FOPException {
         if (bookmarks == null) {
            missingChildElementError("(fo:bookmark+)");
         }
@@ -66,16 +65,21 @@ public class BookmarkTree extends FObj {
 
     /**
      * {@inheritDoc}
-        XSL/FOP: (bookmark+)
+     * <br>XSL/FOP: (bookmark+)
      */
-    protected void validateChildNode(Locator loc, String nsURI, String localName) 
-        throws ValidationException {
-        if (!(FO_URI.equals(nsURI) &&
-            localName.equals("bookmark"))) {
+    protected void validateChildNode(Locator loc, String nsURI, String localName)
+                throws ValidationException {
+        if (FO_URI.equals(nsURI)) {
+            if (!localName.equals("bookmark")) {
                 invalidChildError(loc, nsURI, localName);
+            }
         }
     }
 
+    /**
+     * Get the descendant {@link Bookmark}s.
+     * @return an <code>ArrayList</code> containing the {@link Bookmark} objects.
+     */
     public ArrayList getBookmarks() {
         return bookmarks;
     }
@@ -87,6 +91,7 @@ public class BookmarkTree extends FObj {
 
     /**
      * {@inheritDoc}
+     * @return {@link org.apache.fop.fo.Constants#FO_BOOKMARK_TREE}
      */
     public int getNameId() {
         return FO_BOOKMARK_TREE;
