@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: CommonAbsolutePosition.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: CommonAbsolutePosition.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.fo.properties;
 
@@ -33,40 +33,42 @@ public class CommonAbsolutePosition {
     /**
      * The "absolute-position" property.
      */
-    public int absolutePosition;
+    public int absolutePosition;                                // CSOK: VisibilityModifier
 
     /**
      * The "top" property.
      */
-    public Length top;
+    public Length top;                                          // CSOK: VisibilityModifier
 
     /**
      * The "right" property.
      */
-    public Length right;
-    
+    public Length right;                                        // CSOK: VisibilityModifier
+
     /**
      * The "bottom" property.
      */
-    public Length bottom;
-    
+    public Length bottom;                                       // CSOK: VisibilityModifier
+
     /**
      * The "left" property.
      */
-    public Length left;
+    public Length left;                                         // CSOK: VisibilityModifier
 
     /**
      * Create a CommonAbsolutePosition object.
      * @param pList The PropertyList with propery values.
+     * @throws PropertyException if a property exception is raised
      */
     public CommonAbsolutePosition(PropertyList pList) throws PropertyException {
         absolutePosition = pList.get(Constants.PR_ABSOLUTE_POSITION).getEnum();
         top = pList.get(Constants.PR_TOP).getLength();
         bottom = pList.get(Constants.PR_BOTTOM).getLength();
         left = pList.get(Constants.PR_LEFT).getLength();
-        right = pList.get(Constants.PR_RIGHT).getLength();      
+        right = pList.get(Constants.PR_RIGHT).getLength();
     }
-    
+
+    /** {@inheritDoc} */
     public String toString() {
         StringBuffer sb = new StringBuffer("CommonAbsolutePosition{");
         sb.append(" absPos=");
