@@ -15,12 +15,11 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractCodePointMapping.java 731479 2009-01-05 07:47:02Z jeremias $ */
+/* $Id: AbstractCodePointMapping.java 1296483 2012-03-02 21:34:30Z gadams $ */
 
 package org.apache.fop.fonts;
 
 import java.util.Arrays;
-import java.util.Map;
 
 import org.apache.xmlgraphics.fonts.Glyphs;
 
@@ -31,13 +30,12 @@ import org.apache.fop.util.CharUtilities;
  */
 public class AbstractCodePointMapping implements SingleByteEncoding {
 
-    private String name;
+    private final String name;
     private char[] latin1Map;
     private char[] characters;
     private char[] codepoints;
     private char[] unicodeMap; //code point to Unicode char
     private String[] charNameMap; //all character names in the encoding
-    private Map fallbackMap; //Here we accumulate all mappings we have found through substitution
 
     /**
      * Main constructor.
@@ -127,7 +125,8 @@ public class AbstractCodePointMapping implements SingleByteEncoding {
                 return latin1;
             }
         }
-        int bot = 0, top = characters.length - 1;
+        int bot = 0;
+        int top = characters.length - 1;
         while (top >= bot) {
             int mid = (bot + top) / 2;
             char mc = characters[mid];
@@ -140,44 +139,9 @@ public class AbstractCodePointMapping implements SingleByteEncoding {
                 bot = mid + 1;
             }
         }
-
-        //Fallback: using cache
-        synchronized (this) {
-            if (fallbackMap != null) {
-                Character fallback = (Character)fallbackMap.get(new Character(c));
-                if (fallback != null) {
-                    return fallback.charValue();
-                }
-            }
-        }
-        //Fallback: find alternatives (slow!)
-        String glyphName = Glyphs.charToGlyphName(c);
-        if (glyphName.length() > 0) {
-            String[] alternatives = Glyphs.getCharNameAlternativesFor(glyphName);
-            if (alternatives != null) {
-                for (int i = 0, ic = alternatives.length; i < ic; i++) {
-                    int idx = getCodePointForGlyph(alternatives[i]);
-                    if (idx >= 0) {
-                        putFallbackCharacter(c, (char)idx);
-                        return (char)idx;
-                    }
-                }
-            }
-        }
-
-        putFallbackCharacter(c, NOT_FOUND_CODE_POINT);
         return NOT_FOUND_CODE_POINT;
     }
 
-    private void putFallbackCharacter(char c, char mapTo) {
-        synchronized (this) {
-            if (this.fallbackMap == null) {
-                this.fallbackMap = new java.util.HashMap();
-            }
-            this.fallbackMap.put(new Character(c), new Character(mapTo));
-        }
-    }
-
     /**
      * Returns the main Unicode value that is associated with the given code point in the encoding.
      * Note that multiple Unicode values can theoretically be mapped to one code point in the
@@ -239,6 +203,7 @@ public class AbstractCodePointMapping implements SingleByteEncoding {
     }
 
     /** {@inheritDoc} */
+    @Override
     public String toString() {
         return getName();
     }
