@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,36 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: CIDFont.java 627495 2008-02-13 15:55:34Z jeremias $ */
+/* $Id: CIDFont.java 721430 2008-11-28 11:13:12Z acumiskey $ */
 
 package org.apache.fop.fonts;
 
 //Java
-import java.util.Map;
 
 /**
  * Abstract base class for CID fonts.
  */
 public abstract class CIDFont extends CustomFont {
 
-    /**
-     * usedGlyphs contains orginal, new glyph index
-     */
-    public Map usedGlyphs = new java.util.HashMap();
-
-    /**
-     * usedGlyphsIndex contains new glyph, original index
-     */
-    public Map usedGlyphsIndex = new java.util.HashMap();
-    public int usedGlyphsCount = 0;
-
-    /**
-     * usedCharsIndex contains new glyph, original char
-     */
-    public Map usedCharsIndex = new java.util.HashMap();
-
-    //private PDFWArray warray = new PDFWArray();
-    public int width[] = null;
+    /** Contains the character widths for all characters in the font */
+    protected int[] width = null;
 
     // ---- Required ----
     /**
@@ -73,6 +56,11 @@ public abstract class CIDFont extends CustomFont {
      */
     public abstract int getSupplement();
 
+    /**
+     * Returns the subset information for this font.
+     * @return the subset information
+     */
+    public abstract CIDSubset getCIDSubset();
 
     // ---- Optional ----
     /**
@@ -88,9 +76,4 @@ public abstract class CIDFont extends CustomFont {
         return true;
     }
 
-    /**
-     * Returns a char array containing all Unicode characters that have been accessed.
-     * @return a char array with all used Unicode characters
-     */
-    public abstract char[] getCharsUsed();
-}
\ No newline at end of file
+}
