@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,15 @@
  * limitations under the License.
  */
 
-/* $Id: FontReader.java 647403 2008-04-12 09:02:01Z jeremias $ */
+/* $Id: FontReader.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.fonts;
 
 //Java
 import java.io.IOException;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -49,19 +52,19 @@ import org.apache.fop.fonts.apps.TTFReader;
  */
 public class FontReader extends DefaultHandler {
 
-    private Locator locator = null;
+    // private Locator locator = null; // not used at present
     private boolean isCID = false;
     private CustomFont returnFont = null;
     private MultiByteFont multiFont = null;
     private SingleByteFont singleFont = null;
     private StringBuffer text = new StringBuffer();
 
-    private List cidWidths = null;
+    private List<Integer> cidWidths = null;
     private int cidWidthIndex = 0;
 
-    private Map currentKerning = null;
+    private Map<Integer, Integer> currentKerning = null;
 
-    private List bfranges = null;
+    private List<CMapSegment> bfranges = null;
 
     private void createFont(InputSource source) throws FOPException {
         XMLReader parser = null;
@@ -113,6 +116,14 @@ public class FontReader extends DefaultHandler {
         returnFont.setKerningEnabled(enabled);
     }
 
+    /**
+     * Enable/disable use of advanced typographic features for the font
+     * @param enabled true to enable, false to disable
+     */
+    public void setAdvancedEnabled(boolean enabled) {
+        returnFont.setAdvancedEnabled(enabled);
+    }
+
     /**
      * Sets the font resolver. Needed for URI resolution.
      * @param resolver the font resolver
@@ -143,19 +154,22 @@ public class FontReader extends DefaultHandler {
     /**
      * {@inheritDoc}
      */
+    @Override
     public void startDocument() {
     }
 
     /**
      * {@inheritDoc}
      */
+    @Override
     public void setDocumentLocator(Locator locator) {
-        this.locator = locator;
+        // this.locator = locator; // not used at present
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
+    @Override
     public void startElement(String uri, String localName, String qName,
                              Attributes attributes) throws SAXException {
         if (localName.equals("font-metrics")) {
@@ -181,15 +195,15 @@ public class FontReader extends DefaultHandler {
             returnFont.setEmbedResourceName(attributes.getValue("class"));
         } else if ("cid-widths".equals(localName)) {
             cidWidthIndex = getInt(attributes.getValue("start-index"));
-            cidWidths = new java.util.ArrayList();
+            cidWidths = new ArrayList<Integer>();
         } else if ("kerning".equals(localName)) {
-            currentKerning = new java.util.HashMap();
+            currentKerning = new HashMap<Integer, Integer>();
             returnFont.putKerningEntry(new Integer(attributes.getValue("kpx1")),
                                         currentKerning);
         } else if ("bfranges".equals(localName)) {
-            bfranges = new java.util.ArrayList();
+            bfranges = new ArrayList<CMapSegment>();
         } else if ("bf".equals(localName)) {
-            BFEntry entry = new BFEntry(getInt(attributes.getValue("us")),
+            CMapSegment entry = new CMapSegment(getInt(attributes.getValue("us")),
                                         getInt(attributes.getValue("ue")),
                                         getInt(attributes.getValue("gi")));
             bfranges.add(entry);
@@ -209,6 +223,7 @@ public class FontReader extends DefaultHandler {
             currentKerning.put(new Integer(attributes.getValue("kpx2")),
                                new Integer(attributes.getValue("kern")));
         }
+
     }
 
     private int getInt(String str) throws SAXException {
@@ -222,8 +237,9 @@ public class FontReader extends DefaultHandler {
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
+    @Override
     public void endElement(String uri, String localName, String qName) throws SAXException {
         String content = text.toString().trim();
         if ("font-name".equals(localName)) {
@@ -231,7 +247,7 @@ public class FontReader extends DefaultHandler {
         } else if ("full-name".equals(localName)) {
             returnFont.setFullName(content);
         } else if ("family-name".equals(localName)) {
-            Set s = new java.util.HashSet();
+            Set<String> s = new HashSet<String>();
             s.add(content);
             returnFont.setFamilyNames(s);
         } else if ("ttc-name".equals(localName) && isCID) {
@@ -284,25 +300,24 @@ public class FontReader extends DefaultHandler {
             int[] wds = new int[cidWidths.size()];
             int j = 0;
             for (int count = 0; count < cidWidths.size(); count++) {
-                Integer i = (Integer)cidWidths.get(count);
-                wds[j++] = i.intValue();
+                wds[j++] = cidWidths.get(count).intValue();
             }
 
             //multiFont.addCIDWidthEntry(cidWidthIndex, wds);
             multiFont.setWidthArray(wds);
 
         } else if ("bfranges".equals(localName)) {
-            multiFont.setBFEntries((BFEntry[])bfranges.toArray(new BFEntry[0]));
+            multiFont.setCMap(bfranges.toArray(new CMapSegment[0]));
         }
         text.setLength(0); //Reset text buffer (see characters())
     }
 
     /**
-     * {@inheritDoc} 
+     * {@inheritDoc}
      */
+    @Override
     public void characters(char[] ch, int start, int length) {
         text.append(ch, start, length);
     }
+
 }
-
-
