@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,16 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: FontType.java 627679 2008-02-14 08:12:34Z jeremias $ */
- 
-package org.apache.fop.fonts;
+/* $Id: FontType.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
-import org.apache.avalon.framework.ValuedEnum;
+package org.apache.fop.fonts;
 
 /**
  * This class enumerates all supported font types.
  */
-public class FontType extends ValuedEnum {
+public class FontType {
 
     /**
      * Collective identifier for "other" font types
@@ -51,12 +49,19 @@ public class FontType extends ValuedEnum {
      */
     public static final FontType TRUETYPE    = new FontType("TrueType", 5);
 
+    private final String name;
+    private final int value;
+
 
     /**
+     * Construct a font type.
+     * @param name a font type name
+     * @param value a font type value
      * @see org.apache.avalon.framework.Enum#Enum(String)
      */
     protected FontType(String name, int value) {
-        super(name, value);
+        this.name = name;
+        this.value = value;
     }
 
 
@@ -82,8 +87,8 @@ public class FontType extends ValuedEnum {
             throw new IllegalArgumentException("Invalid font type: " + name);
         }
     }
-    
-    
+
+
     /**
      * Returns the FontType by value.
      * @param value Value of the font type to look up
@@ -106,5 +111,28 @@ public class FontType extends ValuedEnum {
             throw new IllegalArgumentException("Invalid font type: " + value);
         }
     }
-    
+
+    /**
+     * Returns the name
+     *
+     * @return the name
+     */
+    public String getName() {
+        return name;
+    }
+
+    /**
+     * Returns the value
+     *
+     * @return the value
+     */
+    public int getValue() {
+        return value;
+    }
+
+    @Override
+    public String toString() {
+        return name;
+    }
+
 }
