@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: FontUtil.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+/* $Id: FontUtil.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.fonts;
 
@@ -23,7 +23,10 @@ package org.apache.fop.fonts;
 /**
  * Font utilities.
  */
-public class FontUtil {
+public final class FontUtil {
+
+    private FontUtil() {
+    }
 
     /**
      * Parses an CSS2 (SVG and XSL-FO) font weight (normal, bold, 100-900) to
