@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: MutableFont.java 721430 2008-11-28 11:13:12Z acumiskey $ */
+/* $Id: MutableFont.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.fonts;
 
@@ -46,7 +46,7 @@ public interface MutableFont {
      * Sets the font's family names (Example: "Helvetica").
      * @param names the font's family names (a Set of Strings)
      */
-    void setFamilyNames(Set names);
+    void setFamilyNames(Set<String> names);
 
     /**
      * Sets the path to the embeddable font file.
@@ -60,6 +60,12 @@ public interface MutableFont {
      */
     void setEmbedResourceName(String name);
 
+    /**
+     * Sets the embedding mode.
+     * @param embeddingMode the embedding mode
+     */
+    void setEmbeddingMode(EmbeddingMode embeddingMode);
+
     /**
      * Sets the capital height value.
      * @param capHeight capital height
@@ -132,11 +138,17 @@ public interface MutableFont {
      */
     void setKerningEnabled(boolean enabled);
 
+    /**
+     * Enables/disabled advanced typographic features.
+     * @param enabled true if advanced typographic features should be enabled if available
+     */
+    void setAdvancedEnabled(boolean enabled);
+
     /**
      * Adds an entry to the kerning table.
      * @param key Kerning key
      * @param value Kerning value
      */
-    void putKerningEntry(Integer key, Map value);
+    void putKerningEntry(Integer key, Map<Integer, Integer> value);
 
 }
