@@ -15,20 +15,22 @@
  * limitations under the License.
  */
 
-/* $Id: CharVector.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: CharVector.java 1311120 2012-04-08 23:48:11Z gadams $ */
 
 package org.apache.fop.hyphenation;
 
 import java.io.Serializable;
 
 /**
- * This class implements a simple char vector with access to the
- * underlying array.
+ * <p>This class implements a simple char vector with access to the
+ * underlying array.</p>
  *
- * @author Carlos Villegas <cav@uniscope.co.jp>
+ * <p>This work was authored by Carlos Villegas (cav@uniscope.co.jp).</p>
  */
 public class CharVector implements Cloneable, Serializable {
 
+    private static final long serialVersionUID = 4263472982169004048L;
+
     /**
      * Capacity increment size
      */
@@ -45,10 +47,17 @@ public class CharVector implements Cloneable, Serializable {
      */
     private int n;
 
+    /**
+     * Construct char vector instance with default block size.
+     */
     public CharVector() {
         this(DEFAULT_BLOCK_SIZE);
     }
 
+    /**
+     * Construct char vector instance.
+     * @param capacity initial block size
+     */
     public CharVector(int capacity) {
         if (capacity > 0) {
             blockSize = capacity;
@@ -59,12 +68,21 @@ public class CharVector implements Cloneable, Serializable {
         n = 0;
     }
 
+    /**
+     * Construct char vector instance.
+     * @param a char array to use
+     */
     public CharVector(char[] a) {
         blockSize = DEFAULT_BLOCK_SIZE;
         array = a;
         n = a.length;
     }
 
+    /**
+     * Construct char vector instance.
+     * @param a char array to use
+     * @param capacity initial block size
+     */
     public CharVector(char[] a, int capacity) {
         if (capacity > 0) {
             blockSize = capacity;
@@ -76,44 +94,66 @@ public class CharVector implements Cloneable, Serializable {
     }
 
     /**
-     * Reset Vector but don't resize or clear elements
+     * Reset length of vector, but don't clear contents.
      */
     public void clear() {
         n = 0;
     }
 
-    public Object clone() {
-        CharVector cv = new CharVector((char[])array.clone(), blockSize);
-        cv.n = this.n;
+    /** {@inheritDoc} */
+    public Object clone() throws CloneNotSupportedException {
+        CharVector cv = (CharVector) super.clone();
+        cv.array = (char[])array.clone();
         return cv;
     }
 
+    /**
+     * Obtain char vector array.
+     * @return char array
+     */
     public char[] getArray() {
         return array;
     }
 
     /**
-     * return number of items in array
+     * Obtain number of items in array.
+     * @return number of items
      */
     public int length() {
         return n;
     }
 
     /**
-     * returns current capacity of array
+     * Obtain capacity of array.
+     * @return current capacity of array
      */
     public int capacity() {
         return array.length;
     }
 
+    /**
+     * Pet char at index.
+     * @param index the index
+     * @param val a char
+     */
     public void put(int index, char val) {
         array[index] = val;
     }
 
+    /**
+     * Get char at index.
+     * @param index the index
+     * @return a char
+     */
     public char get(int index) {
         return array[index];
     }
 
+    /**
+     * This is to implement memory allocation in the array. Like malloc().
+     * @param size to allocate
+     * @return previous length
+     */
     public int alloc(int size) {
         int index = n;
         int len = array.length;
@@ -126,6 +166,9 @@ public class CharVector implements Cloneable, Serializable {
         return index;
     }
 
+    /**
+     * Trim char vector to current length.
+     */
     public void trimToSize() {
         if (n < array.length) {
             char[] aux = new char[n];
