@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,45 +15,67 @@
  * limitations under the License.
  */
 
-/* $Id: Hyphen.java 627367 2008-02-13 12:03:30Z maxberger $ */
- 
+/* $Id: Hyphen.java 1297284 2012-03-05 23:29:29Z gadams $ */
+
 package org.apache.fop.hyphenation;
 
 import java.io.Serializable;
 
 /**
- * This class represents a hyphen. A 'full' hyphen is made of 3 parts:
+ * <p>This class represents a hyphen. A 'full' hyphen is made of 3 parts:
  * the pre-break text, post-break text and no-break. If no line-break
  * is generated at this position, the no-break text is used, otherwise,
  * pre-break and post-break are used. Typically, pre-break is equal to
  * the hyphen character and the others are empty. However, this general
  * scheme allows support for cases in some languages where words change
  * spelling if they're split across lines, like german's 'backen' which
- * hyphenates 'bak-ken'. BTW, this comes from TeX.
+ * hyphenates 'bak-ken'. BTW, this comes from TeX.</p>
  *
- * @author Carlos Villegas <cav@uniscope.co.jp>
+ * <p>This work was authored by Carlos Villegas (cav@uniscope.co.jp).</p>
  */
 
+/**
+ * Represents a hyphen.
+ */
 public class Hyphen implements Serializable {
-    public String preBreak;
-    public String noBreak;
-    public String postBreak;
 
+    private static final long serialVersionUID = 8989909741110279085L;
+
+    /** pre break string */
+    public String preBreak;                                     // CSOK: VisibilityModifier
+
+    /** no break string */
+    public String noBreak;                                      // CSOK: VisibilityModifier
+
+    /** post break string */
+    public String postBreak;                                    // CSOK: VisibilityModifier
+
+    /**
+     * Construct a hyphen.
+     * @param pre break string
+     * @param no break string
+     * @param post break string
+     */
     Hyphen(String pre, String no, String post) {
         preBreak = pre;
         noBreak = no;
         postBreak = post;
     }
 
+    /**
+     * Construct a hyphen.
+     * @param pre break string
+     */
     Hyphen(String pre) {
         preBreak = pre;
         noBreak = null;
         postBreak = null;
     }
 
+    /** {@inheritDoc} */
     public String toString() {
-        if (noBreak == null 
-                && postBreak == null 
+        if (noBreak == null
+                && postBreak == null
                 && preBreak != null
                 && preBreak.equals("-")) {
             return "-";
