@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ImageConverterSVG2G2D.java 724310 2008-12-08 11:30:11Z jeremias $ */
+/* $Id: ImageConverterSVG2G2D.java 1296526 2012-03-03 00:18:45Z gadams $ */
 
 package org.apache.fop.image.loader.batik;
 
@@ -25,19 +25,23 @@ import java.util.Map;
 
 import org.w3c.dom.Document;
 
+import org.apache.commons.logging.Log;
+import org.apache.commons.logging.LogFactory;
+
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.GVTBuilder;
 import org.apache.batik.bridge.UserAgent;
 import org.apache.batik.dom.svg.SVGDOMImplementation;
 import org.apache.batik.gvt.GraphicsNode;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.image.GraphicsConstants;
 import org.apache.xmlgraphics.image.loader.Image;
 import org.apache.xmlgraphics.image.loader.ImageException;
 import org.apache.xmlgraphics.image.loader.ImageFlavor;
 import org.apache.xmlgraphics.image.loader.ImageInfo;
+import org.apache.xmlgraphics.image.loader.ImageManager;
 import org.apache.xmlgraphics.image.loader.ImageProcessingHints;
+import org.apache.xmlgraphics.image.loader.ImageSessionContext;
 import org.apache.xmlgraphics.image.loader.XMLNamespaceEnabledImageFlavor;
 import org.apache.xmlgraphics.image.loader.impl.AbstractImageConverter;
 import org.apache.xmlgraphics.image.loader.impl.ImageGraphics2D;
@@ -70,14 +74,23 @@ public class ImageConverterSVG2G2D extends AbstractImageConverter {
         }
 
         //Prepare
-        float pxToMillimeter = UnitConv.IN2MM / 72; //default: 72dpi
+        float pxToMillimeter = UnitConv.IN2MM / GraphicsConstants.DEFAULT_DPI;
         Number ptm = (Number)hints.get(ImageProcessingHints.SOURCE_RESOLUTION);
         if (ptm != null) {
             pxToMillimeter = (float)(UnitConv.IN2MM / ptm.doubleValue());
         }
         UserAgent ua = createBatikUserAgent(pxToMillimeter);
         GVTBuilder builder = new GVTBuilder();
-        final BridgeContext ctx = new BridgeContext(ua);
+
+        final ImageManager imageManager = (ImageManager)hints.get(
+                ImageProcessingHints.IMAGE_MANAGER);
+        final ImageSessionContext sessionContext = (ImageSessionContext)hints.get(
+                ImageProcessingHints.IMAGE_SESSION_CONTEXT);
+
+        boolean useEnhancedBridgeContext = (imageManager != null && sessionContext != null);
+        final BridgeContext ctx = (useEnhancedBridgeContext
+                ? new GenericFOPBridgeContext(ua, null, imageManager, sessionContext)
+                : new BridgeContext(ua));
 
         Document doc = svg.getDocument();
         //Cloning SVG DOM as Batik attaches non-thread-safe facilities (like the CSS engine)
@@ -117,9 +130,20 @@ public class ImageConverterSVG2G2D extends AbstractImageConverter {
             /** {@inheritDoc} */
             public void displayMessage(String message) {
                 //TODO Refine and pipe through to caller
-                log.debug(message);
+                log.info(message);
             }
 
+            /** {@inheritDoc} */
+            public void displayError(Exception e) {
+                log.error("Error converting SVG to a Java2D graphic", e);
+            }
+
+            /** {@inheritDoc} */
+            public void displayError(String message) {
+                log.error(message);
+            }
+
+
         };
     }
 
