@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,12 +15,11 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractLayoutManager.java 607032 2007-12-27 10:34:15Z jeremias $ */
+/* $Id: AbstractLayoutManager.java 1334058 2012-05-04 16:52:35Z gadams $ */
 
 package org.apache.fop.layoutmgr;
 
 import java.util.ArrayList;
-import java.util.LinkedList;
 import java.util.List;
 import java.util.ListIterator;
 import java.util.Map;
@@ -28,41 +27,43 @@ import java.util.Map;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.util.QName;
+
 import org.apache.fop.area.Area;
+import org.apache.fop.area.AreaTreeObject;
 import org.apache.fop.area.PageViewport;
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.FONode;
 import org.apache.fop.fo.FObj;
+import org.apache.fop.fo.flow.Marker;
 import org.apache.fop.fo.flow.RetrieveMarker;
 
 /**
  * The base class for most LayoutManagers.
  */
-public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager 
-    implements Constants {
+public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager implements Constants {
 
-    /**
-     * logging instance
-     */
+    /** logging instance */
     private static Log log = LogFactory.getLog(AbstractLayoutManager.class);
 
     /** Parent LayoutManager for this LayoutManager */
-    protected LayoutManager parentLM = null;
+    protected LayoutManager parentLayoutManager;
     /** List of child LayoutManagers */
-    protected List childLMs = null;
+    protected List<LayoutManager> childLMs;
     /** Iterator for child LayoutManagers */
-    protected ListIterator fobjIter = null;
+    protected ListIterator fobjIter;
     /** Marker map for markers related to this LayoutManager */
-    private Map markers = null;
+    private Map<String, Marker> markers;
 
     /** True if this LayoutManager has handled all of its content. */
-    private boolean bFinished = false;
-    
-    /** child LM and child LM iterator during getNextKnuthElement phase */
-    protected LayoutManager curChildLM = null;
-    /** child LM and child LM iterator during getNextKnuthElement phase */
-    protected ListIterator childLMiter = null;
-    
+    private boolean isFinished;
+
+    /** child LM during getNextKnuthElement phase */
+    protected LayoutManager curChildLM;
+
+    /** child LM iterator during getNextKnuthElement phase */
+    protected ListIterator<LayoutManager> childLMiter;
+
     private int lastGeneratedPosition = -1;
     private int smallestPosNumberChecked = Integer.MAX_VALUE;
 
@@ -89,12 +90,12 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
 
     /** {@inheritDoc} */
     public void setParent(LayoutManager lm) {
-        this.parentLM = lm;
+        this.parentLayoutManager = lm;
     }
 
     /** {@inheritDoc} */
     public LayoutManager getParent() {
-        return this.parentLM;
+        return this.parentLayoutManager;
     }
 
     /** {@inheritDoc} */
@@ -113,14 +114,26 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
         if (curChildLM != null && !curChildLM.isFinished()) {
             return curChildLM;
         }
-        while (childLMiter.hasNext()) {
-            curChildLM = (LayoutManager) childLMiter.next();
+        if (childLMiter.hasNext()) {
+            curChildLM = childLMiter.next();
             curChildLM.initialize();
             return curChildLM;
         }
         return null;
     }
 
+    /**
+     * Set currently active child layout manager.
+     * @param childLM the child layout manager
+     */
+    protected void setCurrentChildLM(LayoutManager childLM) {
+        curChildLM = childLM;
+        childLMiter = new LMiter(this);
+        do {
+            curChildLM = childLMiter.next();
+        } while (curChildLM != childLM);
+    }
+
     /**
      * Return indication if getChildLM will return another LM.
      * @return true if another child LM is still available
@@ -135,7 +148,7 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
      * ie. the last one returned represents the end of the content.
      */
     public boolean isFinished() {
-        return bFinished;
+        return isFinished;
     }
 
     /**
@@ -143,30 +156,22 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
      * @param fin the flag value to be set
      */
     public void setFinished(boolean fin) {
-        bFinished = fin;
+        isFinished = fin;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void addAreas(PositionIterator posIter, LayoutContext context) {
     }
 
-    /**
-     * {@inheritDoc} 
-     */
-    public LinkedList getNextKnuthElements(LayoutContext context,
-                                           int alignment) {
+    /** {@inheritDoc} */
+    public List getNextKnuthElements(LayoutContext context, int alignment) {
         log.warn("null implementation of getNextKnuthElements() called!");
         setFinished(true);
         return null;
     }
 
-    /**
-     * {@inheritDoc} 
-     */
-    public LinkedList getChangedKnuthElements(List oldList,
-                                              int alignment) {
+    /** {@inheritDoc} */
+    public List getChangedKnuthElements(List oldList, int alignment) {
         log.warn("null implementation of getChangeKnuthElement() called!");
         return null;
     }
@@ -202,11 +207,11 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
      * @param size the requested number of child LMs
      * @return the list with the preloaded child LMs
      */
-    protected List createChildLMs(int size) {
+    protected List<LayoutManager> createChildLMs(int size) {
         if (fobjIter == null) {
             return null;
         }
-        List newLMs = new ArrayList(size);
+        List<LayoutManager> newLMs = new ArrayList<LayoutManager>(size);
         while (fobjIter.hasNext() && newLMs.size() < size ) {
             Object theobj = fobjIter.next();
             if (theobj instanceof FONode) {
@@ -216,8 +221,8 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
                         (RetrieveMarker) foNode);
                 }
                 if (foNode != null) {
-                    getPSLM().getLayoutManagerMaker().
-                        makeLayoutManagers(foNode, newLMs);
+                    getPSLM().getLayoutManagerMaker()
+                        .makeLayoutManagers(foNode, newLMs);
                 }
             }
         }
@@ -226,67 +231,61 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
 
     /** {@inheritDoc} */
     public PageSequenceLayoutManager getPSLM() {
-        return parentLM.getPSLM();
+        return parentLayoutManager.getPSLM();
     }
-    
+
     /**
      * @see PageSequenceLayoutManager#getCurrentPage()
+     * @return the {@link Page} instance corresponding to the current page
      */
     public Page getCurrentPage() {
         return getPSLM().getCurrentPage();
-    }  
-    
+    }
+
     /** @return the current page viewport */
     public PageViewport getCurrentPV() {
         return getPSLM().getCurrentPage().getPageViewport();
-    }  
-    
-    /**
-     * {@inheritDoc}
-     */
+    }
+
+    /** {@inheritDoc} */
     public boolean createNextChildLMs(int pos) {
-        List newLMs = createChildLMs(pos + 1 - childLMs.size());
+        List<LayoutManager> newLMs = createChildLMs(pos + 1 - childLMs.size());
         addChildLMs(newLMs);
         return pos < childLMs.size();
     }
 
-    /**
-     * {@inheritDoc}
-     */
-    public List getChildLMs() {
+    /** {@inheritDoc} */
+    public List<LayoutManager> getChildLMs() {
         if (childLMs == null) {
-            childLMs = new java.util.ArrayList(10);
+            childLMs = new java.util.ArrayList<LayoutManager>(10);
         }
         return childLMs;
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void addChildLM(LayoutManager lm) {
         if (lm == null) {
             return;
         }
         lm.setParent(this);
         if (childLMs == null) {
-            childLMs = new java.util.ArrayList(10);
+            childLMs = new java.util.ArrayList<LayoutManager>(10);
         }
         childLMs.add(lm);
-        log.trace(this.getClass().getName()
-                  + ": Adding child LM " + lm.getClass().getName());
+        if (log.isTraceEnabled()) {
+            log.trace(this.getClass().getName()
+                    + ": Adding child LM " + lm.getClass().getName());
+        }
     }
 
-    /**
-     * {@inheritDoc}
-     */
+    /** {@inheritDoc} */
     public void addChildLMs(List newLMs) {
         if (newLMs == null || newLMs.size() == 0) {
             return;
         }
-        ListIterator iter = newLMs.listIterator();
+        ListIterator<LayoutManager> iter = newLMs.listIterator();
         while (iter.hasNext()) {
-            LayoutManager lm = (LayoutManager) iter.next();
-            addChildLM(lm);
+            addChildLM(iter.next());
         }
     }
 
@@ -300,11 +299,18 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
         if (pos.getIndex() >= 0) {
             throw new IllegalStateException("Position already got its index");
         }
-        lastGeneratedPosition++;
-        pos.setIndex(lastGeneratedPosition);
+
+        pos.setIndex(++lastGeneratedPosition);
         return pos;
     }
-    
+
+    private void verifyNonNullPosition(Position pos) {
+        if (pos == null || pos.getIndex() < 0) {
+            throw new IllegalArgumentException(
+                    "Only non-null Positions with an index can be checked");
+        }
+    }
+
     /**
      * Indicates whether the given Position is the first area-generating Position of this LM.
      * @param pos the Position (must be one with a position index)
@@ -312,9 +318,7 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
      */
     public boolean isFirst(Position pos) {
         //log.trace("isFirst() smallestPosNumberChecked=" + smallestPosNumberChecked + " " + pos);
-        if (pos == null || pos.getIndex() < 0) {
-            throw new IllegalArgumentException("Only non-null Positions with an index can be checked");
-        }
+        verifyNonNullPosition(pos);
         if (pos.getIndex() == this.smallestPosNumberChecked) {
             return true;
         } else if (pos.getIndex() < this.smallestPosNumberChecked) {
@@ -324,17 +328,14 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
             return false;
         }
     }
-    
+
     /**
      * Indicates whether the given Position is the last area-generating Position of this LM.
      * @param pos the Position (must be one with a position index)
      * @return True if it is the last Position
      */
     public boolean isLast(Position pos) {
-        //log.trace("isLast() lastGenPos=" + lastGeneratedPosition + " " + pos);
-        if (pos == null || pos.getIndex() < 0) {
-            throw new IllegalArgumentException("Only non-null Positions with an index can be checked");
-        }
+        verifyNonNullPosition(pos);
         return (pos.getIndex() == this.lastGeneratedPosition
                 && isFinished());
     }
@@ -343,28 +344,127 @@ public abstract class AbstractLayoutManager extends AbstractBaseLayoutManager
      * Transfers foreign attributes from the formatting object to the area.
      * @param targetArea the area to set the attributes on
      */
-    protected void transferForeignAttributes(Area targetArea) {
-        Map atts = getFObj().getForeignAttributes();
+    protected void transferForeignAttributes(AreaTreeObject targetArea) {
+        Map<QName, String> atts = fobj.getForeignAttributes();
         targetArea.setForeignAttributes(atts);
     }
-    
+
+    /**
+     * Transfers extension attachments from the formatting object to the area.
+     * @param targetArea the area to set the extensions on
+     */
+    protected void transferExtensionAttachments(AreaTreeObject targetArea) {
+        if (fobj.hasExtensionAttachments()) {
+            targetArea.setExtensionAttachments(fobj.getExtensionAttachments());
+        }
+    }
+
+    /**
+     * Transfers extensions (foreign attributes and extension attachments) from
+     * the formatting object to the area.
+     * @param targetArea the area to set the extensions on
+     */
+    protected void transferExtensions(AreaTreeObject targetArea) {
+        transferForeignAttributes(targetArea);
+        transferExtensionAttachments(targetArea);
+    }
+
     /**
      * Registers the FO's markers on the current PageViewport
+     *
+     * @param isStarting    boolean indicating whether the markers qualify as 'starting'
+     * @param isFirst   boolean indicating whether the markers qualify as 'first'
+     * @param isLast    boolean indicating whether the markers qualify as 'last'
      */
     protected void addMarkersToPage(boolean isStarting, boolean isFirst, boolean isLast) {
         if (this.markers != null) {
             getCurrentPV().addMarkers(
-                    this.markers, 
-                    isStarting, 
-                    isFirst, 
+                    this.markers,
+                    isStarting,
+                    isFirst,
                     isLast);
         }
     }
 
-    /** {@inheritDoc} */
-    public String toString() {
-        return (super.toString() + (fobj != null ? "[fobj=" + fobj.toString() + "]" : ""));
+    /**
+     * Registers the FO's id on the current PageViewport
+     */
+    protected void addId() {
+        if (fobj != null) {
+            getPSLM().addIDToPage(fobj.getId());
+        }
     }
-    
-    
+
+    /**
+     * Notifies the {@link PageSequenceLayoutManager} that layout
+     * for this LM has ended.
+     */
+    protected void notifyEndOfLayout() {
+        if (fobj != null) {
+            getPSLM().notifyEndOfLayout(fobj.getId());
+        }
+    }
+
+    /**
+     * Checks to see if the incoming {@link Position}
+     * is the last one for this LM, and if so, calls
+     * {@link #notifyEndOfLayout()} and cleans up.
+     *
+     * @param pos   the {@link Position} to check
+     */
+    protected void checkEndOfLayout(Position pos) {
+        if (pos != null
+            && pos.getLM() == this
+            && this.isLast(pos)) {
+
+            notifyEndOfLayout();
+
+            /* References to the child LMs are no longer needed
+             */
+            childLMs = null;
+            curChildLM = null;
+            childLMiter = null;
+
+            /* markers that qualify have been transferred to the page
+             */
+            markers = null;
+
+            /* References to the FO's children can be released if the
+             * LM is a descendant of the FlowLM. For static-content
+             * the FO may still be needed on following pages.
+             */
+            LayoutManager lm = this.parentLayoutManager;
+            while (!(lm instanceof FlowLayoutManager
+                        || lm instanceof PageSequenceLayoutManager)) {
+                lm = lm.getParent();
+            }
+            if (lm instanceof FlowLayoutManager) {
+                fobj.clearChildNodes();
+                fobjIter = null;
+            }
+        }
+    }
+
+    /** {@inheritDoc} */
+    @Override
+    public String toString() {
+        return (super.toString() + (fobj != null ? "{fobj = " + fobj.toString() + "}" : ""));
+    }
+
+    /** {@inheritDoc} */
+    @Override
+    public void reset() {
+        isFinished = false;
+        curChildLM = null;
+        childLMiter = new LMiter(this);
+        /* Reset all the children LM that have been created so far. */
+        for (LayoutManager childLM : getChildLMs()) {
+            childLM.reset();
+        }
+        if (fobj != null) {
+            markers = fobj.getMarkers();
+        }
+        lastGeneratedPosition = -1;
+    }
+
 }
