@@ -15,12 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: BalancingColumnBreakingAlgorithm.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: BalancingColumnBreakingAlgorithm.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.layoutmgr;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.traits.MinOptMax;
 
 /**
@@ -29,13 +30,36 @@ import org.apache.fop.traits.MinOptMax;
  */
 public class BalancingColumnBreakingAlgorithm extends PageBreakingAlgorithm {
 
-    private Log log = LogFactory.getLog(BalancingColumnBreakingAlgorithm.class);
-    
+    private static final Log LOG = LogFactory.getLog(BalancingColumnBreakingAlgorithm.class);
+
     private int columnCount;
     private int fullLen;
     private int idealPartLen;
-    
-    public BalancingColumnBreakingAlgorithm(LayoutManager topLevelLM,
+
+    /**
+     * Construct a balancing column breaking algorithm.
+     * @param topLevelLM the top level layout manager
+     * @param pageProvider the page provider
+     * @param layoutListener the layout listener
+     * @param alignment     alignment of the paragraph/page. One of
+     *   {@link org.apache.fop.fo.Constants#EN_START},
+     *   {@link org.apache.fop.fo.Constants#EN_JUSTIFY},
+     *   {@link org.apache.fop.fo.Constants#EN_CENTER},
+     *   {@link org.apache.fop.fo.Constants#EN_END}.
+     *   For pages, {@link org.apache.fop.fo.Constants#EN_BEFORE} and
+     *   {@link org.apache.fop.fo.Constants#EN_AFTER}
+     *   are mapped to the corresponding inline properties,
+     *   {@link org.apache.fop.fo.Constants#EN_START} and
+     *   {@link org.apache.fop.fo.Constants#EN_END}.
+     * @param alignmentLast alignment of the paragraph's last line
+     * @param footnoteSeparatorLength length of footnote separator
+     * @param partOverflowRecovery  {@code true} if too long elements should be moved to
+     *                              the next line/part
+     * @param columnCount number of columns
+     * @see PageBreakingAlgorithm
+     */
+    public BalancingColumnBreakingAlgorithm(                     // CSOK: ParameterNumber
+            LayoutManager topLevelLM,
             PageProvider pageProvider,
             PageBreakingLayoutListener layoutListener,
             int alignment, int alignmentLast,
@@ -43,18 +67,18 @@ public class BalancingColumnBreakingAlgorithm extends PageBreakingAlgorithm {
             boolean partOverflowRecovery,
             int columnCount) {
         super(topLevelLM, pageProvider, layoutListener,
-                alignment, alignmentLast, 
+                alignment, alignmentLast,
                 footnoteSeparatorLength, partOverflowRecovery, false, false);
         this.columnCount = columnCount;
         this.considerTooShort = true; //This is important!
     }
-    
+
     /** {@inheritDoc} */
     protected double computeDemerits(KnuthNode activeNode,
             KnuthElement element, int fitnessClass, double r) {
         double dem = super.computeDemerits(activeNode, element, fitnessClass, r);
-        if (log.isTraceEnabled()) {
-            log.trace("original demerit=" + dem + " " + totalWidth 
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("original demerit=" + dem + " " + totalWidth
                     + " line=" + activeNode.line + "/" + columnCount
                     + " pos=" + activeNode.position + "/" + (par.size() - 1));
         }
@@ -70,13 +94,13 @@ public class BalancingColumnBreakingAlgorithm extends PageBreakingAlgorithm {
         if (remParts > 0) {
             avgRestLen = restLen / remParts;
         }
-        if (log.isTraceEnabled()) {
-            log.trace("remaining parts: " + remParts + " rest len: " + restLen 
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("remaining parts: " + remParts + " rest len: " + restLen
                     + " avg=" + avgRestLen);
         }
         double balance = (idealPartLen - partLen) / 1000f;
-        if (log.isTraceEnabled()) {
-            log.trace("balance=" + balance);
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("balance=" + balance);
         }
         double absBalance = Math.abs(balance);
         dem = absBalance;
@@ -92,15 +116,15 @@ public class BalancingColumnBreakingAlgorithm extends PageBreakingAlgorithm {
                 dem = dem * 1.2f;
             }
         }
-        //Step 2: This helps keep the trailing parts shorter than the previous ones 
+        //Step 2: This helps keep the trailing parts shorter than the previous ones
         dem += (avgRestLen) / 1000f;
-        
+
         if (activeNode.line >= columnCount) {
             //We don't want more columns than available
             dem = Double.MAX_VALUE;
         }
-        if (log.isTraceEnabled()) {
-            log.trace("effective dem=" + dem + " " + totalWidth);
+        if (LOG.isTraceEnabled()) {
+            LOG.trace("effective dem=" + dem + " " + totalWidth);
         }
         return dem;
     }
