@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: InlineKnuthSequence.java 825646 2009-10-15 20:43:13Z acumiskey $ */
+/* $Id: InlineKnuthSequence.java 1296483 2012-03-02 21:34:30Z gadams $ */
 
 package org.apache.fop.layoutmgr;
 
@@ -32,6 +32,8 @@ import org.apache.fop.layoutmgr.inline.KnuthInlineBox;
  */
 public class InlineKnuthSequence extends KnuthSequence  {
 
+    private static final long serialVersionUID = 1354774188859946549L;
+
     private boolean isClosed = false;
 
     /**
@@ -68,7 +70,8 @@ public class InlineKnuthSequence extends KnuthSequence  {
             return false;
         }
         // does the first element of the first paragraph add to an existing word?
-        ListElement lastOldElement, firstNewElement;
+        ListElement lastOldElement;
+        ListElement firstNewElement;
         lastOldElement = getLast();
         firstNewElement = sequence.getElement(0);
         if (firstNewElement.isBox() && !((KnuthElement) firstNewElement).isAuxiliary()
@@ -95,6 +98,9 @@ public class InlineKnuthSequence extends KnuthSequence  {
         return this;
     }
 
+    /**
+     * Add letter space.
+     */
     public void addALetterSpace() {
         KnuthBox prevBox = (KnuthBox) getLast();
         if (prevBox.isAuxiliary()
