@@ -15,22 +15,19 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPageNumberCitationLayoutManager.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: AbstractPageNumberCitationLayoutManager.java 1296496 2012-03-02 22:19:46Z gadams $ */
 
 package org.apache.fop.layoutmgr.inline;
 
-import org.apache.fop.area.PageViewport;
 import org.apache.fop.area.Resolvable;
 import org.apache.fop.area.Trait;
 import org.apache.fop.area.inline.InlineArea;
 import org.apache.fop.area.inline.TextArea;
-import org.apache.fop.area.inline.UnresolvedPageNumber;
 import org.apache.fop.fo.flow.AbstractPageNumberCitation;
 import org.apache.fop.fonts.Font;
 import org.apache.fop.fonts.FontInfo;
 import org.apache.fop.fonts.FontTriplet;
 import org.apache.fop.layoutmgr.LayoutContext;
-import org.apache.fop.layoutmgr.LayoutManager;
 import org.apache.fop.layoutmgr.PositionIterator;
 import org.apache.fop.layoutmgr.TraitSetter;
 
@@ -82,10 +79,7 @@ public abstract class AbstractPageNumberCitationLayoutManager extends LeafNodeLa
     }
 
     /** {@inheritDoc} */
-    public InlineArea get(LayoutContext context) {
-        curArea = getPageNumberCitationInlineArea();
-        return curArea;
-    }
+    public abstract InlineArea get(LayoutContext context);
 
     /**
      * {@inheritDoc}
@@ -98,36 +92,6 @@ public abstract class AbstractPageNumberCitationLayoutManager extends LeafNodeLa
         }
     }
 
-    /**
-     * If id can be resolved then simply return a word, otherwise
-     * return a resolvable area
-     *
-     * @param parentLM  the parent LayoutManager
-     * @return a corresponding InlineArea
-     */
-    private InlineArea getPageNumberCitationInlineArea() {
-        PageViewport page = getPSLM().getFirstPVWithID(fobj.getRefId());
-        TextArea text;
-        if (page != null) {
-            String str = page.getPageNumberString();
-            // get page string from parent, build area
-            text = new TextArea();
-            int width = getStringWidth(str);
-            text.addWord(str, 0);
-            text.setIPD(width);
-            resolved = true;
-        } else {
-            resolved = false;
-            text = new UnresolvedPageNumber(fobj.getRefId(), font);
-            String str = "MMM"; // reserve three spaces for page number
-            int width = getStringWidth(str);
-            text.setIPD(width);
-        }
-        updateTextAreaTraits(text);
-
-        return text;
-    }
-
     /**
      * Updates the traits for the generated text area.
      * @param text the text area
@@ -138,7 +102,7 @@ public abstract class AbstractPageNumberCitationLayoutManager extends LeafNodeLa
         text.setBaselineOffset(font.getAscender());
         TraitSetter.addFontTraits(text, font);
         text.addTrait(Trait.COLOR, fobj.getColor());
-        TraitSetter.addPtr(text, fobj.getPtr());   // used for accessibility
+        TraitSetter.addStructureTreeElement(text, fobj.getStructureTreeElement());
         TraitSetter.addTextDecoration(text, fobj.getTextDecoration());
     }
 
@@ -154,5 +118,12 @@ public abstract class AbstractPageNumberCitationLayoutManager extends LeafNodeLa
         return width;
     }
 
+    /**
+     * @return bidi level governing abstract page number citation
+     */
+    protected int getBidiLevel() {
+        return fobj.getBidiLevel();
+    }
+
 }
 
