@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ExternalGraphicLayoutManager.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: ExternalGraphicLayoutManager.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -28,22 +28,22 @@ import org.apache.fop.fo.flow.ExternalGraphic;
  * LayoutManager for the fo:external-graphic formatting object
  */
 public class ExternalGraphicLayoutManager extends AbstractGraphicsLayoutManager {
-    
-    private ExternalGraphic fobj;
 
     /**
-     * Constructor
-     * @param node the fo:external-graphic formatting object that creates the area
+     * Constructor.
+     *
+     * @param node
+     *            the fo:external-graphic formatting object that creates the
+     *            area
      */
     public ExternalGraphicLayoutManager(ExternalGraphic node) {
         super(node);
-        fobj = node;
     }
 
     /** {@inheritDoc} */
     protected Area getChildArea() {
-        return new Image(fobj.getSrc());
+        return new Image(((ExternalGraphic) fobj).getSrc());
     }
-    
+
 }
 
