@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: ImageLayout.java 788278 2009-06-25 07:30:08Z cbowditch $ */
+/* $Id: ImageLayout.java 1326053 2012-04-14 04:02:37Z gadams $ */
 
 package org.apache.fop.layoutmgr.inline;
 
@@ -37,7 +37,7 @@ import org.apache.fop.fo.properties.LengthRangeProperty;
 public class ImageLayout implements Constants {
 
     /** logging instance */
-    protected static Log log = LogFactory.getLog(ImageLayout.class);
+    protected static final Log log = LogFactory.getLog(ImageLayout.class);
 
     //Input
     private GraphicsProperties props;
@@ -228,6 +228,9 @@ public class ImageLayout implements Constants {
             } else if (rat1 > rat2) {
                 adjusted.width = (int)(rat2 * size.width);
                 adjusted.height = effHeight;
+            } else {
+                adjusted.width = effWidth;
+                adjusted.height = effHeight;
             }
         } else {
             adjusted.width = effWidth;
@@ -361,4 +364,4 @@ public class ImageLayout implements Constants {
         return this.clip;
     }
 
-}
\ No newline at end of file
+}
