@@ -15,14 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: TableContentPosition.java 635508 2008-03-10 10:06:37Z jeremias $ */
+/* $Id: TableContentPosition.java 679326 2008-07-24 09:35:34Z vhennebert $ */
 
 package org.apache.fop.layoutmgr.table;
 
 import java.util.List;
 
 import org.apache.fop.fo.flow.table.EffRow;
-import org.apache.fop.fo.flow.table.TableBody;
+import org.apache.fop.fo.flow.table.TablePart;
 import org.apache.fop.layoutmgr.LayoutManager;
 import org.apache.fop.layoutmgr.Position;
 
@@ -64,7 +64,7 @@ class TableContentPosition extends Position {
      * Sets the row corresponding to this position if it starts a new page. In which case,
      * if the delay mechanism is on, this is the delayed row that starts the page, and not
      * the current row being extended.
-     * 
+     *
      * @param newPageRow the row that will start the page if this position is the first
      * one on that page
      */
@@ -80,8 +80,8 @@ class TableContentPosition extends Position {
         return row;
     }
 
-    TableBody getTableBody() {
-        return ((CellPart) cellParts.get(0)).pgu.getTableBody();
+    TablePart getTablePart() {
+        return ((CellPart) cellParts.get(0)).pgu.getTablePart();
     }
 
     /**
