@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TableStepper.java 650550 2008-04-22 15:22:31Z vhennebert $ */
+/* $Id: TableStepper.java 1309636 2012-04-05 02:41:59Z gadams $ */
 
 package org.apache.fop.layoutmgr.table;
 
@@ -25,11 +25,14 @@ import java.util.List;
 
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
+
 import org.apache.fop.fo.Constants;
 import org.apache.fop.fo.flow.table.EffRow;
 import org.apache.fop.fo.flow.table.GridUnit;
 import org.apache.fop.fo.flow.table.PrimaryGridUnit;
 import org.apache.fop.layoutmgr.BreakElement;
+import org.apache.fop.layoutmgr.Keep;
+import org.apache.fop.layoutmgr.KnuthBlockBox;
 import org.apache.fop.layoutmgr.KnuthBox;
 import org.apache.fop.layoutmgr.KnuthGlue;
 import org.apache.fop.layoutmgr.KnuthPenalty;
@@ -78,7 +81,7 @@ public class TableStepper {
     /**
      * Flag used to produce an infinite penalty if the height of the current row is
      * smaller than the first step for that row (may happen with row-spanning cells).
-     * 
+     *
      * @see #considerRowLastStep(int)
      */
     private boolean rowHeightSmallerThanFirstStep;
@@ -102,7 +105,7 @@ public class TableStepper {
 
     /**
      * Initializes the fields of this instance to handle a new row group.
-     * 
+     *
      * @param rows the new row group to handle
      */
     private void setup(EffRow[] rows) {
@@ -119,7 +122,7 @@ public class TableStepper {
     private void calcTotalHeight() {
         totalHeight = 0;
         for (int i = 0; i < rowGroup.length; i++) {
-            totalHeight += rowGroup[i].getHeight().opt;
+            totalHeight += rowGroup[i].getHeight().getOpt();
         }
         if (log.isDebugEnabled()) {
             log.debug("totalHeight=" + totalHeight);
@@ -134,19 +137,19 @@ public class TableStepper {
             PrimaryGridUnit pgu = activeCell.getPrimaryGridUnit();
             for (int i = activeRowIndex + 1; i < pgu.getRowIndex() - rowGroup[0].getIndex()
                     + pgu.getCell().getNumberRowsSpanned(); i++) {
-                remain -= rowGroup[i].getHeight().opt;
+                remain -= rowGroup[i].getHeight().getOpt();
             }
             maxW = Math.max(maxW, remain);
         }
         for (int i = activeRowIndex + 1; i < rowGroup.length; i++) {
-            maxW += rowGroup[i].getHeight().opt;
+            maxW += rowGroup[i].getHeight().getOpt();
         }
         return maxW;
     }
 
     /**
      * Creates ActiveCell instances for cells starting on the row at the given index.
-     * 
+     *
      * @param activeCellList the list that will hold the active cells
      * @param rowIndex the index of the row from which cells must be activated
      */
@@ -195,12 +198,14 @@ public class TableStepper {
                 }
             }
 
+            LinkedList footnoteList = new LinkedList();
             //Put all involved grid units into a list
-            List cellParts = new java.util.ArrayList(columnCount);
+            List cellParts = new java.util.ArrayList(activeCells.size());
             for (Iterator iter = activeCells.iterator(); iter.hasNext();) {
                 ActiveCell activeCell = (ActiveCell) iter.next();
                 CellPart part = activeCell.createCellPart();
                 cellParts.add(part);
+                activeCell.addFootnotes(footnoteList);
             }
 
             //Create elements for step
@@ -213,7 +218,13 @@ public class TableStepper {
                 tcpos.setFlag(TableContentPosition.FIRST_IN_ROWGROUP, true);
             }
             lastTCPos = tcpos;
-            returnList.add(new KnuthBox(boxLen, tcpos, false));
+
+            // TODO TableStepper should remain as footnote-agnostic as possible
+            if (footnoteList.isEmpty()) {
+                returnList.add(new KnuthBox(boxLen, tcpos, false));
+            } else {
+                returnList.add(new KnuthBlockBox(boxLen, footnoteList, tcpos, false));
+            }
 
             int effPenaltyLen = Math.max(0, penaltyOrGlueLen);
             TableHFPenaltyPosition penaltyPos = new TableHFPenaltyPosition(getTableLM());
@@ -228,38 +239,36 @@ public class TableStepper {
                 }
             }
 
-            int p = 0;
-            boolean keepWithNext = false;
+            Keep keep = getTableLM().getKeepTogether();
+            int stepPenalty = 0;
             for (Iterator iter = activeCells.iterator(); iter.hasNext();) {
                 ActiveCell activeCell = (ActiveCell) iter.next();
-                keepWithNext |= activeCell.keepWithNextSignal();
-            }
-            if (keepWithNext || getTableLM().mustKeepTogether()) {
-                p = KnuthPenalty.INFINITE;
+                keep = keep.compare(activeCell.getKeepWithNext());
+                stepPenalty = Math.max(stepPenalty, activeCell.getPenaltyValue());
             }
             if (!rowFinished) {
-                if (rowGroup[activeRowIndex].mustKeepTogether()) {
-                    p = KnuthPenalty.INFINITE;
-                }
+                keep = keep.compare(rowGroup[activeRowIndex].getKeepTogether());
             } else if (activeRowIndex < rowGroup.length - 1) {
-                if (rowGroup[activeRowIndex].mustKeepWithNext()
-                        || rowGroup[activeRowIndex + 1].mustKeepWithPrevious()) {
-                    p = KnuthPenalty.INFINITE;
-                }
+                keep = keep.compare(rowGroup[activeRowIndex].getKeepWithNext());
+                keep = keep.compare(rowGroup[activeRowIndex + 1].getKeepWithPrevious());
                 nextBreakClass = BreakUtil.compareBreakClasses(nextBreakClass,
                         rowGroup[activeRowIndex].getBreakAfter());
                 nextBreakClass = BreakUtil.compareBreakClasses(nextBreakClass,
                         rowGroup[activeRowIndex + 1].getBreakBefore());
             }
-            if (nextBreakClass != Constants.EN_AUTO) {
-                log.trace("Forced break encountered");
-                p = -KnuthPenalty.INFINITE; //Overrides any keeps (see 4.8 in XSL 1.0)
-            }
+            int p = keep.getPenalty();
             if (rowHeightSmallerThanFirstStep) {
                 rowHeightSmallerThanFirstStep = false;
                 p = KnuthPenalty.INFINITE;
             }
-            returnList.add(new BreakElement(penaltyPos, effPenaltyLen, p, nextBreakClass, context));
+            p = Math.max(p, stepPenalty);
+            int breakClass = keep.getContext();
+            if (nextBreakClass != Constants.EN_AUTO) {
+                log.trace("Forced break encountered");
+                p = -KnuthPenalty.INFINITE; //Overrides any keeps (see 4.8 in XSL 1.0)
+                breakClass = nextBreakClass;
+            }
+            returnList.add(new BreakElement(penaltyPos, effPenaltyLen, p, breakClass, context));
             if (penaltyOrGlueLen < 0) {
                 returnList.add(new KnuthGlue(-penaltyOrGlueLen, 0, 0, new Position(null), true));
             }
@@ -274,7 +283,7 @@ public class TableStepper {
 
     /**
      * Returns the first step for the current row group.
-     * 
+     *
      * @return the first step for the current row group
      */
     private int getFirstStep() {
@@ -287,7 +296,7 @@ public class TableStepper {
 
     /**
      * Returns the next break possibility.
-     * 
+     *
      * @return the next step
      */
     private int getNextStep() {
@@ -304,7 +313,7 @@ public class TableStepper {
         if (delayingNextRow) {
             int minStep = computeMinStep();
             if (minStep < 0 || minStep >= rowFirstStep
-                    || minStep > rowGroup[activeRowIndex].getExplicitHeight().max) {
+                    || minStep > rowGroup[activeRowIndex].getExplicitHeight().getMax()) {
                 if (log.isTraceEnabled()) {
                     log.trace("Step = " + minStep);
                 }
@@ -327,7 +336,7 @@ public class TableStepper {
     /**
      * Computes the minimal necessary step to make the next row fit. That is, so such as
      * cell on the next row can contribute some content.
-     * 
+     *
      * @param cells the cells occupying the next row (may include cells starting on
      * previous rows and spanning over this one)
      */
@@ -340,7 +349,7 @@ public class TableStepper {
 
     /**
      * Computes the next minimal step.
-     * 
+     *
      * @return the minimal step from the active cells, &lt; 0 if there is no such step
      */
     private int computeMinStep() {
@@ -364,7 +373,7 @@ public class TableStepper {
     /**
      * Signals the first step to the active cells, to allow them to add more content to
      * the step if possible.
-     * 
+     *
      * @see ActiveCell#signalRowFirstStep(int)
      */
     private void signalRowFirstStep() {
@@ -376,7 +385,7 @@ public class TableStepper {
 
     /**
      * Signals the next selected step to the active cells.
-     *  
+     *
      * @param step the next step
      */
     private void signalNextStep(int step) {
@@ -401,7 +410,7 @@ public class TableStepper {
      * infinite penalty for this step. This will prevent the breaking algorithm from
      * choosing this break, but still allow to create the appropriate TableContentPosition
      * for the cells ending on the current row.</p>
-     * 
+     *
      * @param step the next step
      * @return the updated step if any
      */
@@ -430,7 +439,7 @@ public class TableStepper {
             }
             for (Iterator iter = activeCells.iterator(); iter.hasNext();) {
                 ActiveCell activeCell = (ActiveCell) iter.next();
-                activeCell.endRow(activeRowIndex);                        
+                activeCell.endRow(activeRowIndex);
                 if (!activeCell.endsOnRow(activeRowIndex)) {
                     activeCell.signalRowLastStep(maxStep);
                 }
@@ -451,7 +460,7 @@ public class TableStepper {
      */
     private void prepareNextRow() {
         if (activeRowIndex < rowGroup.length - 1) {
-            previousRowsLength += rowGroup[activeRowIndex].getHeight().opt;
+            previousRowsLength += rowGroup[activeRowIndex].getHeight().getOpt();
             activateCells(nextActiveCells, activeRowIndex + 1);
             if (log.isTraceEnabled()) {
                 log.trace("Computing first step for row " + (activeRowIndex + 2));
