@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AlphaRasterImage.java 696365 2008-09-17 16:41:14Z jeremias $ */
+/* $Id: AlphaRasterImage.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.pdf;
 
@@ -116,7 +116,7 @@ public class AlphaRasterImage implements PDFImage {
         return null;
     }
 
-    /** {@inheritDoc} */
+    /** @return null (unless overridden) */
     public String getSoftMask() {
         return null;
     }
@@ -212,6 +212,10 @@ public class AlphaRasterImage implements PDFImage {
         return null;
     }
 
+    /** {@inheritDoc} */
+    public boolean multipleFiltersAllowed() {
+        return true;
+    }
 }
 
 
