@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFAnnotList.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PDFAnnotList.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
@@ -58,22 +58,19 @@ public class PDFAnnotList extends PDFObject {
      */
     public String toPDFString() {
         StringBuffer p = new StringBuffer(128);
-        p.append(getObjectID());
         p.append("[\n");
         for (int i = 0; i < getCount(); i++) {
             p.append(((PDFObject)links.get(i)).referencePDF());
             p.append("\n");
         }
-        p.append("]\nendobj\n");
+        p.append("]");
         return p.toString();
     }
 
     /*
      * example
-     * 20 0 obj
      * [
      * 19 0 R
      * ]
-     * endobj
      */
 }
