@@ -15,18 +15,21 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFactory.java 830293 2009-10-27 19:07:52Z vhennebert $ */
+/* $Id: PDFFactory.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.pdf;
 
 // Java
+import java.awt.Color;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.io.FileNotFoundException;
 import java.io.IOException;
 import java.io.InputStream;
 import java.net.MalformedURLException;
+import java.text.DecimalFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.BitSet;
 import java.util.Iterator;
 import java.util.List;
@@ -40,6 +43,9 @@ import org.apache.commons.io.output.ByteArrayOutputStream;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 
+import org.apache.xmlgraphics.java2d.color.ColorUtil;
+import org.apache.xmlgraphics.java2d.color.NamedColorSpace;
+
 import org.apache.xmlgraphics.xmp.Metadata;
 
 import org.apache.fop.fonts.CIDFont;
@@ -65,10 +71,15 @@ import org.apache.fop.fonts.type1.PFBParser;
  */
 public class PDFFactory {
 
+    /** Resolution of the User Space coordinate system (72dpi). */
+    public static final int DEFAULT_PDF_RESOLUTION = 72;
+
     private PDFDocument document;
 
     private Log log = LogFactory.getLog(PDFFactory.class);
 
+    private int subsetFontCounter = -1;
+
     /**
      * Creates a new PDFFactory.
      * @param document the parent PDFDocument needed to register the generated
@@ -289,12 +300,13 @@ public class PDFFactory {
      * It should be 0 as this is the constructor for sampled functions.
      * @return the PDF function that was created
      */
-    public PDFFunction makeFunction(int theFunctionType, List theDomain,
-                                    List theRange, List theSize,
-                                    int theBitsPerSample, int theOrder,
-                                    List theEncode, List theDecode,
-                                    StringBuffer theFunctionDataStream,
-                                    List theFilter) {
+    public PDFFunction makeFunction(                             // CSOK: ParameterNumber
+            int theFunctionType, List theDomain,
+            List theRange, List theSize,
+            int theBitsPerSample, int theOrder,
+            List theEncode, List theDecode,
+            StringBuffer theFunctionDataStream,
+            List theFilter) {
         // Type 0 function
         PDFFunction function = new PDFFunction(theFunctionType, theDomain,
                                                theRange, theSize,
@@ -462,12 +474,13 @@ public class PDFFactory {
      * @param theFunction The PDF Function that maps an (x,y) location to a color
      * @return the PDF shading that was created
      */
-    public PDFShading makeShading(PDFResourceContext res, int theShadingType,
-                                  PDFDeviceColorSpace theColorSpace,
-                                  List theBackground, List theBBox,
-                                  boolean theAntiAlias, List theDomain,
-                                  List theMatrix,
-                                  PDFFunction theFunction) {
+    public PDFShading makeShading(                               // CSOK: ParameterNumber
+            PDFResourceContext res, int theShadingType,
+            PDFDeviceColorSpace theColorSpace,
+            List theBackground, List theBBox,
+            boolean theAntiAlias, List theDomain,
+            List theMatrix,
+            PDFFunction theFunction) {
         // make Shading of Type 1
         PDFShading shading = new PDFShading(theShadingType,
                                             theColorSpace, theBackground,
@@ -514,12 +527,13 @@ public class PDFFactory {
      * The default is [false, false]
      * @return the PDF shading that was created
      */
-    public PDFShading makeShading(PDFResourceContext res, int theShadingType,
-                                  PDFDeviceColorSpace theColorSpace,
-                                  List theBackground, List theBBox,
-                                  boolean theAntiAlias, List theCoords,
-                                  List theDomain, PDFFunction theFunction,
-                                  List theExtend) {
+    public PDFShading makeShading(                               // CSOK: ParameterNumber
+            PDFResourceContext res, int theShadingType,
+            PDFDeviceColorSpace theColorSpace,
+            List theBackground, List theBBox,
+            boolean theAntiAlias, List theCoords,
+            List theDomain, PDFFunction theFunction,
+            List theExtend) {
         // make Shading of Type 2 or 3
         PDFShading shading = new PDFShading(theShadingType,
                                             theColorSpace, theBackground,
@@ -567,14 +581,15 @@ public class PDFFactory {
      * @param theFunction the PDFFunction
      * @return the PDF shading that was created
      */
-    public PDFShading makeShading(PDFResourceContext res, int theShadingType,
-                                  PDFDeviceColorSpace theColorSpace,
-                                  List theBackground, List theBBox,
-                                  boolean theAntiAlias,
-                                  int theBitsPerCoordinate,
-                                  int theBitsPerComponent,
-                                  int theBitsPerFlag, List theDecode,
-                                  PDFFunction theFunction) {
+    public PDFShading makeShading(                               // CSOK: ParameterNumber
+            PDFResourceContext res, int theShadingType,
+            PDFDeviceColorSpace theColorSpace,
+            List theBackground, List theBBox,
+            boolean theAntiAlias,
+            int theBitsPerCoordinate,
+            int theBitsPerComponent,
+            int theBitsPerFlag, List theDecode,
+            PDFFunction theFunction) {
         // make Shading of type 4,6 or 7
         PDFShading shading = new PDFShading(theShadingType,
                                             theColorSpace, theBackground,
@@ -622,14 +637,15 @@ public class PDFFactory {
      * @param theFunction The PDFFunction that's mapped on to this shape
      * @return the PDF shading that was created
      */
-    public PDFShading makeShading(PDFResourceContext res, int theShadingType,
-                                  PDFDeviceColorSpace theColorSpace,
-                                  List theBackground, List theBBox,
-                                  boolean theAntiAlias,
-                                  int theBitsPerCoordinate,
-                                  int theBitsPerComponent, List theDecode,
-                                  int theVerticesPerRow,
-                                  PDFFunction theFunction) {
+    public PDFShading makeShading(                               // CSOK: ParameterNumber
+            PDFResourceContext res, int theShadingType,
+            PDFDeviceColorSpace theColorSpace,
+            List theBackground, List theBBox,
+            boolean theAntiAlias,
+            int theBitsPerCoordinate,
+            int theBitsPerComponent, List theDecode,
+            int theVerticesPerRow,
+            PDFFunction theFunction) {
         // make shading of Type 5
         PDFShading shading = new PDFShading(theShadingType,
                                             theColorSpace, theBackground,
@@ -672,11 +688,12 @@ public class PDFFactory {
      * @param thePatternDataStream The stream of pattern data to be tiled.
      * @return the PDF pattern that was created
      */
-    public PDFPattern makePattern(PDFResourceContext res, int thePatternType,    // 1
-                                  PDFResources theResources, int thePaintType, int theTilingType,
-                                  List theBBox, double theXStep,
-                                  double theYStep, List theMatrix,
-                                  List theXUID, StringBuffer thePatternDataStream) {
+    public PDFPattern makePattern(                               // CSOK: ParameterNumber
+            PDFResourceContext res, int thePatternType,    // 1
+            PDFResources theResources, int thePaintType, int theTilingType,
+            List theBBox, double theXStep,
+            double theYStep, List theMatrix,
+            List theXUID, StringBuffer thePatternDataStream) {
         // PDFResources theResources
         PDFPattern pattern = new PDFPattern(theResources, 1,
                                             thePaintType, theTilingType,
@@ -770,23 +787,23 @@ public class PDFFactory {
 
         for (currentPosition = 0; currentPosition < lastPosition;
                 currentPosition++) {    // for every consecutive color pair
-            PDFColor currentColor = (PDFColor)theColors.get(currentPosition);
-            PDFColor nextColor = (PDFColor)theColors.get(currentPosition
-                                 + 1);
-            // colorspace must be consistant
-            if (getDocument().getColorSpace() != currentColor.getColorSpace()) {
-                currentColor.setColorSpace(
-                    getDocument().getColorSpace());
+            Color currentColor = (Color)theColors.get(currentPosition);
+            Color nextColor = (Color)theColors.get(currentPosition + 1);
+
+            // colorspace must be consistent, so we simply convert to sRGB where necessary
+            if (!currentColor.getColorSpace().isCS_sRGB()) {
+                //Convert to sRGB
+                currentColor = ColorUtil.toSRGBColor(currentColor);
+                theColors.set(currentPosition, currentColor);
+            }
+            if (!nextColor.getColorSpace().isCS_sRGB()) {
+                //Convert to sRGB
+                nextColor = ColorUtil.toSRGBColor(nextColor);
+                theColors.set(currentPosition + 1, nextColor);
             }
 
-            if (getDocument().getColorSpace()
-                    != nextColor.getColorSpace()) {
-                nextColor.setColorSpace(
-                    getDocument().getColorSpace());
-            }
-
-            theCzero = currentColor.getVector();
-            theCone = nextColor.getVector();
+            theCzero = toColorVector(currentColor);
+            theCone = toColorVector(nextColor);
 
             myfunc = makeFunction(2, null, null, theCzero, theCone,
                                        interpolation);
@@ -834,6 +851,15 @@ public class PDFFactory {
         return (myPattern);
     }
 
+    private List toColorVector(Color nextColor) {
+        List vector = new java.util.ArrayList();
+        float[] comps = nextColor.getColorComponents(null);
+        for (int i = 0, c = comps.length; i < c; i++) {
+            vector.add(new Double(comps[i]));
+        }
+        return vector;
+    }
+
     /* ============= named destinations and the name dictionary ============ */
 
     /**
@@ -871,7 +897,8 @@ public class PDFFactory {
      */
     public PDFNames makeNames() {
         PDFNames names = new PDFNames();
-        getDocument().registerObject(names);
+        getDocument().assignObjectNumber(names);
+        getDocument().addTrailerObject(names);
         return names;
     }
 
@@ -886,35 +913,6 @@ public class PDFFactory {
         return pageLabels;
     }
 
-    /**
-     * Creates and returns a StructTreeRoot object. Used for accessibility.
-     * @param parentTree the value of the ParenTree entry
-     * @return structure Tree Root element
-     */
-    public PDFStructTreeRoot makeStructTreeRoot(PDFParentTree parentTree) {
-        PDFStructTreeRoot structTreeRoot = new PDFStructTreeRoot(parentTree);
-        getDocument().assignObjectNumber(structTreeRoot);
-        getDocument().addTrailerObject(structTreeRoot);
-        getDocument().getRoot().setStructTreeRoot(structTreeRoot);
-        return structTreeRoot;
-    }
-
-    /**
-     * Creates and returns a StructElem object.
-     *
-     * @param structureType the structure type of the new element (value for the
-     * S entry)
-     * @param parent the parent of the new structure element in the structure
-     * hierarchy
-     * @return the newly created element
-     */
-    public PDFStructElem makeStructureElement(PDFName structureType, PDFObject parent) {
-        PDFStructElem structElem = new PDFStructElem(parent, structureType);
-        getDocument().assignObjectNumber(structElem);
-        getDocument().addTrailerObject(structElem);
-        return structElem;
-    }
-
     /**
      * Make a the head object of the name dictionary (the /Dests object).
      *
@@ -1044,6 +1042,8 @@ public class PDFFactory {
         return link;
     }
 
+    private static final String EMBEDDED_FILE = "embedded-file:";
+
     /**
      * Create/find and return the appropriate external PDFAction according to the target
      *
@@ -1056,32 +1056,91 @@ public class PDFFactory {
     public PDFAction getExternalAction(String target, boolean newWindow) {
         int index;
         String targetLo = target.toLowerCase();
-        // HTTP URL?
-        if (targetLo.startsWith("http://")) {
+        if (target.startsWith(EMBEDDED_FILE)) {
+            // File Attachments (Embedded Files)
+            String filename = target.substring(EMBEDDED_FILE.length());
+            return getActionForEmbeddedFile(filename, newWindow);
+        } else if (targetLo.startsWith("http://")) {
+            // HTTP URL?
+            return new PDFUri(target);
+        } else if (targetLo.startsWith("https://")) {
+            // HTTPS URL?
             return new PDFUri(target);
-        // Non PDF files. Try to /Launch them.
         } else if (targetLo.startsWith("file://")) {
+            // Non PDF files. Try to /Launch them.
             target = target.substring("file://".length());
             return getLaunchAction(target);
-        // Bare PDF file name?
         } else if (targetLo.endsWith(".pdf")) {
+            // Bare PDF file name?
             return getGoToPDFAction(target, null, -1, newWindow);
-        // PDF file + page?
-        } else if ((index = targetLo.indexOf(".pdf#page=")) > 0) {
+        } else if ((index = targetLo.indexOf(".pdf#page=")) > 0) { // CSOK: InnerAssignment
+            // PDF file + page?
             String filename = target.substring(0, index + 4);
             int page = Integer.parseInt(target.substring(index + 10));
             return getGoToPDFAction(filename, null, page, newWindow);
-        // PDF file + destination?
-        } else if ((index = targetLo.indexOf(".pdf#dest=")) > 0) {
+        } else if ((index = targetLo.indexOf(".pdf#dest=")) > 0) { // CSOK: InnerAssignment
+            // PDF file + destination?
             String filename = target.substring(0, index + 4);
             String dest = target.substring(index + 10);
             return getGoToPDFAction(filename, dest, -1, newWindow);
-        // None of the above? Default to URI:
         } else {
+            // None of the above? Default to URI:
             return new PDFUri(target);
         }
     }
 
+    private PDFAction getActionForEmbeddedFile(String filename, boolean newWindow) {
+        PDFNames names = getDocument().getRoot().getNames();
+        if (names == null) {
+            throw new IllegalStateException(
+                    "No Names dictionary present."
+                    + " Cannot create Launch Action for embedded file: " + filename);
+        }
+        PDFNameTreeNode embeddedFiles = names.getEmbeddedFiles();
+        if (embeddedFiles == null) {
+            throw new IllegalStateException(
+                    "No /EmbeddedFiles name tree present."
+                    + " Cannot create Launch Action for embedded file: " + filename);
+        }
+
+        //Find filespec reference for the embedded file
+        filename = PDFText.toPDFString(filename, '_');
+        PDFArray files = embeddedFiles.getNames();
+        PDFReference embeddedFileRef = null;
+        int i = 0;
+        while (i < files.length()) {
+            String name = (String)files.get(i);
+            i++;
+            PDFReference ref = (PDFReference)files.get(i);
+            if (name.equals(filename)) {
+                embeddedFileRef = ref;
+                break;
+            }
+            i++;
+        }
+        if (embeddedFileRef == null) {
+            throw new IllegalStateException(
+                    "No embedded file with name " + filename + " present.");
+        }
+
+        //Finally create the action
+        //PDFLaunch action = new PDFLaunch(embeddedFileRef);
+        //This works with Acrobat 8 but not with Acrobat 9
+
+        //The following two options didn't seem to have any effect.
+        //PDFGoToEmbedded action = new PDFGoToEmbedded(embeddedFileRef, 0, newWindow);
+        //PDFGoToRemote action = new PDFGoToRemote(embeddedFileRef, 0, newWindow);
+
+        //This finally seems to work:
+        StringBuffer scriptBuffer = new StringBuffer();
+        scriptBuffer.append("this.exportDataObject({cName:\"");
+        scriptBuffer.append(filename);
+        scriptBuffer.append("\", nLaunch:2});");
+
+        PDFJavaScriptLaunchAction action = new PDFJavaScriptLaunchAction(scriptBuffer.toString());
+        return action;
+    }
+
     /**
      * Create or find a PDF GoTo with the given page reference string and Y offset,
      * and return its PDF object reference
@@ -1296,10 +1355,15 @@ public class PDFFactory {
         } else {
             FontType fonttype = metrics.getFontType();
 
-            PDFFontDescriptor pdfdesc = makeFontDescriptor(descriptor);
+            String fontPrefix = descriptor.isSubsetEmbedded() ? createSubsetFontPrefix() : "";
+
+            String subsetFontName = fontPrefix + basefont;
+
+            PDFFontDescriptor pdfdesc = makeFontDescriptor(descriptor, fontPrefix);
 
             PDFFont font = null;
-            font = PDFFont.createFont(fontname, fonttype, basefont, null);
+
+            font = PDFFont.createFont(fontname, fonttype, subsetFontName, null);
             getDocument().registerObject(font);
 
             if (fonttype == FontType.TYPE0) {
@@ -1314,8 +1378,7 @@ public class PDFFactory {
                     = new PDFCIDSystemInfo(cidMetrics.getRegistry(),
                                          cidMetrics.getOrdering(),
                                          cidMetrics.getSupplement());
-                PDFCIDFont cidFont
-                    = new PDFCIDFont(basefont,
+                PDFCIDFont cidFont = new PDFCIDFont(subsetFontName,
                                    cidMetrics.getCIDType(),
                                    cidMetrics.getDefaultWidth(),
                                    getSubsetWidths(cidMetrics), sysInfo,
@@ -1469,18 +1532,35 @@ public class PDFFactory {
         return warray;
     }
 
+    private String createSubsetFontPrefix() {
+        subsetFontCounter++;
+        DecimalFormat counterFormat = new DecimalFormat("00000");
+        String counterString = counterFormat.format(subsetFontCounter);
+
+        // Subset prefix as described in chapter 5.5.3 of PDF 1.4
+        StringBuffer sb = new StringBuffer("E");
+
+        for (char c : counterString.toCharArray()) {
+            // translate numbers to uppercase characters
+            sb.append((char) (c + ('A' - '0')));
+        }
+        sb.append("+");
+        return sb.toString();
+    }
+
     /**
      * make a /FontDescriptor object
      *
      * @param desc the font descriptor
+     * @param fontPrefix the String with which to prefix the font name
      * @return the new PDF font descriptor
      */
-    public PDFFontDescriptor makeFontDescriptor(FontDescriptor desc) {
+    private PDFFontDescriptor makeFontDescriptor(FontDescriptor desc, String fontPrefix) {
         PDFFontDescriptor descriptor = null;
 
         if (desc.getFontType() == FontType.TYPE0) {
             // CID Font
-            descriptor = new PDFCIDFontDescriptor(desc.getEmbedFontName(),
+            descriptor = new PDFCIDFontDescriptor(fontPrefix + desc.getEmbedFontName(),
                                             desc.getFontBBox(),
                                             desc.getCapHeight(),
                                             desc.getFlags(),
@@ -1595,8 +1675,8 @@ public class PDFFactory {
                         FontFileReader reader = new FontFileReader(in);
 
                         TTFSubSetFile subset = new TTFSubSetFile();
-                        byte[] subsetFont = subset.readFont(reader,
-                                             mbfont.getTTCName(), mbfont.getUsedGlyphs());
+                        subset.readFont(reader, mbfont.getTTCName(), mbfont.getUsedGlyphs());
+                        byte[] subsetFont = subset.getFontSubset();
                         // Only TrueType CID fonts are supported now
 
                         embeddedFont = new PDFTTFStream(subsetFont.length);
@@ -1713,6 +1793,38 @@ public class PDFFactory {
         return cs;
     }
 
+    /**
+     * Create a new Separation color space.
+     * @param res the resource context (may be null)
+     * @param ncs the named color space to map to a separation color space
+     * @return the newly created Separation color space
+     */
+    public PDFSeparationColorSpace makeSeparationColorSpace(PDFResourceContext res,
+            NamedColorSpace ncs) {
+        String colorName = ncs.getColorName();
+        final Double zero = new Double(0d);
+        final Double one = new Double(1d);
+        List theDomain = Arrays.asList(new Double[] {zero, one});
+        List theRange = Arrays.asList(new Double[] {zero, one, zero, one, zero, one});
+        List theCZero = Arrays.asList(new Double[] {one, one, one});
+        List theCOne = new ArrayList();
+        float[] comps = ncs.getRGBColor().getColorComponents(null);
+        for (int i = 0, c = comps.length; i < c; i++) {
+            theCOne.add(new Double(comps[i]));
+        }
+        PDFFunction tintFunction = makeFunction(2, theDomain, theRange,
+                theCZero, theCOne, 1.0d);
+        PDFSeparationColorSpace cs = new PDFSeparationColorSpace(colorName, tintFunction);
+        getDocument().registerObject(cs);
+        if (res != null) {
+            res.getPDFResources().addColorSpace(cs);
+        } else {
+            getDocument().getResources().addColorSpace(cs);
+        }
+
+        return cs;
+    }
+
     /**
      * Make an Array object (ex. Widths array for a font).
      *
