@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFFont.java 731479 2009-01-05 07:47:02Z jeremias $ */
+/* $Id: PDFFont.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
@@ -174,7 +174,7 @@ public class PDFFont extends PDFDictionary {
     }
 
     /** {@inheritDoc} */
-    protected int output(OutputStream stream) throws IOException {
+    public int output(OutputStream stream) throws IOException {
         validate();
         return super.output(stream);
     }
