@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFImageXObject.java 757712 2009-03-24 10:46:59Z vhennebert $ */
+/* $Id: PDFImageXObject.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.pdf;
 
@@ -61,7 +61,7 @@ public class PDFImageXObject extends PDFXObject {
      * @throws IOException if there is an error writing the data
      * @return the length of the data written
      */
-    protected int output(OutputStream stream) throws IOException {
+    public int output(OutputStream stream) throws IOException {
         int length = super.output(stream);
 
         // let it gc
@@ -137,7 +137,7 @@ public class PDFImageXObject extends PDFXObject {
             put("SMask", ref);
         }
         //Important: do this at the end so previous values can be overwritten.
-        pdfimage.populateXObjectDictionary(this);
+        pdfimage.populateXObjectDictionary(getDictionary());
     }
 
     /** {@inheritDoc} */
@@ -166,4 +166,9 @@ public class PDFImageXObject extends PDFXObject {
         return pdfimage.getFilterHint();
     }
 
+    /** {@inheritDoc} */
+    protected boolean multipleFiltersAllowed() {
+        return pdfimage.multipleFiltersAllowed();
+    }
+
 }
