@@ -15,14 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: PDFPattern.java 815358 2009-09-15 15:07:51Z maxberger $ */
+/* $Id: PDFPattern.java 1305467 2012-03-26 17:39:20Z vhennebert $ */
 
 package org.apache.fop.pdf;
 
-// Java
-import java.util.List;
-import java.io.OutputStream;
 import java.io.IOException;
+import java.io.OutputStream;
+import java.util.List;
 
 /**
  * class representing a PDF Function.
@@ -117,11 +116,12 @@ public class PDFPattern extends PDFPathPaint {
      * @param theXUID Optional vector of Integers that uniquely identify the pattern
      * @param thePatternDataStream The stream of pattern data to be tiled.
      */
-    public PDFPattern(PDFResources theResources, int thePatternType,    // 1
-                      int thePaintType, int theTilingType, List theBBox,
-                      double theXStep, double theYStep,
-                      List theMatrix, List theXUID,
-                      StringBuffer thePatternDataStream) {
+    public PDFPattern(                                           // CSOK: ParameterNumber
+            PDFResources theResources, int thePatternType,    // 1
+            int thePaintType, int theTilingType, List theBBox,
+            double theXStep, double theYStep,
+            List theMatrix, List theXUID,
+            StringBuffer thePatternDataStream) {
         super();
         this.resources = theResources;
         // This next parameter is implicit to all constructors, and is
@@ -210,13 +210,12 @@ public class PDFPattern extends PDFPathPaint {
      * @throws IOException if there is an error writing to the stream
      * @return the PDF string.
      */
-    protected int output(OutputStream stream) throws IOException {
+    public int output(OutputStream stream) throws IOException {
 
         int vectorSize = 0;
         int tempInt = 0;
         byte[] buffer;
         StringBuffer p = new StringBuffer(64);
-        p.append(getObjectID());
         p.append("<< \n/Type /Pattern \n");
 
         if (this.resources != null) {
@@ -323,10 +322,6 @@ public class PDFPattern extends PDFPathPaint {
             length += pdfStream.outputStreamData(encodedStream, stream);
         }
 
-        buffer = encode("\nendobj\n");
-        stream.write(buffer);
-        length += buffer.length;
-
         return length;
     }
 
