@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,13 +15,11 @@
  * limitations under the License.
  */
 
-/* $Id: PDFReference.java 611278 2008-01-11 19:50:53Z jeremias $ */
- 
+/* $Id: PDFReference.java 1228243 2012-01-06 16:03:44Z cbowditch $ */
+
 package org.apache.fop.pdf;
 
-import java.io.IOException;
 import java.io.OutputStream;
-import java.io.Writer;
 import java.lang.ref.Reference;
 import java.lang.ref.SoftReference;
 
@@ -32,20 +30,22 @@ import java.lang.ref.SoftReference;
  * PDF file.
  */
 public class PDFReference implements PDFWritable {
-    
-    private String indirectReference;
-    
-    private Reference objReference;
-    
+
+    private int objectNumber;
+    private int generation;
+
+    private Reference<PDFObject> objReference;
+
     /**
      * Creates a new PDF reference.
      * @param obj the object to be referenced
      */
     public PDFReference(PDFObject obj) {
-        this.indirectReference = obj.referencePDF();
-        this.objReference = new SoftReference(obj);
+        this.objectNumber = obj.getObjectNumber();
+        this.generation = obj.getGeneration();
+        this.objReference = new SoftReference<PDFObject>(obj);
     }
-    
+
     /**
      * Creates a new PDF reference, but without a reference to the original object.
      * @param ref an object reference
@@ -54,7 +54,11 @@ public class PDFReference implements PDFWritable {
         if (ref == null) {
             throw new NullPointerException("ref must not be null");
         }
-        this.indirectReference = ref;
+        String[] parts = ref.split(" ");
+        assert parts.length == 3;
+        this.objectNumber = Integer.parseInt(parts[0]);
+        this.generation = Integer.parseInt(parts[1]);
+        assert "R".equals(parts[2]);
     }
 
     /**
@@ -63,7 +67,7 @@ public class PDFReference implements PDFWritable {
      */
     public PDFObject getObject() {
         if (this.objReference != null) {
-            PDFObject obj = (PDFObject)this.objReference.get();
+            PDFObject obj = this.objReference.get();
             if (obj == null) {
                 this.objReference = null;
             }
@@ -72,15 +76,34 @@ public class PDFReference implements PDFWritable {
             return null;
         }
     }
-    
+
+    /**
+     * Returns the object number.
+     * @return the object number
+     */
+    public int getObjectNumber() {
+        return this.objectNumber;
+    }
+
+    /**
+     * Returns the generation.
+     * @return the generation
+     */
+    public int getGeneration() {
+        return this.generation;
+    }
+
     /** {@inheritDoc} */
+    @Override
     public String toString() {
-        return this.indirectReference;
+        StringBuilder textBuffer = new StringBuilder();
+        outputInline(null, textBuffer);
+        return textBuffer.toString();
     }
-    
+
     /** {@inheritDoc} */
-    public void outputInline(OutputStream out, Writer writer) throws IOException {
-        writer.write(toString());
+    public void outputInline(OutputStream out, StringBuilder textBuffer) {
+        textBuffer.append(getObjectNumber()).append(' ').append(getGeneration()).append(" R");
     }
-    
+
 }
