@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RendererFactory.java 820672 2009-10-01 14:48:27Z jeremias $ */
+/* $Id: RendererFactory.java 1237582 2012-01-30 09:49:22Z mehdi $ */
 
 package org.apache.fop.render;
 
@@ -35,6 +35,7 @@ import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.area.AreaTreeHandler;
 import org.apache.fop.fo.FOEventHandler;
 import org.apache.fop.render.intermediate.AbstractIFDocumentHandlerMaker;
+import org.apache.fop.render.intermediate.EventProducingFilter;
 import org.apache.fop.render.intermediate.IFDocumentHandler;
 import org.apache.fop.render.intermediate.IFDocumentHandlerConfigurator;
 import org.apache.fop.render.intermediate.IFRenderer;
@@ -298,7 +299,6 @@ public class RendererFactory {
         AbstractRendererMaker maker = getRendererMaker(outputFormat);
         if (maker != null) {
             Renderer rend = maker.makeRenderer(userAgent);
-            rend.setUserAgent(userAgent);
             RendererConfigurator configurator = maker.getConfigurator(userAgent);
             if (configurator != null) {
                 configurator.configure(rend);
@@ -310,8 +310,7 @@ public class RendererFactory {
     }
 
     private Renderer createRendererForDocumentHandler(IFDocumentHandler documentHandler) {
-        IFRenderer rend = new IFRenderer();
-        rend.setUserAgent(documentHandler.getContext().getUserAgent());
+        IFRenderer rend = new IFRenderer(documentHandler.getContext().getUserAgent());
         rend.setDocumentHandler(documentHandler);
         return rend;
     }
@@ -389,7 +388,7 @@ public class RendererFactory {
         if (configurator != null) {
             configurator.configure(documentHandler);
         }
-        return documentHandler;
+        return new EventProducingFilter(documentHandler, userAgent);
     }
 
     /**
