@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AFPRendererContextConstants.java 631178 2008-02-26 11:07:20Z jeremias $ */
+/* $Id: AFPRendererContextConstants.java 721430 2008-11-28 11:13:12Z acumiskey $ */
 
 package org.apache.fop.render.afp;
 
@@ -27,9 +27,17 @@ import org.apache.fop.render.RendererContextConstants;
 public interface AFPRendererContextConstants extends RendererContextConstants {
 
     /**
-     * Key for a Boolean value that enables grayscale processing instead of color 
+     * Key for a Boolean value that enables grayscale processing instead of color
      * processing.
      */
     String AFP_GRAYSCALE = "afpGrayscale";
 
+    /** The font information for the AFP renderer. */
+    String AFP_FONT_INFO = "afpFontInfo";
+
+    /** The afp resource manager */
+    String AFP_RESOURCE_MANAGER = "afpResourceManager";
+
+    /** The afp painting state */
+    String AFP_PAINTING_STATE = "afpPaintingState";
 }
