@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PNGRenderer.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PNGRenderer.java 1237610 2012-01-30 11:46:13Z mehdi $ */
 
 package org.apache.fop.render.bitmap;
 
@@ -29,6 +29,7 @@ import org.apache.xmlgraphics.image.writer.ImageWriter;
 import org.apache.xmlgraphics.image.writer.ImageWriterParams;
 import org.apache.xmlgraphics.image.writer.ImageWriterRegistry;
 
+import org.apache.fop.apps.FOUserAgent;
 import org.apache.fop.apps.MimeConstants;
 import org.apache.fop.area.PageViewport;
 import org.apache.fop.render.java2d.Java2DRenderer;
@@ -40,6 +41,13 @@ import org.apache.fop.render.java2d.Java2DRenderer;
  */
 public class PNGRenderer extends Java2DRenderer {
 
+    /**
+     * @param userAgent the user agent that contains configuration details. This cannot be null.
+     */
+    public PNGRenderer(FOUserAgent userAgent) {
+        super(userAgent);
+    }
+
     /** The MIME type for png-Rendering */
     public static final String MIME_TYPE = MimeConstants.MIME_PNG;
 
