@@ -15,7 +15,8 @@
  * limitations under the License.
  */
 
-/* $Id: CustomFontMetricsMapper.java 820689 2009-10-01 15:36:10Z jeremias $ */
+/* $Id: CustomFontMetricsMapper.java 1357883 2012-07-05 20:29:53Z gadams $ */
+
 package org.apache.fop.render.java2d;
 
 import java.awt.Font;
@@ -35,8 +36,8 @@ import org.apache.fop.fonts.Typeface;
 
 /**
  * FontMetricsMapper that delegates most methods to an underlying
- * {@link FontMetrics} instance. This class was designed to allow
- * the underlying {@link java.awt.Font} to be loaded from a
+ * {@link org.apache.fop.fonts.FontMetrics} instance. This class was designed to allow
+ * the underlying {@link Font} to be loaded from a
  * user-configured file not registered in the current graphics environment.
  */
 public class CustomFontMetricsMapper extends Typeface implements FontMetricsMapper {
@@ -159,7 +160,7 @@ public class CustomFontMetricsMapper extends Typeface implements FontMetricsMapp
     }
 
     /** {@inheritDoc} */
-    public final Set getFamilyNames() {
+    public final Set<String> getFamilyNames() {
         return typeface.getFamilyNames();
     }
 
