@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PCLGraphics2D.java 687369 2008-08-20 15:13:56Z acumiskey $ */
+/* $Id: PCLGraphics2D.java 1297232 2012-03-05 21:13:28Z gadams $ */
 
 package org.apache.fop.render.pcl;
 
@@ -127,8 +127,8 @@ public class PCLGraphics2D extends AbstractGraphics2D {
 
     /** {@inheritDoc} */
     public GraphicsConfiguration getDeviceConfiguration() {
-        return GraphicsEnvironment.getLocalGraphicsEnvironment().
-                getDefaultScreenDevice().getDefaultConfiguration();
+        return GraphicsEnvironment.getLocalGraphicsEnvironment()
+                .getDefaultScreenDevice().getDefaultConfiguration();
     }
 
     /**
