@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PCLSVGHandler.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: PCLSVGHandler.java 989178 2010-08-25 15:43:55Z vhennebert $ */
 
 package org.apache.fop.render.pcl;
 
@@ -28,13 +28,12 @@ import org.apache.fop.render.RendererContext;
  * PCL XML handler for SVG. Uses Apache Batik for SVG processing.
  * This handler handles XML for foreign objects when rendering to HP GL/2.
  * It renders SVG to HP GL/2 using the PCLGraphics2D.
- * @see PCLGraphics2DAdapter
  */
 public class PCLSVGHandler extends AbstractGenericSVGHandler {
 
     /** {@inheritDoc} */
     public boolean supportsRenderer(Renderer renderer) {
-        return (renderer instanceof PCLRenderer);
+        return false;
     }
 
     /** {@inheritDoc} */
