@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFRendererConfigurator.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: PDFRendererConfigurator.java 1302518 2012-03-19 15:56:19Z vhennebert $ */
 
 package org.apache.fop.render.pdf;
 
@@ -51,22 +51,12 @@ public class PDFRendererConfigurator extends PrintRendererConfigurator {
     }
 
     /**
-     * Configure the PDF renderer.
-     * Get the configuration to be used for pdf stream filters,
-     * fonts etc.
+     * Throws an UnsupportedOperationException.
      *
-     * @param renderer pdf renderer
-     * @throws FOPException fop exception
+     * @param renderer not used
      */
-    public void configure(Renderer renderer) throws FOPException {
-        Configuration cfg = super.getRendererConfig(renderer);
-        if (cfg != null) {
-            PDFRenderer pdfRenderer = (PDFRenderer)renderer;
-            super.configure(renderer);
-
-            PDFRenderingUtil pdfUtil = pdfRenderer.getPDFUtil();
-            configure(cfg, pdfUtil);
-        }
+    public void configure(Renderer renderer) {
+        throw new UnsupportedOperationException();
     }
 
     private void configure(Configuration cfg, PDFRenderingUtil pdfUtil) throws FOPException {
@@ -80,20 +70,20 @@ public class PDFRendererConfigurator extends PrintRendererConfigurator {
             LogUtil.handleException(log, e, false);
         }
 
-        String s = cfg.getChild(PDFRenderer.PDF_A_MODE, true).getValue(null);
+        String s = cfg.getChild(PDFConfigurationConstants.PDF_A_MODE, true).getValue(null);
         if (s != null) {
             pdfUtil.setAMode(PDFAMode.valueOf(s));
         }
-        s = cfg.getChild(PDFRenderer.PDF_X_MODE, true).getValue(null);
+        s = cfg.getChild(PDFConfigurationConstants.PDF_X_MODE, true).getValue(null);
         if (s != null) {
             pdfUtil.setXMode(PDFXMode.valueOf(s));
         }
             Configuration encryptionParamsConfig
-                = cfg.getChild(PDFRenderer.ENCRYPTION_PARAMS, false);
+                = cfg.getChild(PDFConfigurationConstants.ENCRYPTION_PARAMS, false);
         if (encryptionParamsConfig != null) {
-            PDFEncryptionParams encryptionParams = new PDFEncryptionParams();
+            PDFEncryptionParams encryptionParams = pdfUtil.getEncryptionParams();
             Configuration ownerPasswordConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.OWNER_PASSWORD, false);
+                    PDFConfigurationConstants.OWNER_PASSWORD, false);
             if (ownerPasswordConfig != null) {
                 String ownerPassword = ownerPasswordConfig.getValue(null);
                 if (ownerPassword != null) {
@@ -101,7 +91,7 @@ public class PDFRendererConfigurator extends PrintRendererConfigurator {
                 }
             }
             Configuration userPasswordConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.USER_PASSWORD, false);
+                    PDFConfigurationConstants.USER_PASSWORD, false);
             if (userPasswordConfig != null) {
                 String userPassword = userPasswordConfig.getValue(null);
                 if (userPassword != null) {
@@ -109,37 +99,98 @@ public class PDFRendererConfigurator extends PrintRendererConfigurator {
                 }
             }
             Configuration noPrintConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.NO_PRINT, false);
+                    PDFConfigurationConstants.NO_PRINT, false);
             if (noPrintConfig != null) {
                 encryptionParams.setAllowPrint(false);
             }
             Configuration noCopyContentConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.NO_COPY_CONTENT, false);
+                    PDFConfigurationConstants.NO_COPY_CONTENT, false);
             if (noCopyContentConfig != null) {
                 encryptionParams.setAllowCopyContent(false);
             }
             Configuration noEditContentConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.NO_EDIT_CONTENT, false);
+                    PDFConfigurationConstants.NO_EDIT_CONTENT, false);
             if (noEditContentConfig != null) {
                 encryptionParams.setAllowEditContent(false);
             }
             Configuration noAnnotationsConfig = encryptionParamsConfig.getChild(
-                    PDFRenderer.NO_ANNOTATIONS, false);
+                    PDFConfigurationConstants.NO_ANNOTATIONS, false);
             if (noAnnotationsConfig != null) {
                 encryptionParams.setAllowEditAnnotations(false);
             }
-            pdfUtil.setEncryptionParams(encryptionParams);
+            Configuration noFillInForms = encryptionParamsConfig.getChild(
+                    PDFConfigurationConstants.NO_FILLINFORMS, false);
+            if (noFillInForms != null) {
+                encryptionParams.setAllowFillInForms(false);
+            }
+            Configuration noAccessContentConfig = encryptionParamsConfig.getChild(
+                    PDFConfigurationConstants.NO_ACCESSCONTENT, false);
+            if (noAccessContentConfig != null) {
+                encryptionParams.setAllowAccessContent(false);
+            }
+            Configuration noAssembleDocConfig = encryptionParamsConfig.getChild(
+                    PDFConfigurationConstants.NO_ASSEMBLEDOC, false);
+            if (noAssembleDocConfig != null) {
+                encryptionParams.setAllowAssembleDocument(false);
+            }
+            Configuration noPrintHqConfig = encryptionParamsConfig.getChild(
+                    PDFConfigurationConstants.NO_PRINTHQ, false);
+            if (noPrintHqConfig != null) {
+                encryptionParams.setAllowPrintHq(false);
+            }
+            Configuration encryptionLengthConfig = encryptionParamsConfig.getChild(
+                    PDFConfigurationConstants.ENCRYPTION_LENGTH, false);
+            if (encryptionLengthConfig != null) {
+                int encryptionLength = checkEncryptionLength(
+                        Integer.parseInt(encryptionLengthConfig.getValue(null)));
+                encryptionParams.setEncryptionLengthInBits(encryptionLength);
+            }
         }
-        s = cfg.getChild(PDFRenderer.KEY_OUTPUT_PROFILE, true).getValue(null);
+
+        s = cfg.getChild(PDFConfigurationConstants.KEY_OUTPUT_PROFILE, true).getValue(null);
         if (s != null) {
             pdfUtil.setOutputProfileURI(s);
         }
             Configuration disableColorSpaceConfig = cfg.getChild(
-                    PDFRenderer.KEY_DISABLE_SRGB_COLORSPACE, false);
+                    PDFConfigurationConstants.KEY_DISABLE_SRGB_COLORSPACE, false);
         if (disableColorSpaceConfig != null) {
             pdfUtil.setDisableSRGBColorSpace(
                     disableColorSpaceConfig.getValueAsBoolean(false));
         }
+
+        setPDFDocVersion(cfg, pdfUtil);
+    }
+
+    private int checkEncryptionLength(int encryptionLength) {
+        int correctEncryptionLength = encryptionLength;
+        if (encryptionLength < 40) {
+            correctEncryptionLength = 40;
+        } else if (encryptionLength > 128) {
+            correctEncryptionLength = 128;
+        } else if (encryptionLength % 8 != 0) {
+            correctEncryptionLength = ((int) Math.round(encryptionLength / 8.0f)) * 8;
+        }
+        if (correctEncryptionLength != encryptionLength) {
+            PDFEventProducer.Provider.get(userAgent.getEventBroadcaster())
+                    .incorrectEncryptionLength(this, encryptionLength, correctEncryptionLength);
+        }
+        return correctEncryptionLength;
+    }
+
+    private void setPDFDocVersion(Configuration cfg, PDFRenderingUtil pdfUtil) throws FOPException {
+        Configuration pdfVersion = cfg.getChild(PDFConfigurationConstants.PDF_VERSION, false);
+        if (pdfVersion != null) {
+            String version = pdfVersion.getValue(null);
+            if (version != null && version.length() != 0) {
+                try {
+                    pdfUtil.setPDFVersion(version);
+                } catch (IllegalArgumentException e) {
+                    throw new FOPException(e.getMessage());
+                }
+            } else {
+                throw new FOPException("The PDF version has not been set.");
+            }
+        }
     }
 
     /**
