@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfBookmark.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfBookmark.java 1297284 2012-03-05 23:29:29Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,14 +26,15 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
 
 /**
- * RTF Bookmark.
+ * <p>RTF Bookmark.
  * Create an RTF bookmark as a child of given container with default attributes.
- * This class belongs to the "id" attribute processing.
- * @author <a href="mailto:a.putz@skynamics.com">Andreas Putz</a>
+ * This class belongs to the "id" attribute processing.</p>
+ *
+ * <p>This work was authored by Andreas Putz (a.putz@skynamics.com).</p>
  */
 public class RtfBookmark extends RtfElement {
     //////////////////////////////////////////////////
@@ -59,8 +60,8 @@ public class RtfBookmark extends RtfElement {
      * @param writer a <code>Writer</code> value
      * @param bookmark Name of the bookmark
      */
-    RtfBookmark (RtfContainer parent, Writer w, String bookmark) throws IOException {
-        super (parent, w);
+    RtfBookmark (RtfContainer parent, Writer writer, String bookmark) throws IOException {
+        super (parent, writer);
 
         int now = bookmark.length ();
 
