@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfFile.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfFile.java 1297284 2012-03-05 23:29:29Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,18 +26,20 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import org.apache.fop.render.rtf.rtflib.exceptions.RtfStructureException;
-import java.io.Writer;
-import java.io.IOException;
 import java.io.BufferedWriter;
 import java.io.FileWriter;
+import java.io.IOException;
 import java.io.OutputStreamWriter;
+import java.io.Writer;
+
+import org.apache.fop.render.rtf.rtflib.exceptions.RtfStructureException;
 
 /**
- * Models the top-level structure of an RTF file.
- * @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
- * @author Andreas Putz a.putz@skynamics.com
- * @author Christopher Scott scottc@westinghouse.com
+ * <p>Models the top-level structure of an RTF file.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch),
+ * Andreas Putz (a.putz@skynamics.com), and
+ * Christopher Scott (scottc@westinghouse.com).</p>
  */
 
 public class RtfFile
@@ -80,9 +82,10 @@ extends RtfContainer {
      * If called, must be called before startDocumentArea
      * @return the new RtfHeader
      * @throws IOException for I/O problems
+     * @throws RtfStructureException for illegal RTF structure
      */
     public RtfHeader startHeader()
-    throws IOException {
+    throws IOException, RtfStructureException {
         if (header != null) {
             throw new RtfStructureException("startHeader called more than once");
         }
@@ -128,7 +131,7 @@ extends RtfContainer {
      * @throws RtfStructureException for illegal RTF structure
      */
     public RtfPageArea startPageArea()
-    throws IOException {
+    throws IOException, RtfStructureException {
         if (pageArea != null) {
             throw new RtfStructureException("startPageArea called more than once");
         }
@@ -149,7 +152,7 @@ extends RtfContainer {
      * @throws RtfStructureException for illegal RTF structure
      */
     public RtfPageArea getPageArea()
-    throws IOException {
+    throws IOException, RtfStructureException {
         if (pageArea == null) {
             return startPageArea();
         }
@@ -164,7 +167,7 @@ extends RtfContainer {
      * @throws RtfStructureException for illegal RTF structure
      */
     public RtfDocumentArea startDocumentArea()
-    throws IOException {
+        throws IOException, RtfStructureException {
         if (docArea != null) {
             throw new RtfStructureException("startDocumentArea called more than once");
         }
@@ -187,7 +190,7 @@ extends RtfContainer {
      * @throws RtfStructureException for illegal RTF structure
      */
     public RtfDocumentArea getDocumentArea()
-    throws IOException {
+    throws IOException, RtfStructureException {
         if (docArea == null) {
             return startDocumentArea();
         }
