@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfHyperLink.java 627367 2008-02-13 12:03:30Z maxberger $ */
+/* $Id: RtfHyperLink.java 1311120 2012-04-08 23:48:11Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,15 +26,16 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
+import java.io.Writer;
+
+import org.apache.fop.apps.FOPException;
 
 /**
- * Creates an hyperlink.
- * This class belongs to the <fo:basic-link> tag processing.
- * @author <a href="mailto:a.putz@skynamics.com">Andreas Putz</a>
+ * <p>Creates an hyperlink.
+ * This class belongs to the <fo:basic-link> tag processing.</p>
  *
- * {\field {\*\fldinst HYPERLINK "http://www.test.de"   }{\fldrslt Joe Smith}}
+ * <p>This work was authored by Andreas Putz (a.putz@skynamics.com).</p>
  */
 public class RtfHyperLink
 extends RtfContainer
@@ -57,7 +58,7 @@ implements IRtfTextContainer,
 
 
     /**
-     * Default constructor.
+     * A constructor.
      *
      * @param parent a <code>RtfContainer</code> value
      * @param writer a <code>Writer</code> value
@@ -70,7 +71,15 @@ implements IRtfTextContainer,
         super ((RtfContainer) parent, writer, attr);
         new RtfText (this, writer, str, attr);
     }
-    
+
+    /**
+     * A constructor.
+     *
+     * @param parent a <code>RtfContainer</code> value
+     * @param writer a <code>Writer</code> value
+     * @param attr a <code>RtfAttributes</code> value
+     * @throws IOException for I/O problems
+     */
     public RtfHyperLink (RtfTextrun parent, Writer writer, RtfAttributes attr)
         throws IOException {
         super ((RtfContainer) parent, writer, attr);
@@ -150,12 +159,17 @@ implements IRtfTextContainer,
     /**
      * IRtfTextContainer requirement:
      * @return a copy of our attributes
+     * @throws FOPException if attributes cannot be cloned
      */
-    public RtfAttributes getTextContainerAttributes() {
+    public RtfAttributes getTextContainerAttributes() throws FOPException {
         if (attrib == null) {
             return null;
         }
-        return (RtfAttributes) this.attrib.clone ();
+        try {
+            return (RtfAttributes) this.attrib.clone ();
+        } catch (CloneNotSupportedException e) {
+            throw new FOPException(e);
+        }
     }
 
 
@@ -216,9 +230,12 @@ implements IRtfTextContainer,
     public boolean isEmpty () {
         return false;
     }
-    
-    public RtfTextrun getTextrun()
-    throws IOException {
+
+    /**
+     * @return a text run
+     * @throws IOException if not caught
+     */
+    public RtfTextrun getTextrun() throws IOException {
         RtfTextrun textrun = RtfTextrun.getTextrun(this, writer, null);
         return textrun;
     }
