@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfList.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfList.java 1297404 2012-03-06 10:17:54Z vhennebert $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -26,16 +26,16 @@ package org.apache.fop.render.rtf.rtflib.rtfdoc;
  * the FOP project.
  */
 
-import java.io.Writer;
 import java.io.IOException;
-import java.util.Date;
+import java.io.Writer;
 import java.util.Random;
 
 /**
- * Model of an RTF list, which can contain RTF list items
- * @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
- * @author Christopher Scott, scottc@westinghouse.com
- * @author Peter Herweg, pherweg@web.de
+ * <p>Model of an RTF list, which can contain RTF list items.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch),
+ * Christopher Scott (scottc@westinghouse.com), and
+ * Peter Herweg (pherweg@web.de).</p>
  */
 public class RtfList extends RtfContainer {
     private RtfListItem item;
@@ -116,4 +116,4 @@ public class RtfList extends RtfContainer {
     public boolean getHasTableParent() {
         return hasTableParent;
     }
-}
\ No newline at end of file
+}
