@@ -15,8 +15,9 @@
  * limitations under the License.
  */
 
-/* $Id: RtfStringConverter.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: RtfStringConverter.java 1297284 2012-03-05 23:29:29Z gadams $ */
 
+package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
 /*
  * This file is part of the RTF library of the FOP project, which was originally
@@ -25,20 +26,20 @@
  * the FOP project.
  */
 
-package org.apache.fop.render.rtf.rtflib.rtfdoc;
-
-import java.util.Map;
-import java.util.HashMap;
 import java.io.IOException;
 import java.io.Writer;
+import java.util.HashMap;
+import java.util.Map;
 
-/**  Converts java Strings according to RTF conventions
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
+/**
+ * <p>Converts java Strings according to RTF conventions.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch).</p>
  */
 
-public class RtfStringConverter {
-    private static final RtfStringConverter INSTANCE = new RtfStringConverter();
+public final class RtfStringConverter {
 
+    private static final RtfStringConverter INSTANCE = new RtfStringConverter();
     private static final Map SPECIAL_CHARS;
     private static final Character DBLQUOTE = new Character('\"');
     private static final Character QUOTE = new Character('\'');
