@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: RtfTableRow.java 674304 2008-07-06 15:09:10Z jeremias $ */
+/* $Id: RtfTableRow.java 1311120 2012-04-08 23:48:11Z gadams $ */
 
 package org.apache.fop.render.rtf.rtflib.rtfdoc;
 
@@ -30,10 +30,14 @@ import java.io.IOException;
 import java.io.Writer;
 import java.util.Iterator;
 
-/**  Container for RtfTableCell elements
- *  @author Bertrand Delacretaz bdelacretaz@codeconsult.ch
- *  @author Andreas Putz a.putz@skynamics.com
- *  @author Roberto Marra roberto@link-u.com
+import org.apache.fop.apps.FOPException;
+
+/**
+ * <p>Container for RtfTableCell elements.</p>
+ *
+ * <p>This work was authored by Bertrand Delacretaz (bdelacretaz@codeconsult.ch),
+ * Andreas Putz (a.putz@skynamics.com), and
+ * Roberto Marra (roberto@link-u.com).</p>
  */
 
 public class RtfTableRow extends RtfContainer implements ITableAttributes {
@@ -104,15 +108,20 @@ public class RtfTableRow extends RtfContainer implements ITableAttributes {
      * @param cellWidth width of new cell
      * @return new RtfTableCell
      * @throws IOException for I/O problems
+     * @throws FOPException if attributes cannot be cloned
      */
     public RtfTableCell newTableCellMergedHorizontally (int cellWidth,
-           RtfAttributes attrs) throws IOException {
+           RtfAttributes attrs) throws IOException, FOPException {
         highestCell++;
         // Added by Normand Masse
         // Inherit attributes from base cell for merge
         RtfAttributes wAttributes = null;
         if (attrs != null) {
-            wAttributes = (RtfAttributes)attrs.clone();
+            try {
+                wAttributes = (RtfAttributes)attrs.clone();
+            } catch (CloneNotSupportedException e) {
+                throw new FOPException(e);
+            }
         }
 
         cell = new RtfTableCell(this, writer, cellWidth, wAttributes, highestCell);
