@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TXTRendererMaker.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: TXTRendererMaker.java 1237582 2012-01-30 09:49:22Z mehdi $ */
 
 package org.apache.fop.render.txt;
 
@@ -34,7 +34,7 @@ public class TXTRendererMaker extends AbstractRendererMaker {
 
     /**{@inheritDoc} */
     public Renderer makeRenderer(FOUserAgent userAgent) {
-        return new TXTRenderer();
+        return new TXTRenderer(userAgent);
     }
 
     /**{@inheritDoc} */
