@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TXTStream.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: TXTStream.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.render.txt;
 
@@ -71,10 +71,11 @@ public class TXTStream {
      * @param encoding the encoding, if null, "UTF-8" is chosen as default
      */
     public void setEncoding(String encoding) {
-        if (encoding != null)
+        if (encoding != null) {
             this.encoding = encoding;
-        else
+        } else {
             this.encoding = DEFAULT_ENCODING;
+        }
     }
 }
 
