@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: PDFAElementBridge.java 820689 2009-10-01 15:36:10Z jeremias $ */
+/* $Id: PDFAElementBridge.java 1357883 2012-07-05 20:29:53Z gadams $ */
 
 package org.apache.fop.svg;
 
@@ -26,13 +26,12 @@ import org.w3c.dom.svg.SVGAElement;
 
 import org.apache.batik.bridge.AbstractGraphicsNodeBridge;
 import org.apache.batik.bridge.BridgeContext;
-import org.apache.batik.gvt.CompositeGraphicsNode;
 import org.apache.batik.gvt.GraphicsNode;
 
 /**
- * Bridge class for the &lt;a> element.
+ * <p>Bridge class for the &lt;a> element.</p>
  *
- * @author <a href="mailto:keiron@aftexsw.com">Keiron Liddle</a>
+ * <p>This work was authored by Keiron Liddle (keiron@aftexsw.com).</p>
  */
 public class PDFAElementBridge extends AbstractGraphicsNodeBridge {
     private AffineTransform transform;
@@ -65,7 +64,7 @@ public class PDFAElementBridge extends AbstractGraphicsNodeBridge {
     }
 
     /**
-     * Creates a {@link CompositeGraphicsNode}.
+     * Creates a {@link org.apache.batik.gvt.CompositeGraphicsNode}.
      * @return a new PDFANode
      */
     protected GraphicsNode instantiateGraphicsNode() {
