@@ -15,20 +15,20 @@
  * limitations under the License.
  */
 
-/* $Id: FileCompare.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FileCompare.java 1311120 2012-04-08 23:48:11Z gadams $ */
 
 package org.apache.fop.tools.anttasks;
 
-import java.util.Date;
-import java.util.List;
-import java.util.StringTokenizer;
 import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.PrintWriter;
+import java.text.DateFormat;
+import java.util.Date;
+import java.util.List;
+import java.util.StringTokenizer;
 
 import org.apache.tools.ant.BuildException;
-import java.text.DateFormat;
 
 /**
  * This class is an extension of Ant, a script utility from
@@ -38,7 +38,8 @@ import java.text.DateFormat;
 
 public class FileCompare {
 
-    private String referenceDirectory, testDirectory;
+    private String referenceDirectory;
+    private String testDirectory;
     private String[] filenameList;
     private String filenames;
 
@@ -84,6 +85,7 @@ public class FileCompare {
      * @param f1 first file to compare
      * @param f2 second file to compare
      * @return true if files are same, false otherwise
+     * @throws IOException if not caught
      */
     public static boolean compareFiles(File f1, File f2) throws IOException {
         return (compareFileSize(f1, f2) && compareBytes(f1, f2));
@@ -91,13 +93,15 @@ public class FileCompare {
 
     /**
      * Compare the contents of two files.
-     * @param true if files are same byte-by-byte, false otherwise
+     * @param file1 the first file to compare
+     * @param file2 the second file to compare
+     * @return true if files are same byte-by-byte, false otherwise
      */
     private static boolean compareBytes(File file1, File file2) throws IOException {
-        BufferedInputStream file1Input =
-            new BufferedInputStream(new java.io.FileInputStream(file1));
-        BufferedInputStream file2Input =
-            new BufferedInputStream(new java.io.FileInputStream(file2));
+        BufferedInputStream file1Input
+            = new BufferedInputStream(new java.io.FileInputStream(file1));
+        BufferedInputStream file2Input
+            = new BufferedInputStream(new java.io.FileInputStream(file2));
 
         int charact1 = 0;
         int charact2 = 0;
@@ -116,15 +120,13 @@ public class FileCompare {
 
     /**
      * Does a file size compare of two files
-     * @param true if files are same length, false otherwise
+     * @param oldFile the first file to compare
+     * @param newFile the second file to compare
+     * @return true if files are same length, false otherwise
      */
     private static boolean compareFileSize(File oldFile, File newFile) {
-        if (oldFile.length() != newFile.length()) {
-            return false;
-        } else {
-            return true;
-        }
-    }    // end: compareBytes
+        return oldFile.length() == newFile.length();
+    }
 
     private boolean filesExist(File oldFile, File newFile) {
         if (!oldFile.exists()) {
@@ -163,8 +165,8 @@ public class FileCompare {
         File oldFile;
         File newFile;
         try {
-            PrintWriter results =
-                new PrintWriter(new java.io.FileWriter("results.html"), true);
+            PrintWriter results
+                = new PrintWriter(new java.io.FileWriter("results.html"), true);
             this.writeHeader(results);
             for (int i = 0; i < filenameList.length; i++) {
                 oldFile = new File(referenceDirectory + filenameList[i]);
