@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BasicPDFTranscoderTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: BasicPDFTranscoderTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop;
 
@@ -26,18 +26,9 @@ import org.apache.fop.svg.PDFTranscoder;
  * Basic runtime test for the PDF transcoder. It is used to verify that
  * nothing obvious is broken after compiling.
  */
-public class BasicPDFTranscoderTestCase extends AbstractBasicTranscoderTestCase {
+public class BasicPDFTranscoderTestCase extends AbstractBasicTranscoderTest {
 
-    /**
-     * @see junit.framework.TestCase#TestCase(String)
-     */
-    public BasicPDFTranscoderTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.AbstractBasicTranscoderTestCase#createTranscoder()
-     */
+    @Override
     protected Transcoder createTranscoder() {
         return new PDFTranscoder();
     }
