@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BasicPSTranscoderTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: BasicPSTranscoderTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop;
 
@@ -26,18 +26,9 @@ import org.apache.fop.render.ps.PSTranscoder;
  * Basic runtime test for the PS transcoder. It is used to verify that
  * nothing obvious is broken after compiling.
  */
-public class BasicPSTranscoderTestCase extends AbstractBasicTranscoderTestCase {
+public class BasicPSTranscoderTestCase extends AbstractBasicTranscoderTest {
 
-    /**
-     * @see junit.framework.TestCase#TestCase(String)
-     */
-    public BasicPSTranscoderTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.AbstractBasicTranscoderTestCase#createTranscoder()
-     */
+    @Override
     protected Transcoder createTranscoder() {
         return new PSTranscoder();
     }
