@@ -15,22 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontAttributesMissingTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontAttributesMissingTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-/*
+/**
  * this font is without a metrics-url or an embed-url
  */
-public class FontAttributesMissingTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontAttributesMissingTestCase extends BaseDestructiveUserConfigTest {
 
-    public FontAttributesMissingTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_font_attributes_missing.xconf";
     }
