@@ -15,20 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontBaseBadTestCase.java 752153 2009-03-10 16:16:31Z jeremias $ */
+/* $Id: FontBaseBadTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-/*
+/**
  * This font base does not exist and a relative font path is used.
  */
-public class FontBaseBadTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontBaseBadTestCase extends BaseDestructiveUserConfigTest {
 
-    public FontBaseBadTestCase(String name) {
-        super(name);
-    }
-
-    /** {@inheritDoc} */
+    @Override
     public String getUserConfigFilename() {
         return "test_fontbase_bad.xconf";
     }
