@@ -15,22 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontEmbedUrlBadTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontEmbedUrlBadTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
 /**
  * this font has an embed-url that does not exist on filesystem.
  */
-public class FontEmbedUrlBadTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontEmbedUrlBadTestCase extends BaseDestructiveUserConfigTest {
 
-    public FontEmbedUrlBadTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_font_embedurl_bad.xconf";
     }
