@@ -15,25 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontMetricsUrlBadTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontMetricsUrlBadTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-/*
+/**
  * this font has a metrics-url that does not exist on filesystem
  */
-public class FontMetricsUrlBadTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontMetricsUrlBadTestCase extends BaseDestructiveUserConfigTest {
 
-    /**
-     * @see junit.framework.TestCase#TestCase(String)
-     */
-    public FontMetricsUrlBadTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_font_metricsurl_bad.xconf";
     }
