@@ -15,22 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontMetricsUrlMalformedTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontMetricsUrlMalformedTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-/*
+/**
  * this font has a malformed metrics-url
  */
-public class FontMetricsUrlMalformedTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontMetricsUrlMalformedTestCase extends BaseDestructiveUserConfigTest {
 
-    public FontMetricsUrlMalformedTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_font_metricsurl_malformed.xconf";
     }
