@@ -15,22 +15,16 @@
  * limitations under the License.
  */
 
-/* $Id: FontTripletAttributeMissingTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontTripletAttributeMissingTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-/*
+/**
  * this font has a missing font triplet attribute
  */
-public class FontTripletAttributeMissingTestCase extends BaseDestructiveUserConfigTestCase {
+public class FontTripletAttributeMissingTestCase extends BaseDestructiveUserConfigTest {
 
-    public FontTripletAttributeMissingTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_font_tripletattribute_missing.xconf";
     }
