@@ -15,19 +15,13 @@
  * limitations under the License.
  */
 
-/* $Id: FontsAutoDetectTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: FontsAutoDetectTestCase.java 1198853 2011-11-07 18:18:29Z vhennebert $ */
 
 package org.apache.fop.config;
 
-public class FontsAutoDetectTestCase extends BaseConstructiveUserConfigTestCase {
+public class FontsAutoDetectTestCase extends BaseConstructiveUserConfigTest {
 
-    public FontsAutoDetectTestCase(String name) {
-        super(name);
-    }
-
-    /**
-     * @see org.apache.fop.config.BaseUserConfigTestCase#getUserConfigFilename()
-     */
+    @Override
     public String getUserConfigFilename() {
         return "test_fonts_autodetect.xconf";
     }
