@@ -15,17 +15,20 @@
  * limitations under the License.
  */
 
-/* $Id: URISpecificationTestCase.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: URISpecificationTestCase.java 1178747 2011-10-04 10:09:01Z vhennebert $ */
 
 package org.apache.fop.datatypes;
 
-import junit.framework.TestCase;
+import static org.junit.Assert.assertEquals;
+
+import org.junit.Test;
 
 /**
  * Tests for URISpecification.
  */
-public class URISpecificationTestCase extends TestCase {
+public class URISpecificationTestCase {
 
+    @Test
     public void testGetURL() throws Exception {
         String actual;
 
@@ -42,6 +45,7 @@ public class URISpecificationTestCase extends TestCase {
         assertEquals("http://localhost/test", actual);
     }
 
+    @Test
     public void testEscapeURI() throws Exception {
         String actual;
 
