@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: TestElementMapping.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: TestElementMapping.java 1055034 2011-01-04 13:36:10Z spepping $ */
 
 package org.apache.fop.fotreetest.ext;
 
@@ -40,7 +40,7 @@ public class TestElementMapping extends ElementMapping {
     /** @see org.apache.fop.fo.ElementMapping#initialize() */
     protected void initialize() {
         if (foObjs == null) {
-            foObjs = new java.util.HashMap();
+            foObjs = new java.util.HashMap<String, Maker>();
             foObjs.put("assert", new AssertMaker());
         }
     }
