@@ -15,13 +15,14 @@
  * limitations under the License.
  */
 
-/* $Id: ResultCheck.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: ResultCheck.java 1178747 2011-10-04 10:09:01Z vhennebert $ */
 
 package org.apache.fop.layoutengine;
 
-import org.apache.fop.apps.FormattingResults;
 import org.w3c.dom.Node;
 
+import org.apache.fop.apps.FormattingResults;
+
 /**
  * Simple check that requires a result property to evaluate to the expected value
  */
@@ -30,16 +31,6 @@ public class ResultCheck implements LayoutEngineCheck {
     private String expected;
     private String property;
 
-    /**
-     * Creates a new instance
-     * @param expected expected value
-     * @param property property of which the value needs to be evaluated
-     */
-    public ResultCheck(String expected, String property) {
-        this.expected = expected;
-        this.property = property;
-    }
-
     /**
      * Creates a new instance from a DOM node.
      * @param node DOM node that defines this check
@@ -49,9 +40,7 @@ public class ResultCheck implements LayoutEngineCheck {
         this.property = node.getAttributes().getNamedItem("property").getNodeValue();
     }
 
-    /* (non-Javadoc)
-     * @see LayoutEngineCheck#check(LayoutResult)
-     */
+    /** {@inheritDoc} */
     public void check(LayoutResult result) {
         FormattingResults results = result.getResults();
         String actual;
@@ -68,7 +57,7 @@ public class ResultCheck implements LayoutEngineCheck {
 
     }
 
-    /** @see java.lang.Object#toString() */
+    @Override
     public String toString() {
         return "Property: " + property;
     }
