@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: DigestFilter.java 679326 2008-07-24 09:35:34Z vhennebert $ */
+/* $Id: DigestFilter.java 985571 2010-08-14 19:28:26Z jeremias $ */
 
 package org.apache.fop.util;
 
@@ -43,7 +43,7 @@ import org.xml.sax.helpers.XMLFilterImpl;
 public class DigestFilter extends XMLFilterImpl {
 
     private MessageDigest digest;
-    private byte value[];
+    private byte[] value;
     private boolean isNamespaceAware;
 
     public DigestFilter(String algorithm) throws NoSuchAlgorithmException {
@@ -133,7 +133,7 @@ public class DigestFilter extends XMLFilterImpl {
      */
     public void setFeature(String feature, boolean value)
         throws SAXNotRecognizedException, SAXNotSupportedException {
-        if(feature.equals("http://xml.org/sax/features/namespaces")) {
+        if (feature.equals("http://xml.org/sax/features/namespaces")) {
             isNamespaceAware = value;
         }
         super.setFeature(feature, value);
