@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: AbstractPSPDFBitmapProducer.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: AbstractPSPDFBitmapProducer.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.visual;
 
@@ -122,7 +122,7 @@ public abstract class AbstractPSPDFBitmapProducer extends AbstractBitmapProducer
         try {
             FOUserAgent userAgent = fopFactory.newFOUserAgent();
             userAgent.setTargetResolution(context.getTargetResolution());
-            userAgent.setBaseURL(src.getParentFile().toURL().toString());
+            userAgent.setBaseURL(src.getParentFile().toURI().toURL().toString());
 
             File tempOut = new File(context.getTargetDir(),
                     src.getName() + "." + index + "." + getTargetExtension());
