@@ -15,7 +15,7 @@
  * limitations under the License.
  */
 
-/* $Id: BitmapProducerJava2D.java 746664 2009-02-22 12:40:44Z jeremias $ */
+/* $Id: BitmapProducerJava2D.java 985537 2010-08-14 17:17:00Z jeremias $ */
 
 package org.apache.fop.visual;
 
@@ -70,7 +70,7 @@ public class BitmapProducerJava2D extends AbstractBitmapProducer implements Conf
         try {
             FOUserAgent userAgent = fopFactory.newFOUserAgent();
             userAgent.setTargetResolution(context.getTargetResolution());
-            userAgent.setBaseURL(src.getParentFile().toURL().toString());
+            userAgent.setBaseURL(src.getParentFile().toURI().toURL().toString());
 
             File outputFile = new File(context.getTargetDir(),
                     src.getName() + "." + index + ".java2d.png");
