@@ -15,29 +15,30 @@
  * limitations under the License.
  */
 
-/* $Id: BitmapProducerPS.java 426576 2006-07-28 15:44:37Z jeremias $ */
+/* $Id: BitmapProducerPS.java 746664 2009-02-22 12:40:44Z jeremias $ */
 
 package org.apache.fop.visual;
 
 import org.apache.fop.apps.MimeConstants;
 
 /**
- * BitmapProducer implementation that uses the PSRenderer and an external converter 
+ * BitmapProducer implementation that uses the PSRenderer and an external converter
  * to create bitmaps.
  * <p>
  * See the superclass' javadoc for info on the configuration format.
  */
 public class BitmapProducerPS extends AbstractPSPDFBitmapProducer {
 
-    /** @see org.apache.fop.visual.AbstractPSPDFBitmapProducer#getTargetExtension() */
+    /**
+     * Default constructor.
+     */
+    public BitmapProducerPS() {
+        this.targetFormat = MimeConstants.MIME_POSTSCRIPT;
+    }
+
+    /** {@inheritDoc} */
     protected String getTargetExtension() {
         return "ps";
     }
-    
-    /** @see org.apache.fop.visual.AbstractPSPDFBitmapProducer#getTargetFormat() */
-    protected String getTargetFormat() {
-        return MimeConstants.MIME_POSTSCRIPT;
-    }
-    
 
 }
